/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.util.ChunkedString;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.sun.kvem.midp.pim.ContactImpl;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.lcdui.Canvas;

import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.system.Display;
import net.rim.device.api.ui.Keypad;


/**
 * ContactItemFilter implementation for t9-like support in contact list
 *
 */
public class T9Search extends T9BasicSearch {

    // These values are defined in 4.7 toolchain, but we need to compile
    // with 4.2.1. Therefore the corresponding values were copied here.
    private static final int HW_LAYOUT_TOUCHSCREEN_24 = 1414738484;
    private static final int HW_LAYOUT_TOUCHSCREEN_29 = 1414738226;

    public T9Search() {
        super();
    }

    protected String[] getStringsFromKeyCode(int keyCode) {

        int layout = Keypad.getHardwareLayout();

        Log.debug("Keyboad layout = " + layout);

        switch (layout) {
            case Keypad.HW_LAYOUT_32:
            case Keypad.HW_LAYOUT_LEGACY:
            case Keypad.HW_LAYOUT_PHONE:
                return getStringsFromKeyCodeNormalKeyboard(keyCode);
            case HW_LAYOUT_TOUCHSCREEN_29:
                return getStringsFromKeyCodeStorm(keyCode);
            case Keypad.HW_LAYOUT_REDUCED_24:
                return getStringsFromKeyCodePearl(keyCode);
            case HW_LAYOUT_TOUCHSCREEN_24:
                return getStringsFromKeyCodeStormSureType(keyCode);
            default:
                return getStringsFromKeyCodeNormalKeyboard(keyCode);
        }
    }
    
    // These are the codes for the BlackBerry Pearl
    private static String[] QW = { "q", "w" };
    private static String[] ER = { "e", "r" };
    private static String[] TY = { "t", "y" };
    private static String[] UI = { "u", "i" };
    private static String[] OP = { "o", "p" };
    private static String[] AS = { "a", "s" };
    private static String[] DF = { "d", "f" };
    private static String[] GH = { "g", "h" };
    private static String[] JK = { "j", "k" };
    private static String[] ZX = { "z", "x" };
    private static String[] CV = { "c", "v" };
    private static String[] BN = { "b", "n" };
    private static String[] M  = { "m"      };
    private static String[] L  = { "l"      };

    /**
     * returns the array of strings corresponding to the given keycode. This
     * method assumes the keyboard is the BlackBerry keyboard.
     *
     */
    private  String[] getStringsFromKeyCodeStorm(int keyCode){
        
        if ((char)keyCode == 'a') {
            return ABC;
        } else if ((char)keyCode == 'd') {
            return DEF;
        } else if ((char)keyCode == 'g') {
            return GHI;
        } else if ((char)keyCode == 'j') {
            return JKL;
        } else if ((char)keyCode == 'm') {
            return MNO;
        } else if ((char)keyCode == 'p') {
            return PQRS;
        } else if ((char)keyCode == 't') {
            return TUV;
        } else if ((char)keyCode == 'w') {
            return WXYZ;
        } else if (keyCode>0) {
            String [] s = new String[1];
            s[0] = "" + (char)keyCode;
            return s;
        }
        // if nothing works, we return nothing!
        return new String[0];
    }

    /**
     * returns the array of strings corresponding to the given keycode. This
     * method assumes the keyboard is the BlackBerry keyboard.
     *
     */
    private  String[] getStringsFromKeyCodePearl(int keyCode){
        
        if ((char)keyCode == 'q') {
            return QW;
        } else if ((char)keyCode == '1') {
            return ER;
        } else if ((char)keyCode == '2') {
            return TY;
        } else if ((char)keyCode == '3') {
            return UI;
        } else if ((char)keyCode == 'o') {
            return OP;
        } else if ((char)keyCode == 'a') {
            return AS;
        } else if ((char)keyCode == '4') {
            return DF;
        } else if ((char)keyCode == '5') {
            return GH;
        } else if ((char)keyCode == '6') {
            return JK;
        } else if ((char)keyCode == 'z') {
            return ZX;
        } else if ((char)keyCode == '7') {
            return CV;
        } else if ((char)keyCode == '8') {
            return BN;
        } else if ((char)keyCode == '9') {
            return M;
        } else if ((char)keyCode == 'l') {
            return L;
        } else if (keyCode>0) {
            String [] s = new String[1];
            s[0] = "" + (char)keyCode;
            return s;
        }
        // if nothing works, we return nothing!
        return new String[0];
    }

    private  String[] getStringsFromKeyCodeStormSureType(int keyCode){
        
        if ((char)keyCode == 'q') {
            return QW;
        } else if ((char)keyCode == 'e') {
            return ER;
        } else if ((char)keyCode == 't') {
            return TY;
        } else if ((char)keyCode == 'u') {
            return UI;
        } else if ((char)keyCode == 'o') {
            return OP;
        } else if ((char)keyCode == 'a') {
            return AS;
        } else if ((char)keyCode == 'd') {
            return DF;
        } else if ((char)keyCode == 'g') {
            return GH;
        } else if ((char)keyCode == 'j') {
            return JK;
        } else if ((char)keyCode == 'z') {
            return ZX;
        } else if ((char)keyCode == 'c') {
            return CV;
        } else if ((char)keyCode == 'b') {
            return BN;
        } else if ((char)keyCode == 'm') {
            return M;
        } else if ((char)keyCode == 'l') {
            return L;
        } else if (keyCode>0) {
            String [] s = new String[1];
            s[0] = "" + (char)keyCode;
            return s;
        }
        // if nothing works, we return nothing!
        return new String[0];
    }



}
