/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.utils;

/**
 * Utility class to get some basic device properties.
 * 
 * At the moment the implementation allows some devices to be recognized. The
 * set of recognizable devices it the one needed by the client to customize its
 * behavior.
 */
public class DevInfo {
    private static String S60 = "sw_platform=s60";

    private static String EDITION_5TH = "sw_platform_version=5.0";

    public static final String FAMILY_E71  = "E71";
    public static final String FAMILY_E71X = "E71X";
    
    public static final String UNKNOWN = "Unknown";

    public static String getFamily() {
        String platform = getPlatform();
        if (platform != null && platform.indexOf("e71x") != -1) {
            return FAMILY_E71X;
        } else if (platform != null && platform.indexOf("e71") != -1) {
            return FAMILY_E71;
        }
        return UNKNOWN;
    }

    public static String getManufacturer() {
        String platform = getPlatform();
        if (platform != null && platform.indexOf("nokia") != -1) {
            return "Nokia";
        }
        return UNKNOWN;
    }

    public static String getPlatform() {
        String platform = System.getProperty("microedition.platform");
        if (platform != null) {
            platform = platform.toLowerCase();
        }
        return platform == null ? UNKNOWN : platform;
    }

    public static boolean isTouchscreenSupported() {
        String platform = getPlatform();
        boolean ret = false;
        if (platform.indexOf(S60) > -1 && platform.indexOf(EDITION_5TH) > -1) {
            ret = true;
        }
        return ret;
    }
}


