/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.controller;

import javax.microedition.lcdui.Displayable;
import java.io.InputStream;
import java.io.IOException;

import com.funambol.mailclient.ui.view.FileExplorerScreenListener;
import com.funambol.mailclient.mm.InputStreamBodyPart;

import com.funambol.platform.FileAdapter;
import com.funambol.mail.Message;
import com.funambol.mail.BodyPart;
import com.funambol.mail.Multipart;
import com.funambol.mailclient.ui.view.ComposeMessageScreen;
import com.funambol.util.Log;

/**
 * This class is a listener of the FileExplorerScreen used for attaching files.
 * The purpose of this class is to be notified when an item is selected and the
 * actions necessary to attach a file are undertaken.
 */
public class AttachFileListener implements FileExplorerScreenListener {

    private Displayable screen;
    private Message message;
    private  FileOpener fo;

    public AttachFileListener(Displayable screen, Message message) {
        this.screen  = screen;
        this.message = message;
    }

    public void fileSelected(String filePath, String fileName, String mimeType,
                             int fileSize, String currDir) {

        fo = new FileOpener(filePath, fileName, fileSize, mimeType, currDir);
       
    }

    public void attachFile(){
        fo.start();
    }

    private class FileOpener extends Thread {
        private String name;
        private String path;
        private int fileSize;
        private String mimeType;
        private String currDir;

        public FileOpener(String path, String name, int fileSize, String mimeType, String currDir)
        {
            this.path = path;
            this.name = name;
            this.fileSize = fileSize;
            this.mimeType = mimeType;
            this.currDir  = currDir;
        }

        public void run() {

            String fileName = "file:///" + path + name;
            try {

                // We don't read the content as it would be loaded in memory upfront
                // but we open its input stream so that the user gets prompted here
                // just after selecting the file.
                // We open the file here, as the very first thing, so if the
                // user denies access to the file, then the message is left
                // unchanged and this attachment not attached
                FileAdapter fa = new FileAdapter(fileName, true);
                InputStream is = fa.openInputStream();

                // Update the message
                Multipart multipart = null;
                String textContent = message.getTextContent();

                if(!message.isMultipart()){
                    multipart = new Multipart();

                    // the first bodypart content is the text bodyof the mail
                    BodyPart bodyPart= new BodyPart();
                    bodyPart.addHeader(BodyPart.CONTENT_TRANSFER_ENCODING, "8bit");
                    bodyPart.setContent(textContent);
                    multipart.addBodyPart(bodyPart);
                }else{
                    multipart = (Multipart)message.getContent();
                }

                InputStreamBodyPart bodyPart = new InputStreamBodyPart();
                bodyPart.setDisposition(BodyPart.CD_ATTACHMENT);
                bodyPart.addHeader(BodyPart.CONTENT_TRANSFER_ENCODING, "base64");
                bodyPart.setSize("" + fileSize);
                bodyPart.setFileName(name);
                bodyPart.setAttachUrl(path);
                bodyPart.setContentType(mimeType);

                multipart.addBodyPart(bodyPart);
                message.setContent(multipart);

                UIController.getComposeMessageScreen().setMessage(message);

                UIController.getConfig().setRememberedDir(currDir);
                UIController.saveConfig();

                // Store the input stream into the body part to be used later
                // when the content will be read
                bodyPart.setInputStream(is);
                if (fileSize == -1) {
                    int size = (int)fa.getSize();
                    bodyPart.setSize("" + size);
                }
            } catch (Exception e) {
                Log.error("Cannot open input stream for: " + fileName + "," + e.toString());
                // If we do not set the input stream here, the file will be reopened
                // later
            }

            if (screen instanceof ComposeMessageScreen) {
                UIController.showComposeMessageScreen(null,UIController.getComposeMessageScreen().subject);
            }
        }
    }

}
