/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 */

/*
 * Utils.java
 */

package com.funambol.mailclient.ui.utils;

import com.funambol.mail.Address;
import com.funambol.mail.BodyPart;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mail.Folder;
import com.funambol.mail.Multipart;
import com.funambol.mailclient.syncml.FolderData;
import com.funambol.util.ChunkedString;
import com.funambol.util.Log;
import com.funambol.util.MailDateFormatter;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

public  class UiUtils {
    
    private static String CONTINUE_STRING = "..";
    
    //private constructor to avoid instantiation
    private  UiUtils() {
    }
    
    /**
     * @return the best string to be drawn in the given region, i.e. the complete
     * string if it fits, or partial string with ".." if it does not fit
     *
     * @param text the text to be drawn
     * @g the graphics object
     * @w2 the width of the region
     *
     */
    //Should we put this somewhere else??
    public static String cutString(String text, int w, Font font) {
        if(font.stringWidth(text)<=w) {
            return text;
        } else {
            int width=0;
            for (int i=1;i<text.length();i++) {
                //width = font.substringWidth( text, 0, text.length()-i)+font.stringWidth(CONTINUE_STRING);
                
                width=font.stringWidth(text.substring(0,text.length()-i).trim())+font.stringWidth(CONTINUE_STRING);
                
                if (width<w) {
                    return text.substring(0,text.length()-i).trim()+ CONTINUE_STRING;
                }
            }
        }
        //we should never arrive here!
        return "";
    }
    
    /**
     * Get the attachments (BodyParts) given the message
     * @param m the message where to find the attached file names
     * @return Vector the BodyParts of the give message  
     */
    public static Vector getAttachments(Message m) {

        Multipart multipart = (Multipart) m.getContent();
        int numberOfBodyPart = multipart.getCount();
        
        Vector files = new Vector(numberOfBodyPart);
        
        for (int i = 0; i < numberOfBodyPart; i++) {
            BodyPart bp = multipart.getBodyPart(i);

            String fileName = bp.getFileName();
            if (fileName != null && !fileName.startsWith("Part1.")) {

                files.addElement(bp);
            }
        }
        
        return files;
    }
    
    /**
     * @return the best string to be drawn in the given region, i.e. the complete
     * string if it fits, or partial string with ".." in head if it does not fit
     *
     * @param text the text to be drawn
     * @g the graphics object
     * @w2 the width of the region
     *
     */
    //Should we put this somewhere else??
    public static String cutHeadString(String text, int w, Font font) {
        if(font.stringWidth(text)<=w) {
            return text;
        } else {
            int width=0;
            for (int i=1;i<text.length();i++) {
                width=font.stringWidth(text.substring(0,text.length()-i).trim())+font.stringWidth(CONTINUE_STRING);
                if (width<w) {
                    return CONTINUE_STRING + text.substring(0+i,text.length()).trim() ;
                }
            }
        }
        //we should never arrive here!
        return "";
    }
    
    /**
     * @return true if message is in inbox, false if in another folder or if no
     * folder is set as parent
     * @param message the message
     */
    public static boolean isIncoming(Message message) {
        Folder parent = message.getParent();
        if (parent == null ) {
            return false;
        }
        return parent.getRole().equals(FolderData.INBOX_ROLE);
    }
    /**
     * @return true if message is in outbox or draft
     * @param message the message
     */
    public static boolean isOutgoing(Message message) {
        Folder parent = message.getParent();
        if (parent == null ) {
            return false;
        }
        return parent.getRole().equals(FolderData.OUTBOX_ROLE) ||
               parent.getRole().equals(FolderData.DRAFTS_ROLE);
    }
    
    /**
     * @return true if message is in sent
     * @param message the message
     */
    public static boolean isSent(Message message) {
        Folder parent = message.getParent();
        if (parent == null ) {
            return false;
        }
        return parent.getRole().equals(FolderData.SENT_ROLE);
    }
    
    /**
     *
     * @param inDate the input date
     * @return the given date as a formatted string, if the date is in the
     * same day we are now it return the time, otherwise the day and month
     */
    public static String formatDate(Date inDate) {
        
        String date;
        Calendar now=Calendar.getInstance();
        Calendar sent=Calendar.getInstance();
        sent.setTime(inDate);
        if (now.get(Calendar.DAY_OF_MONTH)==sent.get(Calendar.DAY_OF_MONTH)
        && now.get(Calendar.MONTH)==sent.get(Calendar.MONTH) &&
                now.get(Calendar.YEAR)==sent.get(Calendar.YEAR) )
            date= MailDateFormatter.getFormattedStringFromDate(inDate,
                    MailDateFormatter.FORMAT_HOURS_MINUTES, ":");
        else
            date= MailDateFormatter.getFormattedStringFromDate(inDate,
                    MailDateFormatter.FORMAT_MONTH_DAY, "/");
        return  date;
    }
    
    /**
     * swaps two elements in the given array
     *
     * @param vector the vector containing the elements
     * @param element1 index of the first element
     * @param element2 index of the second element
     *
     */
    public static void swapElements(Vector vector, int element1, int element2) {
        
        Object tmp = vector.elementAt(element1);
        
        vector.setElementAt(vector.elementAt(element2), element1);
        vector.setElementAt(tmp, element2);
        
        
    }
    
    
     /**
     * get the addresses from the given message
     *
     * @return a vector of addresses containint the to, cc and bcc recipients
     * retrieved from the given message
     */
    public  static Vector getMessageAddresses(Message message) {
        if (message == null) {
            Log.info("[getMessageAddresses] message is null, returning empty vector");
            return new Vector(0);
        }
        
        Address[] toAddressList ;
        Address[] ccAddressList ;
        Address[] bccAddressList ;

        int toSize, ccSize, bccSize;
        
        try {
            toAddressList=message.getTo();
            toSize = toAddressList == null ? 0 : toAddressList.length;
            ccAddressList=message.getCc();
            ccSize = ccAddressList == null ? 0 : ccAddressList.length;
            bccAddressList=message.getBcc();
            bccSize = bccAddressList == null ? 0 : bccAddressList.length;
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("getMessageAddresses: "+ex.toString());
            return new Vector(0);
        }
        
        int size = toSize + ccSize + bccSize;
        Vector messageAddresses=new Vector(size);
        
        for (int i=0; i < toSize; i++) {
            messageAddresses.addElement(toAddressList[i]);
        }
        for (int i=0; i<ccSize; i++) {
            messageAddresses.addElement(ccAddressList[i]);
        }
        for (int i=0; i<bccSize; i++) {
            messageAddresses.addElement(bccAddressList[i]);
        }
        return messageAddresses;
    }

    /**
     * Creates a String[] that fits on a region given the font and the region 
     * width. Doesn't use particular rules to separate strings words.
     * @param text is the text to be wrappe don more lines
     * @param w1 the region width after the custom item label
     * @param w2 the region width for the text lines after the first
     * @param f the font to be use when the text is drawn
     * @return String[] the array of lines that represents the wrapper text
     */
    public static String[] getCustomItemWrappedText(String text, int w1, int w2, Font f) {
        char[] cText = text.toCharArray();
        
        //the text total width
        int totalStringLength = f.stringWidth(text);

        ChunkedString csText = new ChunkedString(text);
        
        //The text string char counter used to 
        int csEnd = 0;
        int csStart = 0;
        
        //Gets the first line
        while (f.charsWidth(cText, csStart, csEnd - csStart) < w1) {
            csEnd++;
        }

        //Resize the first line
        while (f.charsWidth(cText, csStart, csEnd - csStart) > w1) {
            csEnd--;
        }

        //this string represents the first line
        String firstLine = csText.substring(csStart, csEnd).toString();

        //this string represents the text without the line
        String textWithoutFirstLine = csText.substring(csEnd).toString();
        
        //the total number of lines of w2 width
        int textLinesNumber = (totalStringLength - w1)/w2;
        
        //the width of the last line
        int lastLineTextWidth = (totalStringLength - w1)%w2;

        //if the last line width is greater than 0, the total lines number
        //must be increased in order to add the last line
        if (lastLineTextWidth > 0) {
            textLinesNumber+=2;
        } else {
            textLinesNumber++;
        }

        String[] strings = new String[textLinesNumber];
        
        strings[0] = firstLine;
        
        csStart = csEnd;
        
        for (int i = 1; i < textLinesNumber; i++) {
        
            if (i == textLinesNumber-1) {
                strings[i] = csText.substring(csStart).toString();
                break;
            }
            
            while (f.charsWidth(cText, csStart, csEnd - csStart) < w2) {
                csEnd++;
            }
            
            while (f.charsWidth(cText, csStart, csEnd - csStart) > w2) {
                csEnd--;
            }
            
            strings[i] = csText.substring(csStart, csEnd).toString();

            csStart = csEnd;
            
        }
        
        return strings;
    }
    
    /**
     * Creates a String[] that fits on a region given the font and the region 
     * width. Doesn't use particular rules to separate strings words.
     * @param text is the text to be wrappe don more lines
     * @param maxLines the max lines of text after the label
     * @param w1 the region width after the custom item label
     * @param w2 the region width for the text lines after the first
     * @param f the font to be use when the text is drawn
     * @return String[] the array of lines that represents the wrapper text
     */
    public static String[] getCustomItemWrappedTextGivenRowNumber(String text, int maxLines, int w1, int w2, Font f) {
        char[] cText = text.toCharArray();
        
        //the text total width
        int totalStringLength = f.stringWidth(text);

        ChunkedString csText = new ChunkedString(text);
        
        //The text string char counter used to 
        int csEnd = 0;
        int csStart = 0;
        
        String firstLine = null;
        
        if (maxLines > 1) {
            //Gets the first line
            while (f.charsWidth(cText, csStart, csEnd - csStart) < w1) {
                csEnd++;
            }

            //Resize the first line
            while (f.charsWidth(cText, csStart, csEnd - csStart) > w1) {
                csEnd--;
            }

            //this string represents the first line
            firstLine = csText.substring(csStart, csEnd).toString();
        } else {
            firstLine = cutString(text, w1, f);
            String[] strings = new String[1];
            strings[0] = firstLine;
            return strings;
        }
        
        //this string represents the text without the line
        String textWithoutFirstLine = csText.substring(csEnd).toString();
        
        //the total number of lines of w2 width
        int textLinesNumber = (totalStringLength - w1)/w2;
        
        //the width of the last line
        int lastLineTextWidth = (totalStringLength - w1)%w2;

        //if the last line width is greater than 0, the total lines number
        //must be increased in order to add the last line
        if (lastLineTextWidth > 0) {
            textLinesNumber+=2;
        } else {
            textLinesNumber++;
        }

        boolean isExceedingLines = false;
        
        if (textLinesNumber > maxLines) {
            textLinesNumber = maxLines;
            isExceedingLines = true;
        }
        
        String[] strings = new String[textLinesNumber];
        
        strings[0] = firstLine;
        
        csStart = csEnd;
        
        for (int i = 1; i < textLinesNumber; i++) {
        
            if (i == textLinesNumber-1) {
                if (isExceedingLines) {
                    strings[i] = cutString(csText.substring(csStart).toString(), w2, f);
                } else {
                    strings[i] = csText.substring(csStart).toString();
                }
                break;
            }
            
            while (f.charsWidth(cText, csStart, csEnd - csStart) < w2) {
                csEnd++;
            }
            
            while (f.charsWidth(cText, csStart, csEnd - csStart) > w2) {
                csEnd--;
            }
            
            strings[i] = csText.substring(csStart, csEnd).toString();

            csStart = csEnd;
            
        }

        return strings;
    }
        
    /**
     * this method creates a vector of strings that can fit
     * on a single line of screenWidth pixel with given font.
     */
    public static String[] getStringArray(String text, int screenWidth, Font font) {
        // Log.debug(this, "GetStingArray() " + text.substring(0,10));
        
        Vector lines = new Vector();
        String [] ret;
        
        ChunkedString csLine = new ChunkedString(text);
        ChunkedString csWord;
        String[] lineSep = {"\r\n", "\n\r", "\r", "\n" };
        String[] wordSep = {" ", "\t"};
        
        
        //ChunkedString line;
        StringBuffer wordStringBuffer = new StringBuffer();
        String word;
        StringBuffer myLine=new StringBuffer();
        int widthLeft;
        while ( (csWord=csLine.getNextChunk(lineSep)) != null ) {
            //csWord = new ChunkedString(line);
            widthLeft = screenWidth;
            myLine.setLength(0);
            
            // get the words
            while ( (word = csWord.getNextString(wordSep)) != null ) {
                wordStringBuffer = new StringBuffer(word);
                
                if ( myLine.length() != 0 ) {
                    //if this is not the first word of the line we add a space
                    wordStringBuffer.insert(0," ");
                }
                int width = font.stringWidth(wordStringBuffer.toString());
                //if it fits inside the line
                if (width < widthLeft) {
                    //we append the word at the line and update widthleft
                    myLine.append(wordStringBuffer.toString());
                    widthLeft -=width;
                } else {
                    //if we've a space we can cut it out since this word
                    // will go to the next line
                    if (wordStringBuffer.charAt(0) == ' ') {
                        wordStringBuffer.deleteCharAt(0);
                    }
                    width = font.stringWidth(wordStringBuffer.toString());
                    //if can go on a new line
                    if ( width < screenWidth ) {
                        //we put it on a new line
                        lines.addElement(myLine.toString());
                        myLine.setLength(0);
                        myLine.append(wordStringBuffer.toString());
                        widthLeft = screenWidth - width;
                        continue;
                    } else {
                        int start = 0;
                        int end = 0;
                        int length = wordStringBuffer.length();
                        //String aString = myLine.toString();
                        while (start < length) {
                            // check length of the substring, adding a single char each time
                            for (int w = 0; w < widthLeft && end < length; end++) {
                                w = font.substringWidth( wordStringBuffer.toString(), start, (end - start) +1 );
                            }
                            myLine.append(wordStringBuffer.toString().substring(start,end));
                            if (end < length) {
                                lines.addElement(myLine.toString());
                                myLine.setLength(0);
                                widthLeft = screenWidth;
                            } else {
                                widthLeft = screenWidth - font.stringWidth(myLine.toString());
                            }
                            start = end;
                        }
                    }
                }
            }
            lines.addElement(myLine.toString());
        }
        ret = new String[lines.size()];
        //        Log.debug(this, "created vector, copying it");
        lines.copyInto(ret);
        lines=null;
        //       Log.debug(this, "returning string array");
        return ret;
    }
    
    /**
     * get the middle color given two colors
     * 
     * @param aStartColor the starting color
     * @param aEndColor the ending color
     * @return the middle color (sort of, no mapping in hsb is performed, so this
     *  is not really the middle color...)
     */
     public static int getMiddleColor(int aStartColor, int aEndColor) {

        int r1 = aStartColor>>16;
        int g1 = aStartColor>>8;
        g1 = g1 & 0x0000FF;
        int b1 = aStartColor;
        b1 = b1 & 0x0000FF;

        int r2 = aEndColor>>16;
        int g2 = aEndColor>>8;
        g2 = g2 & 0x0000FF;
        int b2 = aEndColor;
        b2 = b2 & 0x0000FF;

        int r = r1 - (r1 - r2) /2 ;
        r = r << 16;
        int g = g1 - (g1 - g2) /2 ;
        g = g << 8;
        int b = b1 - (b1 - b2) /2 ;

        return r | g | b;
    }
     
     public static int[] getGradient(int aStartColor, int aEndColor, int aLength){
        int r1 = aStartColor>>16;
        int g1 = aStartColor>>8;
        g1 = g1 & 0x0000FF;
        int b1 = aStartColor;
        b1 = b1 & 0x0000FF;
        
        int r2 = aEndColor>>16;
        int g2 = aEndColor>>8;
        g2 = g2 & 0x0000FF;
        int b2 = aEndColor;
        b2 = b2 & 0x0000FF;
        
        int res_r = 0;
        int res_g = 0;
        int res_b = 0;
        
        int gradient[] = new int[aLength];
        
        int counter = 0;
        int dif;
        while(counter < aLength){
            dif = (aLength-counter);
            res_r = (r1 * dif + r2*counter)/aLength ;
            res_g = (g1 * dif + g2*counter)/aLength ;
            res_b = (b1 * dif + b2*counter)/aLength ;
            
            res_r = res_r <<16;
            res_g = res_g <<8;
            
            gradient[counter] = (res_r | res_g | res_b);
            
            counter++;
        }
        return gradient;
    }
}
