/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.view;

import java.util.Vector;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;

/**
 * This form shows the log content collected by using the
 * application
 */
public class AlertDetailsScreen extends CustomMenuCanvas implements CommandListener {
  
    private static final int X_BORDER = 5;
    private String lines[];
    private String message;
    private int    firstVisible = 0;
    private Font   messageFont  = null;
    private int    lastWidth    = 0;
    private int    lastHeight   = 0;

    /** Creates a new instance of LogViewerForm */
    public AlertDetailsScreen(String message) {
        super();

        Drawer drawer = UIController.getDrawer();
        GraphicalTheme theme = drawer.getGraphicalTheme();
        messageFont = theme.getDefaultFont();

        this.message = message;
        addCommand(UIController.lowLevelBackCommand);
    }

    private void initMessage(int width) {
        lines = UiUtils.getStringArray(message, width, messageFont);
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == UIController.lowLevelBackCommand) {
            UIController.showBackScreen();
        }
    }

    protected void handleKeyPressed(int keyCode) {

        int dir = getGameAction(keyCode);
        if (isDownKey(dir, keyCode)) {
            scrollDown(1);
        } else if (isUpKey(dir, keyCode)) {
            scrollUp(1);
        } else if (isFireKey(dir, keyCode)) {
//#ifdef isBlackberry
//#         handleFireKeyBB();
//#endif
        }
    }

    private void handleFireKeyBB() {
        // Show the menu when the user presses the fire button
        openMainMenu();
    }

    /**
     * This is equivalent to keyRepeated after filtering out events for the menu.
     */
    protected void handleKeyRepeated(int keyCode) {
        handleKeyPressed(keyCode);
    }

    private void scrollDown(int amount) {

        int fontHeight = messageFont.getHeight();
        int numDisplayable = getHeight() / fontHeight;

        if (lines.length - firstVisible > numDisplayable) {
            if (firstVisible + amount < lines.length) {
                firstVisible += amount;
            } else {
                firstVisible = lines.length - 1;
            }
            repaint();
        }
    }

    private void scrollUp(int amount) {
        if (firstVisible - amount >= 0) {
            firstVisible -= amount;
        } else {
            firstVisible = 0;
        }
        repaint();
    }

    /**
     * This is equivalent to paint
     */
    protected void paintContent(Graphics graphics) {

        if (getWidth() != lastWidth || getHeight() != lastHeight) {
            initMessage(getWidth() - 2 * X_BORDER);
        }

        graphics.setColor(0xFFFFFF);
        graphics.fillRect(0,0, getWidth(), getHeight());
        int fontHeight = messageFont.getHeight();
        graphics.setColor(0x0);
        graphics.setFont(messageFont);
        for (int i=firstVisible;i<lines.length;++i) {
            String line = lines[i];
            int y = (i+1) * fontHeight;
            graphics.drawString(line, X_BORDER, y, Graphics.BOTTOM | Graphics.LEFT);
            if (y >= getHeight()) {
                break;
            }
        }

        lastWidth = getWidth();
        lastHeight = getHeight();
    }

    /**
     * This is equivalent to handleFTouchEvent after filtering out events for the menu.
     */
    protected void handleTouchEvent(FTouchEvent event) {
        int type     = event.getType();
        int tapCount = event.getTapCount();

        switch (type) {
            case FTouchEvent.DRAGGING_DOWN:
                scrollDown(1);
                break;
            case FTouchEvent.DRAGGING_UP:
                scrollUp(1);
                break;
        }
    }
}
