/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.view.components.EditField;
import com.funambol.mailclient.ui.view.components.BodyItem;
import com.funambol.mailclient.ui.view.components.Title;
import com.funambol.mail.AccountFolder;
import com.funambol.mail.Address;
import com.funambol.mail.BodyPart;
import com.funambol.mail.MailException;
import java.util.Vector;

import com.funambol.mail.Message;
import com.funambol.mail.Multipart;
import com.funambol.mailclient.MailAccount;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.syncml.FolderData;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;


public class ComposeMessageScreen extends MessageScreen implements CommandListener, ItemStateListener {

    private static final String TAG_LOG = "ComposeMessageScreen";

    //Commands Priorities:

    /** Send Command priority*/
    private static int SEND_CMD_PRIORITY = 10;
    /** Add attachment Command priority*/
    private static int ADD_ATTACHMENTS_CMD_PRIORITY = 20;
    /** Remove attachment Command priority*/
    private static int REMOVE_ATTACHMENTS_CMD_PRIORITY = 25;
    /** Select sender Command priority*/
    private static int SELECT_SENDER_CMD_PRIORITY = 30;
    /** Send Later Command priority*/
    private static int SEND_LATER_CMD_PRIORITY = 40;
    /** Save Command priority*/
    private static int SAVE_CMD_PRIORITY = 50;
    /** Cancel Compose Command priority*/
    protected static int CANCEL_CMD_PRIORITY = 70;

    //Custom Items indexes:

    /** Recipients item index*/
    private static final int RECIPIENTS_ITEM_INDEX = 0;
    /** Attachments item index*/
    private static final int ATTACHMENTS_ITEM_INDEX = 1;
    /** Subject item index*/
    private static final int SUBJECT_ITEM_INDEX = 2;
    /** Body item index*/
    private static final int BODY_ITEM_INDEX = 3;

    //Editable items max size limit

    /** Max size of the input text for the message body text editor*/
    private final int MAX_BODY_SIZE=5000;
    /** Max size of the input text for the subject text editor*/
    private final int MAX_SUBJECT_SIZE=256;

    //Commands

    /** Send command object*/
    private Command sendCommand;
    /** Send Later command object*/
    private Command sendLaterCommand;
    /** Save command object*/
    private Command saveCommand;
    /** Add Attachment command object*/
    private Command addAttachmentCommand;
    /** Remove attachment command object*/
    private Command removeAttachmentCommand;
    /** Cancel Compose command object*/
    private Command cancelCommand;

    private MenuItem sendMenuItem = null;
    /** The attachments subMenu*/
    private Menu filesSubmenu = null;
    /** The vector representation for attached files*/
    private Vector files = null;
    /** The commad array to remove attached files*/
    private Command removeFileCommand[];
    /** The menuItem array to remove attached files*/
    private MenuItem removeMultipleFileItem[];
    /** Cache for the original message */
    private Message originalMessage = null;
    /** MenuItem to display when the attachments list is a single file set*/
    private MenuItem removeSingleFileMenuItem = null;
    /** MenuItem to display when the attachments list is a multiple files set*/
    private MenuItem removeMultipleFileMenuItem = null;
    /** Fixed command to remove automatically all attachments*/
    private Command removeAllFilesCommand = null;
    /** The MenuItem used to select the sender in case of multiple accounts*/
    private MenuItem sendersMenuItem = null;

    /**
     * The string to append to the text of a copose message body before sending
     * it after a reply, reply all and Forward operation
     */
    private String toAppend = "";

    /** 
     * indicate how flags should be changed when forwarding or replying to a 
     * message 
     */
    private int flagToBeSet;

    public ComposeMessageScreen (Message message, String toAppend) {
        super();
    }
    
    public ComposeMessageScreen(Message message) {
        this(message, "");
    }

    public ComposeMessageScreen() {
        super();

        message = new Message();

        title = new Title(theme, Localization.getMessages().COMPOSE_MESSAGE_FORM_TITLE, "", getWidth());
        
        //Initialize the recipients custom item
        recipients = new RecipientsItem(message, drawer, theme);
        recipients.setMaxTextLinesPerLabel(highLightedRecipientsMaxLines);
        recipients.setLabel(Localization.getMessages().SELECT_RECIPIENTS);
        recipients.setText("");
        recipients.setToolTipText(Localization.getMessages().CMF_RECIPIENTS_HELP_MESSAGE);
        recipients.setVisible(true);
        //recipients.setLighter(true);
        
        //Initialize the subject custom item
        subject = new EditField(drawer, theme);
        subject.setMaxTextLinesPerLabel(subjectMaxLines);
        subject.setLabel(Localization.getMessages().CMF_SUBJECT_TF_LABEL);
        subject.setText("");
        subject.setToolTipText(Localization.getMessages().CMF_SUBJECT_HELP_MESSAGE);
        subject.setVisible(true);
        subject.setMaxContentSize(MAX_SUBJECT_SIZE);
        subject.setReadOnly(false);
        subject.setEditFormTitle(Localization.getMessages().CMF_SUBJECT_EDITOR_TITLE);
        subject.setEditCommand(Localization.getMessages().CMF_EDIT_SUBJ_COMMAND);
        //subject.setLighter(true);

        attachments = new EditField(drawer,theme);
        //attachments.setLighter(true);
        //#ifndef noSendAttachments
        updateAttachments();
        //#endif
        
        //Initialize the body custom item
        body = new BodyItem(drawer, theme);
        body.setLabel(Localization.getMessages().CMF_BODY_TF_LABEL);
        body.setText("");
        body.setToolTipText(Localization.getMessages().CMF_BODY_HELP_MESSAGE);
        body.setVisible(true);
        body.setMaxContentSize(MAX_BODY_SIZE);
        body.setReadOnly(false);
        body.setEditFormTitle(Localization.getMessages().CMF_MESSAGE_EDITOR_TITLE);
        body.setEditCommand(Localization.getMessages().CMF_EDIT_MSG_COMMAND);
        body.enableScrollBar(true);
        
        recipients.setItemStateListener(this);
        subject.setItemStateListener(this);
        attachments.setItemStateListener(this);
        body.setItemStateListener(this);
        
        recipients.setDefaultScreenIndex(RECIPIENTS_ITEM_INDEX);
        attachments.setDefaultScreenIndex(ATTACHMENTS_ITEM_INDEX);
        subject.setDefaultScreenIndex(SUBJECT_ITEM_INDEX);
        body.setDefaultScreenIndex(BODY_ITEM_INDEX);
        
        addScreenElement(recipients);
        addScreenElement(subject);
        addScreenElement(body);
        
        currentItem = (EditField) items.elementAt(0);
        
        addCommands();
        
        activateItem(subject);
        title.setSubTitle(currentItem.getToolTipText());
        repaint();
    }

    protected void fireCommand() {
        if (recipients.isHighlighted()) {
            commandAction(recipients.getAddRecipientsCommand(), this);
        } else if (attachments.isHighlighted()) {
            commandAction(addAttachmentCommand, this);
        } else if (subject.isHighlighted()) {
            commandAction(subject.getEditCommand(), this);
        } else if (body.isHighlighted()) {
            commandAction(body.getEditCommand(), this);
        }
    }

    protected void updateScreenCommands() {
        if (currentItem == recipients) {
            addCommand(recipients.getAddRecipientsCommand());
        } else {
            removeCommand(recipients.getAddRecipientsCommand());
        }
        
        if (currentItem == body) {
            addCommand(body.getEditCommand());
        } else {
            removeCommand(body.getEditCommand());
        }
        
        if (currentItem == subject) {
            addCommand(subject.getEditCommand());
        } else {
            removeCommand(subject.getEditCommand());
        }
    }

    private void updateAttachments() {
        
        if (addAttachmentCommand!=null) {
            removeCommand(addAttachmentCommand);
        }

        if (removeSingleFileMenuItem!=null) {
            removeMenuItem(removeSingleFileMenuItem);
        }
        
        if (removeSingleFileMenuItem!=null) {
            removeMenuItem(removeMultipleFileMenuItem);
        }
        
        addAttachmentCommand = new Command (Localization.getMessages().ATTACH_FILE_COMMAND, UIController.COMMAND_TYPE, ADD_ATTACHMENTS_CMD_PRIORITY);
        removeAttachmentCommand = new Command(Localization.getMessages().CMF_REMOVE_SINGLE_FILE, UIController.COMMAND_TYPE, REMOVE_ATTACHMENTS_CMD_PRIORITY);
        
        removeSingleFileMenuItem = new MenuItem(removeAttachmentCommand);
        
        removeMultipleFileMenuItem = new MenuItem(removeAttachmentCommand);
        
        if(message.hasAttachments()){
            //Initializes the attachment item on the screen
            attachments.setToolTipText(Localization.getMessages().CMF_ATTACH_HELP_MESSAGE);
            attachments.setLabel(Localization.getMessages().CMF_ATTACH_LABEL);
            attachments.setVisible(true);
            if (!isItemOnScreen(attachments)) {
                addScreenElement(attachments);
            }
            
            files = UiUtils.getAttachments(message);
            
            //more than 1 attachment found
            if (message.attachmentsNumber() > 1) {
                attachments.setText(" " + message.attachmentsNumber() + " " + Localization.getMessages().CMF_ATTACH_FILES);
                //Creates the submenus to remove the attachments by file name
                filesSubmenu = new Menu(Localization.getMessages().REMOVE_COMMAND_LABEL);
                removeFileCommand = new Command[message.attachmentsNumber()+1];
                removeMultipleFileItem = new MenuItem[message.attachmentsNumber()+1];

                for(int i = 0; i < removeFileCommand.length; ++i) {
                    //Last command is the "Remove All" command
                    if (i == message.attachmentsNumber()) {
                        removeAllFilesCommand = new Command(Localization.getMessages().REMOVE_ALL_ATTACHMENTS_LABEL, Command.SCREEN, i);
                        removeFileCommand[i] = removeAllFilesCommand;
                        removeMultipleFileItem[i] = new MenuItem(removeFileCommand[i]);
                        filesSubmenu.addMenuItem(removeMultipleFileItem[i]);
                        continue;
                    }
                    removeFileCommand[i] = new Command(((BodyPart)files.elementAt(i)).getFileName(), Command.SCREEN, i);
                    removeMultipleFileItem[i] = new MenuItem(removeFileCommand[i]);
                    filesSubmenu.addMenuItem(removeMultipleFileItem[i]);
                }
            //1 attachment found
            } else {
                attachments.setText(" " + ((BodyPart) files.elementAt(0)).getFileName());
            }
            
        //NO attachment found
        } else {
            attachments.setToolTipText(Localization.getMessages().ATTACH_FILE_COMMAND);
            removeScreenElement(attachments);
            attachments.setVisible(false);
        }

        //Add the command to the menu list
        //#ifndef noSendAttachments
        addCommand(addAttachmentCommand);
        //#endif
        if (!message.hasAttachments()) {
            addMenuItem(removeSingleFileMenuItem);
            disableCommand(removeSingleFileMenuItem.getCommand());
        } else if (message.attachmentsNumber() > 1) {
            //Adds a "Remove" menu item with submenus. The submenus entries are 
            //the file names
            removeMultipleFileMenuItem = new MenuItem(filesSubmenu, REMOVE_ATTACHMENTS_CMD_PRIORITY);
            addMenuItem(removeMultipleFileMenuItem);
            
         } else if (message.attachmentsNumber() == 1) {
            //Adds a "Remove File" command to remove the single attached file
            addMenuItem(removeSingleFileMenuItem);
        }
        
        addSeparator(removeAttachmentCommand.getPriority() + 1);
        
    }

    /**
     * Update the senders menu item in case there are more than one email 
     * account
     */
    private void updateSenderAddresses(String visibleName, String emailAddress) {

        // On SEA servers we may have a null email address,
        // in this case we make sure the message has a null
        // from, so that the sender address will be filled by the email
        // connector
        if (emailAddress != null && emailAddress.length() > 0) {
            // Resets the message from
            try {
                Address from = new Address();
                from.setType(Address.FROM);
                from.setName(visibleName);
                from.setEmail(emailAddress);
                message.setFrom(from);
            } catch (MailException me) {
                Log.error(TAG_LOG, "Cannot set from address: " + visibleName +
                        " " + emailAddress, me);
                message.setFrom(null);
            }
        } else {
            message.setFrom(null);
        }

        if (sendersMenuItem != null) {
            removeMenuItem(sendersMenuItem);
        }
        
        AccountFolder[] accounts = mm.getAccounts();
        Menu accountSubMenu = new Menu(Localization.getMessages().CMF_SENDER_LABEL);
        int priority = 0;
        if(accounts.length > 1) {
            for(int i=0; i<accounts.length; i++) {
                Image icon = null;
                if(emailAddress != null && emailAddress.equals(accounts[i].getEmailAddress()))
                {
                    icon = Theme.getSELECT_IMG();
                }
                accountSubMenu.addMenuItem(new MenuItem(
                        new AccountCommand(accounts[i], priority),
                        "", icon));
                priority++;
            }
            // Add separator
            accountSubMenu.addMenuItem(new MenuItem(priority));
     
            sendersMenuItem = new MenuItem(accountSubMenu, SELECT_SENDER_CMD_PRIORITY);
            addMenuItem(sendersMenuItem);
        }     
    }

    /**
     * Update the senders menu item in case there are more than one email 
     * account
     */
    private void setNewSenderAddress(int senderIndex) {

    }
    
    public void update() {
        //Avoid NPE at startup
        AccountFolder af = UIController.getCurrentMailAccount();
        if (af != null) {

            String name = af.getVisibleName();
            String email = af.getEmailAddress();
            
            // Update the sender address
            updateSenderAddresses(name, email);
        }

        //#ifndef noSendAttachments
        updateAttachments();
        //#endif

        
        String content = message.getTextContent();

        int composedMsgLength = message.getComposedMessageLength();
        if(content != null && composedMsgLength>-1){
            String contentToVisualize = content.substring(0,composedMsgLength);
            String contentToAppend = content.substring(composedMsgLength);
            setToAppend(contentToAppend);
            content = contentToVisualize;
        }
        
        //int maxBodySize = body.getMaxContentSize();
        
        if ((content!=null) && (content.length() > MAX_BODY_SIZE)) {
                String sub = content.substring(0, MAX_BODY_SIZE - 5);
            body.setText(sub + " ...");
        } else {
            body.setText(content);
        }
        
        if ((message.getSubject()!=null) && (message.getSubject().length() > MAX_SUBJECT_SIZE)) {
            subject.setText(message.getSubject().substring(0, MAX_SUBJECT_SIZE - 5) + " ...");
        } else {
            subject.setText(message.getSubject());
        }
        
        recipients.setMessage(message);
        repaint();
    }

    
    protected void handleKeyPressed(int keyCode) {
        int dir = getGameAction(keyCode);
        
        if (isFireKey(dir, keyCode)) {
            fireCommand();
        } else if (keyCode == -7) {
//#ifdef isBlackberry
//#         commandAction(UIController.lowLevelBackCommand, this);
//#endif
        } else {
            super.handleKeyPressed(keyCode);
        }
    }

    protected void handleKeyRepeated(int keyCode) {
    
    }

    protected void addCommands() {
        sendCommand = new Command(Localization.getMessages().SEND_COMMAND_LABEL, UIController.COMMAND_TYPE, SEND_CMD_PRIORITY);
        sendMenuItem = new MenuItem(sendCommand);

        addMenuItem(sendMenuItem);
        //#ifdef isBlackberry
        //# addSeparator(sendCommand.getPriority() + 1);
        //#endif
        sendLaterCommand = new Command(Localization.getMessages().CMF_SEND_LATER_COMMAND, UIController.COMMAND_TYPE, SEND_LATER_CMD_PRIORITY);
        saveCommand = new Command(Localization.getMessages().SAVE_DRAFT_COMMAND, UIController.COMMAND_TYPE, SAVE_CMD_PRIORITY);
        
        cancelCommand = new Command(Localization.getMessages().CMF_RECIPIENTS_CANCEL_COMMAND, UIController.COMMAND_TYPE,CANCEL_CMD_PRIORITY);
        if (UIController.isOkToAddOnlineCommands()) {
            sendMenuItem.enable();   
        }else{
            sendMenuItem.disable();
        }

        addCommand(sendLaterCommand);
        
        addCommand(cancelCommand);

        addCommand(saveCommand);
        
        updateScreenCommands();
    }

    /**
     * Enable or disable the send command
     * @param enable if true enable the command, if false disable the command
     */
    public void enableSendCommand(boolean enable) {
        Log.debug(TAG_LOG, "Enable sync commands: " + enable);
        if(enable) {
            sendMenuItem.enable();
        } else {
            sendMenuItem.disable();
        }
    }


    public void commandAction(Command command, Displayable d) {
        Log.debug(TAG_LOG, "Command pressed in compose message form: " + command.getLabel());
        if (command == cancelCommand 
                //#ifdef isBlackberry
                //# || (command == UIController.lowLevelBackCommand)
                //#endif
                )
        {
            UIController.cancelComposeWithConfirmation(message);
        } else if (command == sendCommand) {
            if (prepareMessage()) {
                addToFlagQueue();
                UIController.setSyncCaller(UIController.USER);
                UIController.sendMessage(message);
                if (!UIController.getContactsLoader().isLocked()) {
                    UIController.getContactList().removeNotStoredContacts();
                    UIController.getContactList().SortAlphabetically();
                }
                this.originalMessage = null;
            }
        } else if (command == sendLaterCommand) {
            if (prepareMessage()) {
                addToFlagQueue();
                UIController.getContactList().removeNotStoredContacts();
                UIController.sendLaterMessage(message);
                this.originalMessage = null;
            }
        } else if (command == recipients.getAddRecipientsCommand()) {
            if (!UIController.getContactsLoader().isLocked()) {
                setTitle(Localization.getMessages().LOADING_CONTACTS);
                //Log.debug(this,"title set, opening contact list");
                openContactList();
                setDefaultTitle();
            } else {
                UIController.showErrorAlert(
                        Localization.getMessages().CONTACTSLOADER_LOCKED_ERROR,
                        this);
            }
        } else if (command == this.addAttachmentCommand){
            message.setSubject(subject.getItemContent());

            try {
                String bodyContent = body.getItemContent();
                message.setTextBody(bodyContent + getToAppend());
                message.setComposedMessageLength(bodyContent.length());
            } catch (MailException ex) {
                ex.printStackTrace();
                Log.error(TAG_LOG, "CommandAction Message body invalid!");
            }

            UIController.showFileBrowseScreen(this, message);
        } else if (command == removeAttachmentCommand) {
            message.setSubject(subject.getItemContent());

            try {
                String bodyContent = body.getItemContent();
                message.setTextBody(bodyContent + getToAppend());
                message.setComposedMessageLength(bodyContent.length());
            } catch (MailException ex) {
                ex.printStackTrace();
                Log.error(TAG_LOG, "Message body invalid!");
            }
            
            removeAttachment((BodyPart) UiUtils.getAttachments(message).elementAt(0));
            update();
            activateItem(subject);
        } else if (isRemoveSingleFileCommand(command)) {
            message.setSubject(subject.getItemContent());

            try {
                String bodyContent = body.getItemContent();
                message.setTextBody(bodyContent + getToAppend());
                message.setComposedMessageLength(bodyContent.length());
            } catch (MailException ex) {
                ex.printStackTrace();
                Log.error(TAG_LOG, "Message body invalid!");
            }
            
            removeAttachment((BodyPart) UiUtils.getAttachments(message).elementAt(command.getPriority()));
            update();
            activateItem(subject);
        } else if (command == removeAllFilesCommand) {
            message.setSubject(subject.getItemContent());

            try {
                String bodyContent = body.getItemContent();
                message.setTextBody(bodyContent + getToAppend());
                message.setComposedMessageLength(bodyContent.length());
            } catch (MailException ex) {
                ex.printStackTrace();
                Log.error(TAG_LOG, "Message body invalid!");
            }
            
            Vector v = UiUtils.getAttachments(message);
            for (int i = 0; i < v.size(); i++) {
                removeAttachment((BodyPart) v.elementAt(i));
            }
            update();
            activateItem(subject);
        } else if (command == saveCommand) {
            if (prepareMessage()) {
                message.setComposedMessageLength(body.getItemContent().length());
                UIController.saveMessage(message);
                UIController.getContactList().removeNotStoredContacts();
                this.setToAppend("");
            }
        } else if(command instanceof AccountCommand && command.getCommandType() == AccountCommand.ACCOUNT_COMMAND_TYPE) {
            AccountCommand ac = (AccountCommand)command;
            //Update the selected sender with the check icon
            updateSenderAddresses(ac.getAccount().getVisibleName(), 
                    ac.getAccount().getEmailAddress());
        } else if (command == body.getEditCommand()) {
            UIController.showScreen(body.getItemContentEditor(), this);
        } else if (command == subject.getEditCommand()) {
            UIController.showScreen(subject.getItemContentEditor(), this);
        } else {
            super.commandAction(command, d);
        }
    }

    private boolean isRemoveSingleFileCommand(Command c) {
        if (removeFileCommand!=null) {
            //The last command submenu is the remove all commad
            //And it has his own logic
            for (int i = 0; i < removeFileCommand.length - 1; i++) {
                if (removeFileCommand[i]!=null && c == removeFileCommand[i]) {
                    return true;
                }
            }
        }
        return false;
    }
    
    public EditField getSubject() {
        return subject;
    }
    
    public EditField getBody() {
        return body;
    }
    
    public EditField getRecipients() {
        return recipients;
    }
    
    public void setDefaultTitle() {
        title.setMainTitle(Localization.getMessages().COMPOSE_MESSAGE_FORM_TITLE);
        title.setSubTitle(currentItem.getToolTipText());
    }

    public Message getMessage(){
        return this.message;
    }

    public void setMessage(Message message) {
        this.message=message;
        update();
    }

    public String getToAppend() {
        return toAppend;
    }

    public void setToAppend(String toAppend) {
        this.toAppend = toAppend;
    }

    /**
     * Set the flags that will be added to the flagqueue for the original message
     * when this message will be sent
     * original message must be set using setOriginalMessage before addToFlagQueue
     * method is called (usually at the send or sendlater command action)
     */
    public void setMessageFlag(int flag) {
        flagToBeSet = flag;
    }

    /**
     * If this method is called, with a non-null message,
     * when the send or send later command is pressed,
     * the message passed will have the flags given by
     * the value set by setFlagsToSet added to the flagqueue
     */
    public void setOriginalMessage(Message message) {
        //   Log.info(this, "setting original message to " +
        //           message.getSubject() + "(" + message.getMessageId() +")");
        this.originalMessage = message;
    }

    private boolean prepareMessage() {
        try {
            if (recipients.isItemContentEmpty()) {
                UIController.showAlert(
                        Localization.getMessages().COMPOSE_MESSAGE_FORM_ERROR_MESSAGE,
                        UIController.display.getCurrent());
                return false;
            } else {
                message.setSubject(subject.getItemContent());
                message.setTextBody(body.getItemContent() + getToAppend());
                return true;
            }
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error(TAG_LOG, "Message body invalid!");
            return false;
        }
    }
    
    /**
     * This method adds the messagequeue
     */
    private void addToFlagQueue() {
        if (originalMessage != null) {
            UIController.messageManager.updateMessageFlag(originalMessage, flagToBeSet, true);
            originalMessage = null;
        }
    }
    
    private void openContactList() {

        message.setSubject(subject.getItemContent());
        try {
            //message.setContent(body.getString());
            String bodyContent = body.getItemContent(); 
            message.setTextBody(bodyContent + getToAppend());
            message.setComposedMessageLength(bodyContent.length());
        
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error(TAG_LOG, "Message body invalid!");
        }
        UIController.getContactList().setMessage(message);
        UIController.getContactList().SortByState();
        UIController.getContactList().handleKey(Canvas.KEY_NUM0);
        UIController.showContactList(this);
    }
    
    public void notifyChanged() {
        
        String subj = subject.getItemContent();
        String noCarriage = StringUtil.replaceAll(subj, '\r', ' ');
        noCarriage = StringUtil.replaceAll(subj, '\n', ' ');
        subject.setText(noCarriage);
        
        if (currentItem == recipients) {
            subject.setChanged(true);
            attachments.setChanged(true);
            body.setChanged(true);
        }
        
        if (currentItem == subject) {
            body.setChanged(true);
            getNextItem();
        }
    }
    
    private void removeAttachment(BodyPart bp){
        // start from 1 because the first bodypart is the mail text body
        Multipart multipart = (Multipart) message.getContent();
        for (int i= 1; i < multipart.getCount(); i++ ){
            BodyPart search = multipart.getBodyPart(i);
            if (bp.getAttachUrl().equals(search.getAttachUrl())&&
                bp.getFileName().equals(search.getFileName())){
                multipart.removeBodyPart(i);
                //Avoid twice removal of the same attachment name
                break;
            }
        }

        message.setContent(multipart);
    }
}
