/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.view.components.Title;
import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.loc.LocalizedMessages;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.utils.DevInfo;
import com.funambol.util.Log;

import java.util.Vector;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;

/**
 * <code>ContactList</code> represents a <code>List</code> of
 * <code>ContactItem</code> elements.
 */
public class ContactList extends List implements ContactItemCollectionListener {

    private static final String TAG_LOG = "ContactList";

    private static final int OK_COMMAND_PRIORITY = 10;
    private static final int SELECT_COMMAND_PRIORITY = 20;
    private static final int BB_TO_COMMAND_PRIORITY = 21;
    private static final int BB_CC_COMMAND_PRIORITY = 22;
    private static final int BB_BCC_COMMAND_PRIORITY = 23;
    private static final int BB_CLEAR_SELECTION_COMMAND_PRIORITY = 24;
    private static final int ADD_CONTACT_COMMAND_PRIORITY = 29;
    private static final int EDIT_CONTACT_COMMAND_PRIORITY = 30;
    private static final int DELETE_CONTACT_COMMAND_PRIORITY = 40;
    private static final int SEARCH_COMMAND_PRIORITY = 50;
    private static final int CLEAR_SEARCH_COMMAND_PRIORITY = 50;
    private static final int SYNC_COMMAND_PRIORITY = 70;
    private static final int RESET_COMMAND_PRIORITY = 80;
    private static final int SAVE_CONTACT_COMMAND_PRIORITY = 70;
    private static final int CANCEL_COMMAND_PRIORITY = 90;
    private static final int BB_SHOW_KEYBOARD_COMMAND_PRIORITY = 100;

    /**
     * The message from wich the recipients are shown and to which the
     * recipients are set. It's protected to allow access from the
     * recipient list
     */
    protected Message message;
    
    /** The number to be shown in the upper left corner */
    private int contactCount;

    // commands
    private Command syncCommand;
    private Command addContactCommand;
    private Command okCommand;
    private Command selectCommand;
    private Command editContactCommand;
    private Command deleteContactCommand;
    private Command saveContactCommand;
    private Command resetContactsCommand;
    protected Command toContactCommand;
    protected Command ccContactCommand;
    protected Command bccContactCommand;
    protected Command clearContactCommand;
    private Command cancelCommand;
    private Command searchCommand;
    private Command clearSearchCommand;
    private Command showKeyboard;
    private Command hideKeyboard;

    private ContactItemCollection coll;

    private Drawer drawer;
    
    private T9Search t9;
    private BasicContactItemFilter basicFilter;

    // are we filtering the contact list? (this is true if we are in the process
    // of filtering)
    private boolean filtering = false;

    // This is true if there is a filter currently active. This means that the
    // list of contacts is currently filtered
    private boolean isFilterActive = false;

    private String subtitle;

    /**
     * Specifies if item commands are currently available. This is used to
     * optimize adding/removing commands operations
     */
    protected boolean itemCommandsAvailable = false;

    public ContactList() {
        this(new Message());
    }

    /** Creates a new instance of ContactList with the given message */
    public ContactList(Message message) {
        super();
        this.message = message;
        this.drawer = UIController.getDrawer();

        this.title = new Title(GraphicalThemeFactory.getGraphicalTheme(), getDefaultTitle(), null, getWidth());
        this.title.enableCounterSubstring(true);

        setSubtitle(Localization.getMessages().KEYPAD_TO_SEARCH);

        setCommandListener(this);
        t9 = new T9Search();
        basicFilter = new BasicContactItemFilter();
        initializeContactList();
    }

    /*
     * Used to add/remove menu commands based on the current selection.
     * @see com.funambol.mailclient.ui.view.FunCanvasList#updateItemContext()
     */
    protected void updateItemContext() {
        //Null check
        if (getActiveContactItem() == null) {
            return;
        }

        //Happens when we are viewing contacts in an email. 
        if (clearContactCommand == null || bccContactCommand == null ||
                ccContactCommand == null || toContactCommand == null) {
            return;
        }

//#ifdef isBlackberry
//#     //For BlackBerry
//#     //Start with all item commands. Remove ones we do not want.
//#     addItemCommands();
//#             
//#     switch (getActiveContactItem().getState()) {
//#         case ContactItem.ADDRESS_STATE_NONE:
//#             removeCommand( clearContactCommand );
//#             break;
//#         case Address.BCC:
//#             removeCommand( bccContactCommand );
//#             addCommand( clearContactCommand );        
//#             break;
//#         case Address.CC:
//#             removeCommand( ccContactCommand );
//#             addCommand( clearContactCommand );
//#             break;
//#         case Address.TO:
//#             removeCommand( toContactCommand );
//#             addCommand( clearContactCommand );
//#             break;
//#         default:
//#             break;
//#     }
//#endif
        enableDisableOKCommand();
    }

    /**
     * reset the contact list using the given message.
     * @param message the message
     */
    public void resetContactList(Message message) {
        this.message = message;
        setContactCount(getRmsCount());
        setTitle(getDefaultTitle());
        resetSubtitle();
        emptySearchString();
        initializeContactList();

        // We are about to reset the Contact list to a new message. The
        // collection handler shall lose its reference to the old message
        // as well
        getContactItemCollection().setMessage(message);
    }

    /**
     * @return the default title
     */
    public String getDefaultTitle() {
        return Localization.getMessages().CL_SELECT_RECIPIENTS;
    }

    int paintElement(int elementId, Graphics g) {
        int h = 0;
        //can be null if trying to paint during a sync
        if (elements.elementAt(elementId) != null) {
            g.translate(0, current_paint_h);
            h = ((ContactItem) elements.elementAt(elementId)).paint(g, elementId, isActive(elementId));
            g.translate(0, -current_paint_h);
        }
        return h;
    }

    public ContactItem getActiveContactItem() {
        return (ContactItem) getActiveElement();
    }

    protected void handleKeyPressed(int keyCode) {
        int dir = getGameAction(keyCode);
        // The fire key is handled by the list to trigger the fire action. In
        // the contact list the fire action is used to trigger the to/cc/bcc of
        // the selected item. Usually the "5" acts as fire, but not on this
        // screen, where it is used to search for contacts. So we must intercept
        // the key "5" here and avoid it is handled by the List
        if (dir == Canvas.FIRE && keyCode == 53) {
            handleKey(keyCode);
        } else {
            super.handleKeyPressed(keyCode);
        }
    }

    void handleKey(int keyCode) {

        int dir = getGameAction(keyCode);
        //Spacebar events do not map with getGameAction. Use keycode
        //#ifdef isBlackberry
//#         //keycode 32 is spacebar
        //# if(keyCode == 32) {
        //#    if(getActiveContact() != null) {
        //#        commandAction( toContactCommand, this );
        //#    }
        //#    return;
//#         //keycode 10 is enter
        //# } else if (keyCode == 10) {
        //#    if (getActiveContactItem().hasMultipleEmails()) {
        //#        getActiveContactItem().toNextEmail();
        //#    }
        //# //Keycode 8 is delete on blackberry
        //# } else if (keyCode == 8) {
        //#    if (getActiveContact() !=null) {
        //#        commandAction( deleteContactCommand,this);
        //#    }
        //# } else if (keyCode == -7) {
        //#        commandAction(UIController.lowLevelBackCommand, this);
        //# }
        //#endif

        if (isLeftKey(dir, keyCode)) {
            if (getActiveContactItem().hasMultipleEmails()) {
                getActiveContactItem().toPreviousEmail();
            } else {
                scroll(-1);
            }
            repaint();
            return;
        } else if (isRightKey(dir, keyCode)) {
            if (getActiveContactItem().hasMultipleEmails()) {
                getActiveContactItem().toNextEmail();
            } else {
                scroll(1);
            }
            repaint();
            return;
        }

        // we set the filter if needed
        if (coll.getFilter() == basicFilter) {
            // we need to add the current string to the t9 field
            t9.setString(basicFilter.getSearchString());
            coll.setFilter(t9);
        }
        // clear
        if (UIController.isClearKey(keyCode)) {
            coll.getFilter().clear();
            coll.setToBeFiltered(true);

            removeCommand(clearSearchCommand);
            addSearchCommand();
            isFilterActive = false;
        } else if ((keyCode > 0) && (!isSoftKey(dir, keyCode))) {
            Log.trace(TAG_LOG, "Adding key " + keyCode);
            t9.addKey(keyCode);
            coll.setToBeFiltered(true);
        }

        if (coll.isToBeFiltered()) {

            //repainting only the subtitle
            filtering = true;
            setSubtitle(generateSubtitle());
            repaint();
            serviceRepaints();

            try {
                super.setElements(coll.getItems(coll.NONE));
                filtering = false;
                setSubtitle(generateSubtitle());
                repaint();
                rewindList();
                isFilterActive = true;
            } catch (NoMatchingContactsException ex) {
                // nothing to do if no contacts has been found
                Log.debug(TAG_LOG, "No matching contacts found, nothing to change.");
                setSubtitle(Localization.getMessages().NO_MATCHING_RESULTS);
                repaint();
                serviceRepaints();
                //  ex.printStackTrace();
                filtering = false;
                isFilterActive = false;
            } finally {
                coll.setToBeFiltered(false);
                updateItemContext();
            }
        }
    }

    /**
     * empty the search string
     */
    public void emptySearchString() {
        coll.getFilter().clear();
    }

    protected void initCommands() {
        int callCmdType = Command.BACK;

//#ifdef UseOKCmd
//#         callCmdType = UIController.COMMAND_TYPE;
//#endif
        
        okCommand =
                new Command(Localization.getMessages().AL_OK_COMMAND,
                UIController.COMMAND_TYPE, OK_COMMAND_PRIORITY);

        selectCommand =
                new Command(Localization.getMessages().SELECT_COMMAND,
                UIController.COMMAND_TYPE, SELECT_COMMAND_PRIORITY);

//#ifdef isBlackberry
//#        toContactCommand = new Command(Localization.getMessages().TO_LABEL, UIController.COMMAND_TYPE, BB_TO_COMMAND_PRIORITY);
//#        ccContactCommand = new Command(Localization.getMessages().CC_LABEL, UIController.COMMAND_TYPE, BB_CC_COMMAND_PRIORITY);
//#        bccContactCommand = new Command(Localization.getMessages().BCC_LABEL, UIController.COMMAND_TYPE, BB_BCC_COMMAND_PRIORITY);
//#        clearContactCommand = new Command(Localization.getMessages().REMOVE_CONTACT_COMMAND, UIController.COMMAND_TYPE, BB_CLEAR_SELECTION_COMMAND_PRIORITY);
//#        showKeyboard = new Command(Localization.getMessages().SHOW_KEYBOARD_COMMAND, UIController.COMMAND_TYPE, BB_SHOW_KEYBOARD_COMMAND_PRIORITY);
//#        hideKeyboard = new Command(Localization.getMessages().HIDE_KEYBOARD_COMMAND, UIController.COMMAND_TYPE, BB_SHOW_KEYBOARD_COMMAND_PRIORITY);
//#endif

        addContactCommand =
                new Command(Localization.getMessages().AL_ADD_CONTACT_COMMAND,
                UIController.COMMAND_TYPE, ADD_CONTACT_COMMAND_PRIORITY);

        editContactCommand =
                new Command(Localization.getMessages().EDIT_COMMAND_LABEL,
                UIController.COMMAND_TYPE, EDIT_CONTACT_COMMAND_PRIORITY);

        deleteContactCommand =
                new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, DELETE_CONTACT_COMMAND_PRIORITY);

        searchCommand =
                new Command(Localization.getMessages().AL_SEARCH_COMMAND,
                UIController.COMMAND_TYPE, SEARCH_COMMAND_PRIORITY);
        clearSearchCommand =
                new Command(Localization.getMessages().AL_CLEAR_SEARCH_COMMAND,
                UIController.COMMAND_TYPE, CLEAR_SEARCH_COMMAND_PRIORITY);
        syncCommand =
                new Command(Localization.getMessages().AL_SYNC_COMMAND,
                UIController.COMMAND_TYPE, SYNC_COMMAND_PRIORITY);
        resetContactsCommand =
                new Command(Localization.getMessages().RESET_CONTACTS_COMMAD,
                UIController.COMMAND_TYPE, RESET_COMMAND_PRIORITY);
        saveContactCommand =
                new Command(Localization.getMessages().ABF_SAVE_CONTACT_COMMAND,
                UIController.COMMAND_TYPE, SAVE_CONTACT_COMMAND_PRIORITY);
        cancelCommand = new Command(Localization.getMessages().CMF_RECIPIENTS_CANCEL_COMMAND, Command.BACK, CANCEL_COMMAND_PRIORITY);
    }

    protected void addCommands() {
        super.addCommands();

        addSeparator(SELECT_COMMAND_PRIORITY+1);
        addSearchCommand();

        //#ifdef isBlackberry
        //  addSeparator(BB_BCC_COMMAND_PRIORITY+1);
        //#endif
        
        addSearchCommand();

        addSeparator(DELETE_CONTACT_COMMAND_PRIORITY+1);
        addSeparator(RESET_COMMAND_PRIORITY+1);
        addCommand(cancelCommand);

        // Do not show "Ok" until a contact is selected in TO
        // Now "addCommand(okCommand);" is in toNextState() and in addAddress
        // methocs in the ContactItem class
        //addCommand(okCommand);

        // be sure to have not null addContactCommand
        if (addContactCommand != null) {
            addCommand(addContactCommand);
        }

        // These commands are always available, but they might be disabled if a
        // sync is in progress
        addCommand(syncCommand);
        addCommand(resetContactsCommand);

        enableSyncCommand(UIController.isOkToAddOnlineCommands());
    }

    private void addSearchCommand() {
        if (DevInfo.isTouchscreenSupported()) {
            //#ifdef isBlackberry
            //# addKeyboardCommands();
            //#else
            addCommand(searchCommand);
            //#endif
        }
    }

    void addKeyboardCommands() {
        if (isKeyBoardOnScreen()) {
            removeCommand(showKeyboard);
            addCommand(hideKeyboard);
        } else {
            removeCommand(hideKeyboard);
            addCommand(showKeyboard);
        }
    }

    protected void showKeyboard(boolean show) {
        //#ifdef isBlackberry
        //# super.showKeyboard(show);
        //#endif
    }

    protected boolean isKeyBoardOnScreen() {
        boolean ret = false;
        //#ifdef isBlackberry
        //# ret = super.isKeyboardOnScreen();
        //#endif
        return ret;
    }

    /**
     * Called when the contact list is set to a non empty list
     * removes item-specific commands
     */
    void removeItemCommands() {
        removeCommand(selectCommand);
        removeCommand(editContactCommand);
        removeCommand(deleteContactCommand);


        //#ifdef isBlackberry
//#        if (toContactCommand != null) {
//#            removeCommand( toContactCommand );
//#        }
//#        if (ccContactCommand != null) {
//#            removeCommand( ccContactCommand );
//#        }
//#        if (bccContactCommand != null) {
//#            removeCommand( bccContactCommand );
//#        }
//#        if (clearContactCommand != null) {
//#            removeCommand( clearContactCommand );
//#        }
        //#endif

        setFireCommand(null);
    //ifdef searchform
    //     removeCommand(searchCommand);
    //endif

        itemCommandsAvailable = false;
    }

    /**
     * called when the contact list is set to a non-empty list.
     * adds item-specific commands
     */
    protected void addItemCommands() {

        //#ifdef isBlackberry
//#     if (toContactCommand != null) {
//#         addCommand( toContactCommand );
//#         setFireCommand( toContactCommand );
//#     }
//#     if (ccContactCommand != null) {
//#         addCommand( ccContactCommand );
//#     }
//#     if (bccContactCommand != null) {
//#         addCommand( bccContactCommand );
//#     }
//#     if (clearContactCommand != null) {
//#         addCommand( clearContactCommand );
//#     }
        //#else
        addCommand(selectCommand);
        setFireCommand(selectCommand);
        //#endif
        addCommand(editContactCommand);
        addCommand(deleteContactCommand);

        addSearchCommand();

        itemCommandsAvailable = true;
    }

    void paintBackground(Graphics graphics) {
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
    }

    /**
     * paint the title
     * @return the height of the painted area
     */
    public int paintTitle(Graphics g) {
        //Set the contacts counter
        title.setCounterSubstring(getContactsCounter());
        title.setSubTitle(subtitle);
        return title.paint(g, getWidth(), getHeight());
    }

    private String getContactsCounter() {
        int num = Math.max(elements.size(), contactCount);
        String counterString = null;
        if (elements.size() > 0) {
            counterString = " " + (getActiveElementId() + 1) + "/" + num;
        } else {
            counterString = " " + num;
        }
        return counterString;
    }

    /**
     * @return true if element is activable.
     */
    boolean isActivable(int elementId) {
        return isVisible(elementId);
    }

    /**
     * @return true if element must be shown
     */
    boolean isVisible(int elementId) {
        return true;
    }

    public void commandAction(Command command, Displayable displayable) {

        // The back command has a special meaning depending on the context.
        // If filtering is enabled it can be used to clear the search,
        // otherwise it acts as the compose cancel
        if (command == UIController.lowLevelBackCommand) {
            if (isFilterActive) {
                command = clearSearchCommand;
            } else {
                command = cancelCommand;
            }
        }

        if (command == cancelCommand) {

            //if this is not null, we reset the title
            if (UIController.composeMessageScreen != null) {
                UIController.getComposeMessageScreen().setDefaultTitle();
            }
            // we may have some contactitems in the middle of the list
            // to a state other than NONE, we need to clear them!
            int num = UiUtils.getMessageAddresses(message).size();
            for (int i = num; i < elements.size(); i++) {
                ContactItem c = (ContactItem) elements.elementAt(i);
                c.setState(ContactItem.ADDRESS_STATE_NONE);
            }
            if (! (UIController.backStack.peek() instanceof ComposeMessageScreen)) {
                removeNotStoredContacts();
            }
            this.handleKey(Canvas.KEY_NUM0);
            removeCommand(clearSearchCommand);
            isFilterActive = false;

            UIController.showBackScreen();
        } // if we have the search command
        else if (command == searchCommand) {
            UIController.display.setCurrent(new SearchAddressForm(this));
            addCommand(clearSearchCommand);
            isFilterActive = true;
        } else if (command == showKeyboard) {
            showKeyboard(true);
            addKeyboardCommands();
        } else if (command == hideKeyboard) {
            showKeyboard(false);
            addKeyboardCommands();
        } else if (command == clearSearchCommand) {
            // sending a 0 to clear the search
            this.handleKey(Canvas.KEY_NUM0);
            removeCommand(clearSearchCommand);
            isFilterActive = false;
        } else if (command == okCommand) {

            // we may have some contacts in the current message that have
            // been unselected, we need to put them back in the right position

            coll.partialSort(0, UiUtils.getMessageAddresses(message).size(), coll.ALPHABETICALLY);
            updateMessageRecipients();
            UIController.updateComposeMessageScreen(message);
            UIController.showComposeMessageScreen(
                    null, UIController.getComposeMessageScreen().getSubject());
        } else if (command == addContactCommand) {
            updateMessageRecipients();
            UIController.showAddContactScreen(UIController.display.getCurrent(), this);
        } else if (command == syncCommand) {
            this.setTitle(Localization.getMessages().FAL_SYNC_IN_PROGRESS);
            UIController.setSyncCaller(UIController.USER);
            UIController.synchContacts();
        } //item command
        else if (command == selectCommand) {
            nextActiveContactState();
        } else if (command == toContactCommand) {
            updateActiveContactState(Address.TO);
        } else if (command == ccContactCommand) {
            updateActiveContactState(Address.CC);
        } else if (command == bccContactCommand) {
            updateActiveContactState(Address.BCC);
        } else if (command == clearContactCommand) {
            updateActiveContactState(ContactItem.ADDRESS_STATE_NONE);
        } else if (command == editContactCommand) {
            UIController.showEditContactScreen(this, getActiveContact(), this, AddContactForm.MODE_EDIT_CONTACT);
        } else if (command == deleteContactCommand) {
            deleteContact();
        } else if (command == resetContactsCommand) {
            UIController.resetContactsWithConfirmation();
        } //else if (command == callCommand){
        //     UIController.phoneCall(getActiveContact(), this);
        // }
        else {
            super.commandAction(command, displayable);
        }
    }

    private void updateActiveContactState(int state) {
        getActiveContactItem().setState(state);
        updateItemContext();
        // Restore the original contact list if search is enabled
        restoreContactListFromSearch();
    }

    private void nextActiveContactState() {
        getActiveContactItem().toNextState();
        enableDisableOKCommand();
        // Restore the original contact list if search is enabled
        restoreContactListFromSearch();
    }

    private void restoreContactListFromSearch() {

        // Check if the user is using the search filter
        if((coll.getFilter() == t9 && t9.getStrings() != null) ||
           (coll.getFilter() == basicFilter && basicFilter.getSearchString() != null)) {

            // Clear the search string
            emptySearchString();
            
            filtering = true;
            coll.setToBeFiltered(true);

            setSubtitle(generateSubtitle());
            repaint();
            serviceRepaints();

            try {
                Contact current  = getActiveContact();

                int currentIndex = getActiveElementId();
                int offset = currentIndex-firstVisible;

                super.setElements(coll.getItems(coll.NONE));
                activateElement(current);
                
                currentIndex = getActiveElementId();
                int visibleOffset = Math.max(currentIndex - offset, 0)-firstVisible;

                firstVisible += visibleOffset;
                lastVisible  += visibleOffset;
                
            } catch (NoMatchingContactsException ex) {
                // nothing to do if no contacts has been found
                Log.debug(TAG_LOG, "Not able to recover original contact list");
            }
        }
        filtering = false;
        coll.setToBeFiltered(false);
        setSubtitle(generateSubtitle());

        // Repaint contact list
        repaint();
        serviceRepaints();
    }

    private void deleteContact() {
        Contact contact = getActiveContact();
        if (UIController.getConfig().getDeleteConfirmation()) {
            // In this case there is no need to force a repaint because the
            // popup will force it at the screen change
            UIController.deleteContactWithConfirmation(contact);
        } else {
            UIController.deleteContact(contact);
            // The ContactManager will inform all listeners (including this
            // object) that an item has been deleted. But the update
            // implementation does not force a repaint, so we must do it here.
            repaint();
        }
    }

    private void updateMessageRecipients() {
        message.removeAllRecipients(Address.TO);
        message.removeAllRecipients(Address.CC);
        message.removeAllRecipients(Address.BCC);

        Address address;
        Vector selected = coll.getSelectedItems();
        for (int i = 0; i < selected.size(); i++) {
            ContactItem item = (ContactItem) selected.elementAt(i);
            address = item.getAddress();
            if (address != null) {
                try {
                    message.addRecipient(address);
                } catch (MailException me) {
                    Log.error(TAG_LOG, "mail exception updateMessageRecipients:" + me.toString());
                    me.printStackTrace();
                }
            }
        }
    }

    /**
     * Initialize the contact list
     */
    public void initializeContactList() {
        initializeContactList(ContactItemCollection.ALPHABETICALLY);
    }

    protected void setContactItemList(final Vector contactItems) {
        super.setElements(contactItems);
        enableDisableOKCommand();
        updateItemContext();
        repaint();
    }

    public Contact getActiveContact() {
        if (getActiveContactItem()!=null) {
            return getActiveContactItem().getContact();
        } else {
            return null;
        }
    }

    /**
     * Set the search string array to the given array.
     *
     * @param string an array containing the strings to be used by
     * isVisible function ( if the contact first name, last name or visiblename
     * starts for one of the given strings, the element is visible )
     */
    public void setSearchString(String string) {
        ContactItemFilter oldFilter = coll.getFilter();
        String oldString = null;
        if (oldFilter == basicFilter) {
            // keeping the old string in case we need a rollback
            oldString = basicFilter.getSearchString();
        }

        coll.setFilter(basicFilter);
        basicFilter.setSearchString(string);
        coll.setToBeFiltered(true);
        removeCommand(searchCommand);
        addCommand(clearSearchCommand);
        isFilterActive = true;

        try {
            this.setElements(coll.getItems(coll.NONE));
            setSubtitle(Localization.getMessages().ZERO_TO_CLEAR);
        } catch (NoMatchingContactsException ex) {
            Log.debug(TAG_LOG, "No matching contacts found, doing nothing");
            this.setSubtitle(Localization.getMessages().NO_MATCHING_RESULTS);
            //rollback
            coll.setFilter(oldFilter);

            removeCommand(clearSearchCommand);
            isFilterActive = false;
            addSearchCommand();

            if (oldString != null) {
                // if we're here we have set the filter back to basicfilter, so
                // we can set the string back to what it was
                basicFilter.setSearchString(oldString);
            }
        }
        rewindList();
        repaint();
    }

    /**
     * Add an address to the list of the message recipients
     * @param address the adderss to be added
     */
    public void addAddress(Address address) {
        ContactItemComparator oldComparator = coll.getComparator();
        try {
            // apart from this new address, the contact
            // list is sorted, so we don't need to use the complete sorter
            // but we can just use the one based on the state of the items
            message.addRecipient(address);
            coll.addAddress(address);
            enableDisableOKCommand();
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error(TAG_LOG, "Unable to add recipient " + address.getEmail() + "to message");
        } finally {
            SortByState();
        }
    }

    /**
     * Calls resetHeight() method of the active contact item.
     * This need to be called when the active elements height should be
     * recalculated, maybe because a secondary email address has been added
     */
    void resetActiveElementHeight() {
        getActiveContactItem().resetHeight();
    }

    /**
     * Check if we should add or remove the ok command.
     * ok command is added if there is at least one
     */
    protected void enableDisableOKCommand() {

        ContactItem ci = getActiveContactItem();

        removeCommand(selectCommand);
        removeCommand(okCommand);

        int okPriority = OK_COMMAND_PRIORITY;

        // Change priority only for other devices because
        // on BlackBerry there is not "Select" command
        //#ifndef isBlackberry
            
            int selectPriority = SELECT_COMMAND_PRIORITY;
        
            boolean isItemAddressed = ci != null && ci.getState() != ContactItem.ADDRESS_STATE_NONE;

            if (!isItemAddressed) {
                okPriority = SELECT_COMMAND_PRIORITY;
                selectPriority = OK_COMMAND_PRIORITY;
            }
        
            if(elements.size() > 0){
                selectCommand =
                    new Command(Localization.getMessages().SELECT_COMMAND,
                    UIController.COMMAND_TYPE, selectPriority);
                addCommand(selectCommand);
                setFireCommand(selectCommand);
            }
        //#endif
        
        okCommand = new Command(Localization.getMessages().AL_OK_COMMAND,
                    UIController.COMMAND_TYPE, okPriority);
        
        if (coll.getSelectedItems().size()>0) {
            addCommand(okCommand);
        }
    }

    /**
     * @return contact count
     */
    private int getRmsCount() {
        return UIController.getContactManager().getContactCount();
    }

    /**
     * Set the contact count to be shown in the upper left corner
     * @param count the contact count to be shown in the upper left corner,
     * if a negative number is passed, the contact count used will be the
     * total number of contacts in rms
     */
    public void setContactCount(int count) {
        if (count < 0) {
            count = getRmsCount();
        }
        contactCount = count;

    }

    /**
     * Enable or disable the sync command and the reset contacts command
     * @param enable if true enable the command, if false disable the command
     */
    public void enableSyncCommand(boolean enable) {
        Log.debug(TAG_LOG, "Enable sync commands: " + enable);
        if (enable) {
            //These are null when viewing an email's contacts. Perform null check. 
            if (syncCommand != null) {
                enableCommand(syncCommand);
            }
            if (resetContactsCommand != null) {
                enableCommand(resetContactsCommand);
            }
        } else {
            disableCommand(syncCommand);
            disableCommand(resetContactsCommand);
        }
    }

    /**
     * If the given contact is in the list, it is set as the active element.
     * @param contact the contact that must be set as active element
     */
    public void activateElement(Contact contact) {
        int id = -1;
        for (int i = 0; i < elements.size(); i++) {
            Contact c = ((ContactItem) elements.elementAt(i)).getContact();
            if (c.equals(contact)) {
                id = i;
                break;
            }
        }
        if (id > 0) {
            setActiveElement(id);
        }
    }

    /**
     * Scan the contact list looking for a contact with given email address.
     * @param email is the given contact mail
     * @return the Contact with the give email address , or null if
     * the contact is not present
     */
    public Contact exists(String email) {
        for (int i = 0; i < elements.size(); i++) {
            ContactItem item = (ContactItem) elements.elementAt(i);
            if (item.getContact().hasEmail(email)) {
                return item.getContact();
            }
        }
        return null;
    }

    /**
     * Is given element active?
     */
    private boolean isActive(int elementId) {
        return elementId == getActiveElementId();
    }

    /**
     * @return The contact item collection
     */
    private ContactItemCollection getContactItemCollection() {
        if (coll == null) {
            coll = new ContactItemCollection(message);
            coll.setListener(this);
            coll.setFilter(t9);
        }
        return coll;
    }

    /**
     * Set the contact list message. contact list is updated and resorted.
     * if message has no recipients, contact list is sortted alphabetically,
     * otherwhise is sorted based on the state of the items
     */
    public void setMessage(Message message) {
        this.message = message;
        getContactItemCollection().setMessage(message);
        if (UiUtils.getMessageAddresses(message).size() == 0) {
            initializeContactList(ContactItemCollection.ALPHABETICALLY);
        } else {
            initializeContactList(ContactItemCollection.BY_STATE);
        }
    }

    /**
     * Sort the contact items alphabetically
     */
    public void SortAlphabetically() {
        Sort(ContactItemCollection.ALPHABETICALLY);

    }

    /**
     * Sort the contact list in alpabetical or state order
     */
    private void Sort(int mode) {
        try {
            setContactItemList(coll.getItems(mode));
        } catch (NoMatchingContactsException ex) {
            ex.printStackTrace();
            setContactItemList(new Vector());
        }
    }

    /**
     * Sort the contact items by state
     */
    public void SortByState() {
        Sort(ContactItemCollection.BY_STATE);
    }

    /**
     * Called when an update happens.
     * used to add and remove item commands if needed
     */
    public void update(ContactItemCollectionEvent event) {

        switch (event.getEventCode()) {
            case ContactItemCollectionEvent.EVENT_CODE_ADD_CONTACTITEM:
            case ContactItemCollectionEvent.EVENT_CODE_SORT_CONTACTITEM_LIST:
            case ContactItemCollectionEvent.EVENT_CODE_FILTER_CONTACTITEM_LIST:
                Log.debug(TAG_LOG, "contactlist: end filtering event received");
                if (elements.size() > 0) {
                    if (!itemCommandsAvailable) {
                        addItemCommands();
                    }
                }
                break;
            case ContactItemCollectionEvent.EVENT_CODE_DELETE_CONTACTITEM:
                if (elements.size() == 0) {
                    if (itemCommandsAvailable) {
                        removeItemCommands();
                    }
                }
                enableDisableOKCommand();
                break;
            case ContactItemCollectionEvent.EVENT_CODE_RESET_CONTACTITEM_LIST:
                removeItemCommands();
                if (elements.size() > 0) {
                    addItemCommands();
                }
                break;
            default:
                break;
        }
    }

    public void activate(int newActiveElement) {
        super.activate(newActiveElement);
        removeCommand(editContactCommand);
        removeCommand(deleteContactCommand);
        if (elements.size() > 0) {
            if (getActiveContact().getContactId() > 0) {
                addCommand(editContactCommand);
                addCommand(deleteContactCommand);
                
            }
        }
    }
    
    /**
     * Removes from contact list the contacts that are not stored in the rms
     */
    public void removeNotStoredContacts(){
        for (int i = 0; i< elements.size(); i++) {
            Contact c = ((ContactItem)elements.elementAt(i)).getContact();
            if (c.getContactId()<=0) {
                elements.removeElementAt(i);
                i--;
            }
        }
    }

    /**
     * Generate current subtitle, based on the state of the contact list.
     * subtitle is one of
     * <ul>
     * <li>CONTACT_LIST_SEARCHING</li>
     * <li>KEYPAD_TO_SEARCH</li>
     * <li>ZERO_TO_CLEAR</li>
     * </ul>
     */
    private String generateSubtitle() {
        String s = "";
        if (filtering) {
            s = Localization.getMessages().CONTACT_LIST_SEARCHING;
        } else if (t9.getStrings() == null) {
            s = Localization.getMessages().KEYPAD_TO_SEARCH;
        } else {
            s = Localization.getMessages().ZERO_TO_CLEAR;
        }
        return s;
    }

    private void setSubtitle(String subtitle) {
        this.subtitle = subtitle;
        UiUtils.cutString(subtitle, getWidth(),
                UIController.getDrawer().getGraphicalTheme().getDefaultFont());
        repaint();
    }

    private void initializeContactList(int sortingMode) {
        try {
            setContactItemList(getContactItemCollection().getItems(sortingMode));
        } catch (NoMatchingContactsException ex) {
            Log.debug(TAG_LOG, "No matching contact found, using empty vector");
            setContactItemList(new Vector());
        }
    }

    /**
     * Reset the subtitle
     */
    public void resetSubtitle() {
        setSubtitle(generateSubtitle());
    }

    protected void handleFTouchEvent(FTouchEvent event) {

        int type = event.getType();
        if (type== FTouchEvent.PRESSED) {
            super.handleFTouchEvent(event);
            activate(getActiveElementId());
        }else if (type == FTouchEvent.SCROLL_LEFT) {
            if (getActiveContactItem().hasMultipleEmails()) {
                getActiveContactItem().toPreviousEmail();
            } else {
                scroll(-1);
            }
            repaint();
        } else if (type == FTouchEvent.SCROLL_RIGHT) {
            if (getActiveContactItem().hasMultipleEmails()) {
                getActiveContactItem().toNextEmail();
            } else {
                scroll(1);
            }
            repaint();
        } else {
            super.handleFTouchEvent(event);
        }
    }

    protected void scroll(int amount) {
        super.scroll(amount);
        enableDisableOKCommand();
    }
}
