/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

/**
 *
 * simple drawing interface, implements some methods for drawing simple items
 * TODO: check if an (abstract) class may be better
 *
 */
public interface Drawer {
    
    /**
     * draw the canvas background in the given region
     *
     * @param g a graphics object
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     */
    void drawCanvasBackground(Graphics g, int w, int h);
    
    
    
   /**
     * draw the title background in the given region
     *
     * @param g a graphics object
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     */
    void drawTitleBackground(Graphics g, int w, int h);
    
    /**
     * draw the title background in the given region
     *
     * @param g a graphics object
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     */
    void drawGradientBackground(Graphics g, int x, int y, int w, int h, int startColor, int endColor);
    
    /**
     * draw a background in the given region
     *
     * @param g a graphics object
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     * @selected is the item selected or not?
     */
    void drawBackground(Graphics g, int w, int h, boolean selected);

    /**
     * draw a background in the given region
     * @param g a graphics object
     * @param x the x start coordinate
     * @param y the y start coordinate
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     * @ratio is the int % to switch the color of the list items
     */
    void drawBackground(Graphics g, int x, int y, int w, int h, int ratio, boolean isActive);

    /**
     * Draw a background in the given region and draw a background grayWidth wide
     * with the gray scale increased of grayScaleOffset.
     * 
     * @param g a graphics object
     * @param x the x start coordinate
     * @param y the y start coordinate
     * @param w w the width of the region to be drawn
     * @param h the height of the region to be drawn
     * @param ratio
     * @param isActive True to draw an active element background
     * @param grayWidth The width of the increased gray scale section.
     * @param grayScaleIncrease The gray scale increase to be added to the left
     * side of the background.
     */
    public void drawBackground(Graphics g, int x, int y, int w, int h, int ratio,
            boolean isActive, int grayWidth, int grayScaleIncrease);

    /**
     * draw a background in the given region
     *
     *@param g a graphics object
     *@param x the x coordinate
     *@param y the y coordinate
     *@param w the width of the region to be drawn
     *@param h the height of the region to be drawn
     *@selected is the item selected or not?
     */
    void drawBackground(Graphics g, int x, int y, int w, int h, boolean selected);
    
    /**
     * draw a string in the given graphics at the given coordinates with
     * layout set to Graphics.TOP|Graphics.LEFT
     * @param string the string to be drawn
     *@param graphics the graphics object
     *@param x the x coordinate
     *@param y the y coordinate
     *@param selected if true draw the string with highlighted font
     *
     */
    void drawString(String string, Graphics graphics, int x, int y, boolean selected);
    
    /**
     * draw a string in the given graphics at the given coordinates
     * @param string the string to be drawn
     * @param graphics the graphics object
     * @param x the x coordinate
     * @param y the y coordinate
     * @param layout the layout described with combination of the Graphics layout constants
     * @param selected if true draw the string with highlighted font
     *
     */
    void drawString(String string, Graphics graphics, int x, int y, int layout, boolean selected);
    
    
    /**
     * draw a string using bold font in the given graphics at the given coordinates with
     * layout set to Graphics.TOP|Graphics.LEFT
     * @param string the string to be drawn
     * @param graphics the graphics object
     * @param x the x coordinate
     * @param y the y coordinate
     * @param selected if true draw the string with highlighted font
     *
     */
    void drawBoldString(String string, Graphics graphics, int x, int y, boolean selected);
    
    /**
     * @returns the current graphicaltheme
     */
    GraphicalTheme getGraphicalTheme();

    /**
     * draw the highlighted custom item background
     * @param graphics the graphics object
     * @param x the x coordinate
     * @param y the y coordinate
     * @param w the width of the custom item
     * @param totalH the height of the custom item
     * @param usesGradient draws a gradient background if true, rectangles
     * otherwise
     */
    public void drawHighlightedCustomItemGradientBackground(Graphics g, int x, int y, int w, int h);

    /**
     * Draws the body item background 
     * draw the highlighted custom item background
     * @param graphics the graphics object
     * @param x the x coordinate
     * @param y the y coordinate
     * @param w the width of the custom item
     * @param totalItemH the height of the custom item
     */
    public void drawHighlightedBodyItemBackground(Graphics g, int x, int y, int w, int totalItemH);

    /**
     * draw the highlighted custom item background
     * @param graphics the graphics object
     * @param x the x coordinate
     * @param y the y coordinate
     * @param w the width of the custom item
     * @param totalH the height of the custom item
     * @param usesGradient draws a gradient background if true, rectangles
     * otherwise
     */
    public void drawHighlightedCustomItemSquaredBackground(Graphics g, int x, int y, int w, int h);

    /**
     * Draw a text string and sorround it with a label
     * @param labelText the String to be filled into the label
     * @param labelFont The text font for the label
     * @param g the graphics object
     * @param labelFrameX the x starting point for the label frame
     * @param labelFrameY the y starting point for the label frame
     * @param labelFrameW the label frame width
     * @param labelFrameH the label frame height
     * @param labelFrameArcWidth the arc width
     * @param labelFrameArcHeight the arc eight
     * @param labelTextX the label text x starting point
     * @param labelTextY the label text y starting point
     * @param layout the text alignment layout
     */    
    public void drawCustomItemLabel(String labelText, Font labelFont, Graphics g, int labelFrameX, int labelFrameY, int labelFrameW, int labelFrameH, int labelFrameArcWidth, int labelFrameArcHeight, int labelTextX, int labelTextY, int layout);

    /**
     * Draw the text for the custom item
     * @param s the String to be Drawn
     * @param g the Graphics object
     * @param x the x start point
     * @param y the y start point
     * @param layout the anchor point for the text
     * @param f the text font
     */
    public void drawCustomItemTextString(String s, Graphics g, int x, int y, int layout, Font f, int color);

}
