/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.ui.view;

import java.util.Vector;

import com.funambol.util.Log;


/**
 * This class implements a TouchEvent that as it is captured and recognized
 * by the TouchCanvas. The set of recognized gestures is limited, but it is
 * enough for the email application.
 */
public class FTouchEvent {

    public static final int CLICK          = 0;
    public static final int SCROLL_UP      = 1;
    public static final int SCROLL_DOWN    = 2;
    public static final int SCROLL_LEFT    = 3;
    public static final int SCROLL_RIGHT   = 4;
    public static final int DRAGGING_UP    = 5;
    public static final int DRAGGING_DOWN  = 6;
    public static final int DRAGGING_LEFT  = 7;
    public static final int DRAGGING_RIGHT = 8;
    public static final int PRESSED        = 9;

    public class Coordinate {
        private int x;
        private int y;

        public Coordinate(int x, int y) {
            this.x  = x;
            this.y  = y;
        }

        public int getX() {
            return x;
        }

        public int getY() {
            return y;
        }
    }

    private Vector pressCoordinates    = new Vector(2);
    private Vector releaseCoordinates  = new Vector(2);
    private Vector draggingCoordinates = new Vector(2);
    private int    tapCount    = 0;
    private int    type        = -1;

    public FTouchEvent(int type) {
        this.type = type;
    }

    public void addPressCoordinates(int x, int y) {
        Coordinate coord = new Coordinate(x, y);
        pressCoordinates.addElement(coord);
    }

    public void addReleaseCoordinates(int x, int y) {
        Coordinate coord = new Coordinate(x, y);
        releaseCoordinates.addElement(coord);
    }


    public void addDraggingCoordinates(int x, int y) {
        Coordinate coord = new Coordinate(x, y);
        draggingCoordinates.addElement(coord);
    }

    public void setPressCoordinates(Vector coordinates) {
        this.pressCoordinates = coordinates;
    }

    public void setReleaseCoordinates(Vector coordinates) {
        this.releaseCoordinates = coordinates;
    }

    public void setDraggingCoordinates(Vector coordinates) {
        this.draggingCoordinates = coordinates;
    }

    public void setTapCount(int tapCount) {
        this.tapCount = tapCount;
    }

    public int getType() {
        return type;
    }

    public int getTapCount() {
        return tapCount;
    }

    public Vector getPressCoordinates() {
        return pressCoordinates;
    }

    public Vector getReleaseCoordinates() {
        return releaseCoordinates;
    }


    public Vector getDraggingCoordinates() {
        return releaseCoordinates;
    }

    public int getPressY(int idx) {
        if (idx < pressCoordinates.size()) {
            Coordinate coord = (Coordinate)pressCoordinates.elementAt(idx);
            return coord.getY();
        }
        return -1;
    }

    public int getPressX(int idx) {
        if (idx < pressCoordinates.size()) {
            Coordinate coord = (Coordinate)pressCoordinates.elementAt(idx);
            return coord.getX();
        }
        return -1;
    }

}
