/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mail.Message;
import com.funambol.mail.Multipart;
import com.funambol.mail.BodyPart;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;
import javax.microedition.io.file.FileSystemRegistry;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;

import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.mail.Message;
import com.funambol.mailclient.ui.view.components.Title;


/**
 * Screen to browse the file system to choose files.
 */
public class FileExplorerScreen extends List implements Runnable {

    private static final String TAG_LOG = "FileExplorerScreen";

    private Drawer drawer;

    private static final String MEGA_ROOT = "/";
    private String currDirName;
    private String prevDirName;

    /* special string denotes upper directory */
    private static final String UP_DIRECTORY = "Up";

    /* separator character as defined by FC specification */
    private static final char SEP = '/';

    /* separator string as defined by FC specification */
    private static final String SEP_STR = "/";

    private Command selectCommand;
    private Command backCommand;
    private Command previewCommand;

    private Displayable screen;

    /**
     * the margin
     */
    private static final int MARGIN = 3;

    private Thread thread;

    private Hashtable mimeTypes;

    private FileConnection fileConn = null;

    private boolean firstAccess;

    //#ifdef populateRoots
    //# private Vector roots;
    //#endif

    private FileExplorerScreenListener listener;


    /**
     * Constructor.
     */
    public FileExplorerScreen(Displayable d) {
        drawer = UIController.getDrawer();
        this.title = new Title(GraphicalThemeFactory.getGraphicalTheme(), getDefaultTitle(), null, getWidth());
        screen = d;
        populateMimeTypes();
        addCommand(backCommand);

        if(elements == null){
            elements = new Vector();
        }

        currDirName = UIController.getConfig().getRememberedDir();

        firstAccess = true;
        thread = UIController.getThreadPool().startThread(this);
        thread.setPriority(Thread.NORM_PRIORITY);

        //For samsung issue browsing roots
         //#ifdef populateRoots
         //# roots = new Vector(4);
         //# roots.addElement("Photo/");
         //# roots.addElement("Video/");
         //# roots.addElement("Tones/");
         //# //roots.addElement("Mmc/");
         //#endif

    }

    public void setDisplayable(Displayable d){
        screen = d;
        currDirName = UIController.getConfig().getRememberedDir();
        firstAccess = true;
        thread = UIController.getThreadPool().startThread(this);
        thread.setPriority(Thread.NORM_PRIORITY);
    }

    public void setListener(FileExplorerScreenListener listener) {
        this.listener = listener;
    }


    /**
     * Show file list in the current directory .
     */
    private void showCurrDir() {
        Enumeration e;

        CanvasFileBrowserItem fileBrowserItem;
        // addCommand(upCommand);
        int i = 0;
        try {
            if (MEGA_ROOT.equals(currDirName)) {
                
                //#ifdef populateRoots
                //# e = roots.elements();
                //#else
                e = FileSystemRegistry.listRoots();
                //#endif
                elements.removeAllElements();
            } else {
                if(currDirName.indexOf("file:///")!= -1){
                    currDirName = currDirName.substring("file:///".length());
                }
                elements.removeAllElements();
                fileConn = (FileConnection)Connector.open("file:///" + currDirName, Connector.READ);
                e = fileConn.list();
                firstAccess = false;
                // not root - draw UP_DIRECTORY
                fileBrowserItem = new CanvasFileBrowserItem(UP_DIRECTORY, i++);
                elements.addElement(fileBrowserItem);
                rewindList();
                repaint();
            }

            while (e.hasMoreElements()) {
                setTitle(Localization.getMessages().FES_LOADING);
                String fileName = (String)e.nextElement();

                /**
                 * Nokia devices have vrtual roots that we must
                 * filter out.
                 */
                //#ifdef hasVirtualRoots
                //# boolean loadFile = true;
                //# if (MEGA_ROOT.equals(currDirName)) {
                //#     if(!fileName.endsWith(":/")){
                //#         loadFile = false;
                //#     }
                //# }
                //# if(loadFile){
                //#     fileBrowserItem = new CanvasFileBrowserItem(fileName, i++);
                //#     fileBrowserItem.setFilePath(currDirName);
                //#     elements.addElement(fileBrowserItem);
                //# }
                //#else
                    fileBrowserItem = new CanvasFileBrowserItem(fileName, i++);
                    fileBrowserItem.setFilePath(currDirName);
                    elements.addElement(fileBrowserItem);
                //#endif
                    
                rewindList();
                repaint();
            }

            setTitle(currDirName);
            addItemCommands();
        } catch(SecurityException sec) {
            Log.error(TAG_LOG, "Security exception reading directory " + currDirName, sec);
            currDirName = prevDirName;
            if(firstAccess){
                // In any case we go back to the previous screen
                UIController.showBackScreen();

                // Depending on the device, we shall show an error to the user and
                // go back to the previous screen
                //#ifdef showErrorOnFileBrowsing
                //# try {
                //#     Thread.sleep(200);
                //# } catch (Exception e1) {}
                //# UIController.showAlert(Localization.getMessages().ATTACH_FILE_NA);
                //#endif
            }
        } catch (Throwable ex) {
           Log.error(TAG_LOG, "Exception reading directory " + currDirName, ex);
           UIController.showAlert(Localization.getMessages().UNABLE_OPEN_FOLDER);
        } finally{
            try {
                if (fileConn != null) {
                    fileConn.close();
                    fileConn = null;
                }
            } catch (IOException ioex) {
            }
        }
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == backCommand) {
            UIController.showBackScreen();
        } else if (command == selectCommand) {
            final CanvasFileBrowserItem item = (CanvasFileBrowserItem)getActiveElement();
            final String selectedFile = item.name;
            if (selectedFile.endsWith(SEP_STR) || selectedFile.equals(UP_DIRECTORY)) {
                browseDir(selectedFile);
            } else {
                if (listener != null) {
                    listener.fileSelected(item.getFilePath(),
                                          item.getFileName(),
                                          getContentType(item.getFileName()),
                                          item.getFileSize(), currDirName);
                    listener.attachFile();
                }
            }
        } else if (command == previewCommand) {
            final CanvasFileBrowserItem item = (CanvasFileBrowserItem)getActiveElement();
            if (listener != null) {
                    listener.fileSelected(item.getFilePath(),
                                          item.getFileName(),
                                          getContentType(item.getFileName()),
                                          item.getFileSize(), currDirName);
                }
            UIController.showImageViewerScreen(this, 
                                                item.getFilePath(),
                                                item.getFileName(),listener);
        //#ifdef isBlackberry
        //# } else if (command == UIController.lowLevelBackCommand) {
        //#    if (currDirName.equals(MEGA_ROOT)) {
        //#        UIController.showBackScreen();
        //#    } else {
        //#        browseDir(UP_DIRECTORY);
        //#    }            
        //#endif
        } else {
            super.commandAction(command, displayable);
        }
    }


    void browseDir(String fileName) {
        /* In case of directory just change the current directory
         * and show it
         */
        prevDirName = currDirName;
        if (currDirName.equals(MEGA_ROOT)) {
            currDirName = fileName;
        } else if (fileName.equals(UP_DIRECTORY)) {
            // Go up one directory
            int i = currDirName.lastIndexOf(SEP, currDirName.length() - 2);

            if (i != -1) {
                currDirName = currDirName.substring(0, i + 1);
            } else {
                currDirName = MEGA_ROOT;
            }
        }else if(!fileName.equals(currDirName)){
            currDirName = currDirName + fileName;
        }
        thread = UIController.getThreadPool().startThread(this);
        thread.setPriority(Thread.NORM_PRIORITY);
    }

    private void populateMimeTypes(){
        mimeTypes = new Hashtable();

        //text mimes
        mimeTypes.put("htm", "text/html");
        mimeTypes.put("html","text/html");
        mimeTypes.put("rtx", "text/richtext");
        mimeTypes.put("jad", "text/vnd.sun.j2me.app-descriptor");
        mimeTypes.put("java","text/plain");
        mimeTypes.put("txt", "text/plain");
        mimeTypes.put("vcf", "text/x-vcard");
        mimeTypes.put("wml", "text/vnd.wap.wml");

        //image mimes
        mimeTypes.put("bmp", "image/bmp");
        mimeTypes.put("gif", "image/gif");
        mimeTypes.put("ico", "image/x-icon");
        mimeTypes.put("png", "image/png");
        mimeTypes.put("jpe", "image/jpeg");
        mimeTypes.put("jpeg","image/jpeg");
        mimeTypes.put("jpg", "image/jpeg");
        mimeTypes.put("rgb", "image/x-rgb");
        mimeTypes.put("svg", "image/svg+xml");
        mimeTypes.put("tif", "image/tiff");
        mimeTypes.put("tiff","image/tiff");

        //application mimes
        mimeTypes.put("xhtml","application/xhtml+xml");
        mimeTypes.put("pdf", "application/pdf");
        mimeTypes.put("doc", "application/msword");
        mimeTypes.put("xls", "application/vnd.ms-excel");
        mimeTypes.put("pps", "application/mspowerpoint");
        mimeTypes.put("ppt", "application/mspowerpoint");
        mimeTypes.put("rar", "application/x-rar-compressed");
        mimeTypes.put("jar", "application/java-archive");
        mimeTypes.put("xml", "application/xml");
        mimeTypes.put("zip", "application/zip");
        mimeTypes.put("exe", "application/octet-stream");
        mimeTypes.put("odt","application/vnd.oasis.opendocument.text");
        mimeTypes.put("ott","application/vnd.oasis.opendocument.text-template");
        mimeTypes.put("oth","application/vnd.oasis.opendocument.text-web");
        mimeTypes.put("odm","application/vnd.oasis.opendocument.text-master");
        mimeTypes.put("odg","application/vnd.oasis.opendocument.graphics");
        mimeTypes.put("otg","application/vnd.oasis.opendocument.graphics-template");
        mimeTypes.put("odp","application/vnd.oasis.opendocument.presentation");
        mimeTypes.put("otp","application/vnd.oasis.opendocument.presentation-template");
        mimeTypes.put("ods","application/vnd.oasis.opendocument.spreadsheet");
        mimeTypes.put("ots","application/vnd.oasis.opendocument.spreadsheet-template");
        mimeTypes.put("odc","application/vnd.oasis.opendocument.chart");
        mimeTypes.put("odf","application/vnd.oasis.opendocument.formula");
        mimeTypes.put("odb","application/vnd.oasis.opendocument.database");
        mimeTypes.put("odi","application/vnd.oasis.opendocument.image");
        mimeTypes.put("oxt","application/vnd.openofficeorg.extension");

        //audio mimes
        mimeTypes.put("3gpp","audio/3gp");
        mimeTypes.put("kar", "audio/midi");
        mimeTypes.put("m3u", "audio/x-mpegurl");
        mimeTypes.put("m4a", "audio/mp4");
        mimeTypes.put("mid", "audio/x-midi");
        mimeTypes.put("midi","audio/x-midi");
        mimeTypes.put("mp2", "audio/mpeg");
        mimeTypes.put("mp3", "audio/mp3");
        mimeTypes.put("ram", "audio/x-pn-realaudio");
        mimeTypes.put("rmf", "audio/rmf");
        mimeTypes.put("wav", "audio/wav");
        mimeTypes.put("wma", "audio/x-ms-wma");

        //video mimes
        mimeTypes.put("3gp", "video/3gp");
        mimeTypes.put("avi", "video/avi");
        mimeTypes.put("m4u", "video/vnd.mpegurl");
        mimeTypes.put("mov", "video/quicktime");
        mimeTypes.put("mp4", "video/mp4");
        mimeTypes.put("mpeg","video/mpeg");
    }

    int paintElement(int elementId, Graphics g) {
        g.translate(0, current_paint_h);
        int h = ((CanvasFileBrowserItem) elements.elementAt(elementId)).paint(g);
        g.translate(0, -current_paint_h);
        return h;
    }

    int paintTitle(Graphics g) {

        title.setMainTitle(getDefaultTitle());
        
        String subTitle = getFunTitle();
        if (subTitle == null) {
            subTitle = "";
        }

        int subTitlelineSpace = getWidth()- 4*MARGIN;

        String subTitleToShow = UiUtils.cutHeadString(subTitle, subTitlelineSpace , GraphicalThemeFactory.getGraphicalTheme().getDefaultFont());
        
        title.setSubTitle(subTitleToShow);
        
        
        return title.paint(g, getWidth(), getHeight());
    }

    /**
     * @return the default title
     */
    public String getDefaultTitle() {
        return Localization.getMessages().ATTACH_FILE_TITLE;
    }

    protected void initCommands() {
        previewCommand = new Command(Localization.getMessages().PREVIEW_COMMAND, Command.OK, 10);
        selectCommand = new Command(Localization.getMessages().SELECT_COMMAND, Command.OK, 20);
        backCommand = new Command(Localization.getMessages().BACK_COMMAND, Command.BACK, 30);
        
        setCommandListener(this);
    }

    protected void addCommands() {

    }

    void removeItemCommands() {
        
        removeCommand(selectCommand);
        setFireCommand(null);
    }

    void addItemCommands() {
        
        addCommand(selectCommand);
        setFireCommand(selectCommand);
    }

    public void setActiveElement(int newActiveElement) {
        super.setActiveElement(newActiveElement);
        enableDisablePreview();
    }

    private void enableDisablePreview() {
        final CanvasFileBrowserItem item = (CanvasFileBrowserItem)getActiveElement();
        if (item != null) {
            final String selectedFile = item.name;
            if (StringUtil.endsWithIgnoreCase(selectedFile, "jpg") ||
                StringUtil.endsWithIgnoreCase(selectedFile, "jpeg") ||
                StringUtil.endsWithIgnoreCase(selectedFile, "png"))
            {
                addCommand(previewCommand);
                setFireCommand(previewCommand);
                return;
            }
        }
        removeCommand(previewCommand);
        setFireCommand(selectCommand);
    }

    /**
     * paint the background
     */
    void paintBackground(Graphics graphics) {
        Log.debug(TAG_LOG, "paint background!");
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
    }

    boolean isVisible(int elementId) {
        return elements.elementAt(elementId) != null;
    }

    boolean isActivable(int elementId) {
        return isVisible(elementId);
    }

    public void run() {
       showCurrDir();
    }

    void handleKey(int keyCode) {
        if (keyCode == -7) {
//#ifdef isBlackberry
//#         commandAction(UIController.lowLevelBackCommand, this);
//#endif
        }
    }

    /**
     * Get the selected item from y coordinate
     * @param y
     * @return
     */
    protected int selectItem(int y) {
        if (y > firstElementOffset) {
            int index = firstVisible + ((y - firstElementOffset) / elementHeight);
            return index;
        }
        return -1;
    }

    /**
     * return the content type.
     */
    private String getContentType(String name) {

        String ext = name.substring(name.lastIndexOf('.')+1);
        Object type = mimeTypes.get(ext);

        if (type == null) {
            return "application/octet-stream";
        } else {
            return (String)type;
        }
    }

    //------------------------ CanvasFileBrowserItem ---------------------------------//
    /**
     * this class draws the attach item
     */
    class CanvasFileBrowserItem {

        private int id;
        private String name;
        private int fileSize = -1;
        private String filePath;
        private byte[] fileByte;

        /**
         * the dir/file name that fits on a single line
         */
        private String shortName;

        CanvasFileBrowserItem(String name, int id) {
            this.id = id;
            this.name = name;
        }

        private boolean isActive() {
            return id == getActiveElementId();
        }

        public void setFilePath(String path){
            filePath = path;
        }

        public String getFileName(){
            return name;
        }

        public String getFilePath(){
            return filePath;
        }

        public int getFileSize(){
            return fileSize;
        }

        /**
         * paint the attachmentitem
         */
        public int paint(Graphics graphics) {
            int w = getWidth();
            int h = calculateHeight();
            calculateShortStrings();
            
             drawer.drawBackground(graphics, 0, 0, w, h, (id+1)%2, isActive());
            if(name.charAt(name.length() - 1) == SEP){
                graphics.drawImage(Theme.getDIR_IMG(), MARGIN, MARGIN, Graphics.TOP | Graphics.LEFT);
                drawer.drawString(shortName, graphics, Theme.getDIR_IMG().getWidth()+
                    2 * MARGIN, MARGIN, isActive());
            }else if(name.equals(UP_DIRECTORY)){
                graphics.drawImage(Theme.getUPLEVEL_IMG(), MARGIN , MARGIN, Graphics.TOP | Graphics.LEFT);
                drawer.drawString(Localization.getMessages().FES_UP, graphics,
                        MARGIN+Theme.getUPLEVEL_IMG().getWidth() + 2 * MARGIN, MARGIN , isActive());
            }else{
                graphics.drawImage(Theme.getFILE_IMG(), MARGIN , MARGIN, Graphics.TOP | Graphics.LEFT);
                drawer.drawString(shortName, graphics, Theme.getFILE_IMG().getWidth()+
                    2 * MARGIN, MARGIN, isActive());
            }

            return h;
        }


        /**
         * calculate the short version of address and subject and
         * store them in shortAddress and shortSubject
         */
        private void calculateShortStrings() {
            if (shortName == null) {

                int w = getWidth();
                int firstlineSpace = w - 4 * MARGIN;
                if(name.charAt(name.length() - 1) == SEP ){
                    firstlineSpace= firstlineSpace - Theme.getDIR_IMG().getWidth();
                }
                shortName = UiUtils.cutString(name, firstlineSpace, drawer.getGraphicalTheme().getDefaultFont());
            } else {
            //   Log.error("no need to calculate shortstring");
            }
        }

        private void setId(int newId) {
            this.id = newId;
        }

        /*
         * compute height using information from the bean,
         * the drawer and the icon size
         */
        private int calculateHeight() {
            return Theme.getUPLEVEL_IMG().getHeight() + 2 * MARGIN;
        }

        public byte[] getContent(){
            InputStream fis = null;
            try{
                fileConn = (FileConnection)Connector.open("file:///"+filePath+name, Connector.READ);
                fis = fileConn.openInputStream();
                fileSize = (int)fileConn.fileSize();
                fileByte = new byte[fileSize];

                fis.read(fileByte, 0, fileSize);
                fis.close();
                fileConn.close();
                return fileByte;
            }catch (Exception ex) {
                Log.error(TAG_LOG, "gettingContent exception", ex);
                UIController.showAlert(Localization.getMessages().UNABLE_ATTACH_FILE);
                try {
                    if(fis != null){
                        fis.close();
                    }
                    if (fileConn != null) {
                        fileConn.close();
                        fileConn = null;
                    }
                } catch (IOException ioex) {
                }
                return null;
            }
        }
    }
}
