/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import javax.microedition.lcdui.Font;

/**
 *a color theme
 *
 */

/*
 * TODO: this should really be turned into an (abstract?) class, as some methods
 * should have default values
 */
public interface GraphicalTheme {
    /**
     * @return backgroung color
     */
    int getBackgroundColor();
    
    /**
     * @return border color
     */
    int getBorderColor();
    
    /**
     * @return title color
     */
    int getTitleColor();
    
    /**
     * @return the title background color
     */
    int getTitleBackgroundColor() ;
    
    /**
     * @return the title background color in rgb format
     */
    public int[] getTitleBackgroundRgbColor();
    
    /**
     * @return the subtitle background color in rgb format
     */
    public int[] getSubtitleBackgroundRGBColor();

    
    /**
     * @return the title border color
     */
    int getTitleBorderColor() ;
    
    /**
     *@return title font
     */
    Font getTitleFont();

    /**
     *@return menu font
     */
    Font getMenuFont();
    
    /**
     *@return default font
     */
    Font getDefaultFont();
    
    /**
     *@return small  font
     */
    Font getSmallFont();
    
    /**
     *@return small bold font
     */
    Font getSmallBoldFont();
    
    
    /**
     *@return bold font
     */
    Font getBoldFont();
    
    /**
     *@return bold font
     */
    Font getLabelFont();
    
    /**
     * @return the foreground color
     */
    int getForegroundColor();
    
    /**
     * @return the highlighted bg color
     */
    int getHighlightedBackgroundColor();
    
    /**
     * @return the highlighted bordere color
     */
    int getHighlightedBorderColor();
    
    /**
     * @return the highlighted foreground color
     */
    int getHighlightedForegroundColor();
    
    /**
     * set the background color
     */
    void setBackgroundColor(int backgroundColor);
    /**
     * set the  color
     */
    void setBorderColor(int borderColor);
    
    /**
     * set the foreground color
     */
    void setForegroundColor(int foregroundColor);
    
    /**
     * set the highlighted background color
     */
    void setHighlightedBackgroundColor(int highlightedBackgroundColor);
    
    /**
     * set the  highlighted border color
     */
    void setHighlightedBorderColor(int higlightedBorderColor);
    
    /**
     * set the highlighted foreground color
     */
    void setHighlightedForegroundColor(int higlightedForegroundColor);

    /**
     * get the background color for the scrollbar
     */
    int getScrollbarBackgroundColor();

    /**
     * get the menu item foreground color (color of the menu text)
     */
    int getMenuItemFgColor();

    /**
     * get the menu item background color
     */
    int getMenuItemBgColor();

    int getMenuItemDisabledFgColor();

    /**
     * get the menu item foreground color when the item is selected
     */
    int getMenuItemSelectedFgColor();

    /**
     * get the menu item background color when the item is selected
     */
    int getMenuItemSelectedBgColor();

    /**
     * get the menu selection box border color
     */
    int getMenuItemSelectedBorderColor();

    /**
     * get the menu scroll bar foreground color
     */
    int getMenuScrollBarFgColor();

    /**
     * get the menu scroll bar background color
     */
    int getMenuScrollBarBgColor();

    /**
     * get the command bar background color
     */
    int getMenuBarBgColor();

    /**
     * get the command bar foreground color
     */
    int getMenuBarFgColor();
    
    /**
     * get the color of subtitle
     * @return int the subtitle color
     */
    public int getSubTitleColor();
    
    /**
     * get the color of subtitle background
     * @return int the background subtitle color
     */
    public int getSubTitleBackgroundColor();
    
    /**
     * get the color of title frame
     * @return int the title frame color
     */
    public int[] getTitleFrameColor();
        
    /**
     * get the color of label background 
     * @return int the label background color
     */
    public int getLabelBackgroundColor();

    /**
     * get the color of label text
     * @return int the label text color
     */
    public int getLabelTextColor();
    
    /**
     * get the start color for the cutom item gradient
     * @return int the custom item gradient start color
     */
    public int getCustomItemBgColorStart();
    
    /**
     * get the end color for the cutom item gradient
     * @return int the custom item gradient end color
     */
    public int getCustomItemBgColorEnd();

    /**
     * get the start lighter color for the custom item gradient
     * @return int the custom item gradient start color
     */
    public int getCustomItemLighterBgColorStart();

    /**
     * get the end lighter color for the custom item gradient
     * @return int the custom item gradient end color
     */
    public int getCustomItemLighterBgColorEnd();
    
    /**
     * get the dark color for the squared custom item background
     * when it is highlighted
     * @return int the dark color for the squared custom item background
     */
    public int getCustomItemHighlightedSquaredDarkColor();

    /**
     * get the bright color for the squared custom item background
     * when it is highlighted
     * @return int the bright color for the squared custom item background
     */
    public int getCustomItemHighlightedSquaredBrightColor();

    /**
     * get the dark color for the squared custom item frame
     * @return int the dark color for the squared custom item frame
     */
    public int getCustomItemSquaredFrameDarkColor();

    /**
     * get the bright color for the squared custom item frame
     * @return int the bright color for the squared custom item frame
     */
    public int getCustomItemSquaredFrameBrightColor();

    /**
     * get the dark color for the gradient custom item background
     * when it is highlighted
     * @return int the dark color for the gradient custom item background
     */
    public int getCustomItemHighlightedGradientDarkColor();

    /**
     * get the bright color for the gradient custom item background
     * when it is highlighted
     * @return int the bright color for the gradient custom item background
     */
    public int getCustomItemHighlightedGradientBrightColor();

    /**
     * get the dark color for the gradient custom item frame
     * @return int the dark color for the gradient custom item frame
     */
    public int getCustomItemGradientFrameDarkColor();

    /**
     * get the bright color for the gradient custom item frame
     * @return int the bright color for the gradient custom item frame
     */
    public int getCustomItemGradientFrameBrightColor();

    /**
     * Get the color to paint the frame for the body of a message
     * @return int[] the 9 rgb components to paint the 3 colors
     * of the body frame when it is highlighted
     */
    public int[] getBodyHighLightedFrameColor();
    
    /**
     * Get the color to paint the background of the body of a message
     * @return int[] the 3 rgb components to paint the colors
     * of the body text background when it is highlighted
     */
    public int[] getBodyHighLightedTextColor();

    /**
     * Get the color to paint the arrows in case of submenu
     * @return int the submenu arrows color
     */
    public int getSubMenuArrowColor();

    /**
     * Get the color for shorcuts
     * @return int shortcuts color
     */
    public int getShortcutsFgColor();
    
    
    
}
