/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.view;

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.Font;
import java.io.InputStream;
import java.io.IOException;
import java.util.Enumeration;

import com.funambol.mailclient.loc.Localization;
import com.funambol.platform.FileAdapter;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.ThumbnailGenerator;
import com.funambol.util.Log;

/**
 * This class implements a screen to show an image. The image is resized to fit
 * on the screen. The resizing work is carried out by a Resizer object whose
 * implementation is platform dependent for efficiency reasons.
 */
public class ImageViewerScreen extends CustomMenuCanvas implements CommandListener {

    private Image thumbnail;
    private String error;
    private String fileName;
    private Resizer resizer;

    private Command selectCommand;
    private Command backCommand;
    private FileExplorerScreenListener listener;

    private String path;
    private String name;

    private class Resizer extends Thread {
        private String path;
        private String name;

        public Resizer(String path, String name) {
            this.path = path;
            this.name = name;
        }

        public void run() {
            try {
                ThumbnailGenerator tg = new ThumbnailGenerator(path, name);
                Thread.yield();
                thumbnail = tg.createThumbnail(getWidth(), getHeight());
            } catch (Exception e) {
                Log.error("Error resizing image " + e.toString());
                error = Localization.getMessages().IMAGE_RESIZING_ERROR;
            }
            repaint();
        }
    }


    public ImageViewerScreen(String path, String name,FileExplorerScreenListener listener) {
        super();
        this.path = path;
        this.name = name;
        this.fileName = "file:///" + path + name;
        this.listener = listener;
        initCommands();
        addCommands();
    }

     protected void initCommands() {
        backCommand = new Command(Localization.getMessages().BACK_COMMAND, Command.BACK, 10);
        selectCommand = new Command(Localization.getMessages().SELECT_COMMAND, Command.OK, 20);
    }


    protected void addCommands() {
        this.addCommand(UIController.backCommand);
        this.addCommand(UIController.selectCommand);

        setCommandListener(this);
    }

    protected void paintContent(Graphics graphics) {

        Drawer drawer = UIController.getDrawer();
        Font f = drawer.getGraphicalTheme().getDefaultFont();

        int screenHeight = this.getHeight();
        int screenWidth  = this.getWidth();
        graphics.setColor(0xFFFFFF);
        graphics.fillRect(0, 0, screenWidth, screenHeight);

        if (error != null) {
            int x = (getWidth() - f.stringWidth(error)) / 2;
            if (x < 0) {
                x = 0;
            }
            int y = (getHeight() - f.getHeight()) / 2;
            graphics.setColor(0x0);
            graphics.drawString(error, x, y, Graphics.BOTTOM | Graphics.LEFT);
        } else if (thumbnail == null) {
            String loadingMsg = Localization.getMessages().IVS_LOADING;
            int x = (getWidth() - f.stringWidth(loadingMsg)) / 2;
            if (x < 0) {
                x = 0;
            }
            int y = (getHeight() - f.getHeight()) / 2;
            graphics.setColor(0x0);
            graphics.drawString(loadingMsg, x, y,
                                Graphics.BOTTOM | Graphics.LEFT);
            if (resizer == null) {
                resizer = new Resizer(path, name);
                resizer.start();
            }
        }

        if (thumbnail != null) {
            graphics.setColor(0xFFFFFF);
            graphics.fillRect(0, 0, screenWidth, screenHeight);
            graphics.drawImage(thumbnail, 0, 0, Graphics.TOP | Graphics.LEFT);
        }
    }

    protected void handleKeyPressed(int keyCode) {
        int dir = getGameAction(keyCode);
        if (keyCode == -7) {
//#ifdef isBlackberry
//#         commandAction(UIController.lowLevelBackCommand, this);
//#endif
        } else if (isFireKey(dir, keyCode)) {
//#ifdef isBlackberry
//#         handleFireKeyBB();
//#endif
        }
    }

    private void handleFireKeyBB() {
        // Show the menu when the user presses the fire button
        openMainMenu();
    }

    protected void handleKeyRepeated(int keyCode) {
    }

    protected void handleTouchEvent(FTouchEvent event) {
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == UIController.backCommand ||
            command == UIController.lowLevelBackCommand) {
            UIController.showBackScreen();
        } else if(command == UIController.selectCommand) {
            listener.attachFile();
        }
    }
}
