/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.view.components.Title;
import com.funambol.mail.AccountFolder;
import com.funambol.mail.Folder;
import com.funambol.mail.Message;
import com.funambol.mail.MailException;

import com.funambol.mail.MessageFlags;
import com.funambol.mailclient.Customization;
import com.funambol.mailclient.Funambol;
import com.funambol.mailclient.UpdaterManager;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.mm.MessagesLoaderListener;
import com.funambol.mailclient.mm.UnreadMessagesListener;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UserNotificationManager;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.syncml.FolderData;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.Canvas;
import javax.microedition.rms.RecordStoreException;

public class InboxMessageList extends MessageList implements UnreadMessagesListener, MessagesLoaderListener {

    private static final String TAG_LOG = "InboxMessageList";

    //#ifdef isBlackberry 
    //# private static final int HIDE_CMD_PRIORITY = 135;//99;
    //#else
    private static final int HIDE_CMD_PRIORITY = 10;//69;
    //#endif
    
    private static final int OPEN_CMD_PRIORITY = 20;//8;
    private static final int SYNC_CMD_PRIORITY = 30;//5;

    private static final int DELETE_CMD_PRIORITY = 40;//20;
    private static final int FLAG_CMD_PRIORITY = 50;//30;
    private static final int UNFLAG_CMD_PRIORITY = 50;//30;
    private static final int COMPOSE_CMD_PRIORITY = 60;//10;

    private static final int GOTO_CMD_PRIORITY = 80;//40;
    private static final int SETTINGS_CMD_PRIORITY = 90;//50;

    
    private static final int ABOUT_CMD_PRIORITY = 100;//20;
    
    private static final int UPDATE_CMD_PRIORITY = 110;//20;
    private static final int EXIT_CMD_PRIORITY = 120;//70;

    // Settings SubMenu command priorities
    private static final int ACCOUNT_SETTINGS_CMD_PRIORITY = 10;
    private static final int GENERAL_SETTINGS_CMD_PRIORITY = 20;
    private static final int LOGGING_CMD_PRIORITY = 30;
    private static final int RESET_CMD_PRIORITY = 40;

    private Command openCommand;
    private Command composeCommand;
    private Command flagCommand;
    private Command exitCommand;
    private Command syncCommand;
    private Command deleteCommand;
    private Command backgroundCommand;
    private Command updateCommand;
    private Command aboutCommand;

    // Go to commands
    private Command folderCommand[];

    // Settings commands
    private Command accountSettingsCommand;
    private Command generalSettingsCommand;
    private Command loggingCommand;
    private Command resetCommand;

    private MenuItem flagMenuItem;
    private MenuItem gotoMenuItem;
    private MenuItem settingsMenuItem;
    private MenuItem deleteMenuItem;
    
    private boolean isLoadingMessages = false;

    /**
     * This represents a cache of all the messages contained in the store
     * It is populated by the load method when the list is initialized.
     * If there is not enough space in memory it will contain only the current 
     * mail account messages. If the user change the account, then the messages
     * of the new account selected must be loaded.
     */
    private Hashtable allMessages = new Hashtable();

    /**
     * Cache the informations related to the last message selection for each
     * account
     */
    private Hashtable lastSelectedAccountMessage = new Hashtable();

    /**
     * This is used in order to refresh the titlebar only one time after the
     * synchronization ends.
     */
    private boolean syncEnded = false;

    /**
     * This is a reference to a class responsible to carry out an account switch
     */
    private AccountSwitcher accountSwitcher = new AccountSwitcher(200);

    /** Creates a new instance of InboxMessageList */
    public InboxMessageList(Vector msgInfo) {
        super(msgInfo);
        this.role = FolderData.INBOX_ROLE;
        title = new Title(GraphicalThemeFactory.getGraphicalTheme(), getDefaultTitle(), null, getWidth());
        if (UIController.isOkToAddOnlineCommands()) {
            Log.debug(TAG_LOG, "Enabling sync commands");
            enableSyncCommand(true);
        }
        // Set the UnreadMessagesListener in MessageManager
        MessageManager.getInstance().setUnreadMessagesListener(this);
    }

    public void newAccountAdded(AccountFolder newAccount) {
        allMessages.put(newAccount.getEmailAddress(), new Vector());
        // Update the goto submenu (add one entry)
        gotoMenuItem = getGotoSubMenu();
        updateMenuItem(gotoMenuItem);
    }

    public void accountRemoved(AccountFolder account) {
        accountRemoved(account.getEmailAddress());
    }

    public void accountUpdated(AccountFolder oldAccount, AccountFolder newAccount) {
        String oldEA = oldAccount.getEmailAddress();
        String newEA = newAccount.getEmailAddress();
        if(!oldEA.equals(newEA)) {
            Vector messages = (Vector)allMessages.get(oldEA);
            if(messages != null) {
                allMessages.put(newEA, messages);
                allMessages.remove(oldEA);
            } else {
                Log.error(TAG_LOG, "Cannot find messages from account: " + oldEA); 
            }
        }
    }

    public void accountRemoved(String accountAddress) {
        // If the removed account is the one being displayed, then we shall move
        // to the next one (if it exists)
        if (accountAddress.equals(UIController.getCurrentMailAccount())) {
            Log.debug(TAG_LOG, "Removed current account, moving to the next one");
            UIController.nextMailAccount(this);
        }

        // Now remove the account
        allMessages.remove(accountAddress);

        // Update the goto submenu (remove one entry)
        gotoMenuItem = getGotoSubMenu();
        updateMenuItem(gotoMenuItem);
    }

    public String getDefaultTitle() {
        Log.trace(TAG_LOG, "getDefaultTitle");
        // If there are unread messages, then we show their number
        int unread = getUnreadMessages();
        return getDefaultTitle(unread);
    }

    public String getDefaultTitle(int unread) {
        Log.trace(TAG_LOG, "getDefaultTitle(unread)");

        String defaultTitle = null;
        
        if (UIController.getCurrentMailAccount() != null &&
            UIController.messageManager.getAccounts().length > 1)
        {
            defaultTitle = UIController.getCurrentMailAccount().getEmailAddress();
        }
        
        StringBuffer title = new StringBuffer(StringUtil.isNullOrEmpty(defaultTitle) ?
                                 Localization.getMessages().INBOX_DEFAULT_TITLE :
                                 defaultTitle);

        if (unread > 0) {
            title.append(" (").append(unread).append(")");
        }
        return title.toString();
    }

    protected void addCommands() {
        Log.trace(TAG_LOG, "addCommands");
        super.addCommands();

//#ifdef backgroundCommand
//#     addCommand(backgroundCommand);
//#endif

        addCommand(openCommand);
        disableCommand(openCommand);

        addCommand(syncCommand);
        addSeparator(SYNC_CMD_PRIORITY + 1);
        addMenuItem(deleteMenuItem);
        addMenuItem(flagMenuItem);
        addCommand(composeCommand);
        disableCommand(deleteCommand);
        disableCommand(flagCommand);
        addSeparator(COMPOSE_CMD_PRIORITY + 1);

        // Add folders submenu
        gotoMenuItem = getGotoSubMenu();
        addMenuItem(gotoMenuItem);

        // Add settings submenu
        settingsMenuItem = getSettingsSubMenu();
        addMenuItem(settingsMenuItem);

        addSeparator(SETTINGS_CMD_PRIORITY + 1);
        addCommand(aboutCommand);
        addSeparator(ABOUT_CMD_PRIORITY + 1);

        if (!UIController.isOfflineMode() && UpdaterManager.getInstance().isUpdateAvailable()) {
            addCommand(updateCommand);
        }
        addCommand(exitCommand);
    }


    // remove the commands e.g. when we need to go in ad mode
    protected void removeCommands() {
        Log.trace(TAG_LOG, "removeCommands");
        removeCommand(composeCommand);
        removeCommand(aboutCommand);
        removeCommand(exitCommand);
    }

    /**
     * add the ads command
     */
    /**
     * called by implementation when a list of elements with lenght >0 is set
     */
    protected void addItemCommands() {
        Log.trace(TAG_LOG, "addItemCommands");
        enableCommand(openCommand);
        enableCommand(deleteCommand);
        enableCommand(flagCommand);
        setFireCommand(openCommand);
    }

    /**
     * called by implementation when a list of elements with lenght = 0 is set
     */
    protected void removeItemCommands() {
        Log.trace(TAG_LOG, "removeItemCommands");
        disableCommand(flagCommand);
        disableCommand(openCommand);
        disableCommand(deleteCommand);
        setFireCommand(null);
    }

    public void commandAction(Command command, Displayable displayable) {
        Log.trace(TAG_LOG, "commandAction");

        if (command == aboutCommand) {
            UIController.showAbout(UIController.display.getCurrent());
        } else if (command == composeCommand) {
            UIController.composeNewMessage(UIController.display.getCurrent());
        } else if (command == exitCommand) {
            UIController.midlet.destroyApp(false);
        } else if (command == syncCommand) {
            UIController.setSyncCaller(UIController.USER);
            UIController.sync(UIController.isFirstSync);
        } // ---- item commands --- //
        else if (command == openCommand) {
            if (getSelectedMessage() != null) {
                // Use method getFunTitle instead of getTitle
                // for double titlebar in Nokia S40
                String origTitle = getFunTitle();
                setTitle(Localization.getMessages().OPENING_MESSAGE);
                this.repaint();
                this.serviceRepaints();
                UIController.showViewMessageScreen(this, getSelectedMessage(),false);
                setTitle(origTitle);
            }
        } else if (command == flagCommand) {
            if (getSelectedMessage() != null) {
                UIController.changeFollowUpFlag(getSelectedMessage());
                activate(getActiveElementId());
            }
        } else if (command == deleteCommand) {
            deleteMessage();
        } else if (command == accountSettingsCommand) {
            UIController.showAccountSettings(this);
        } else if (command == generalSettingsCommand) {
            UIController.showMailSyncSettings(this);
        } else if (command == loggingCommand) {
            UIController.showLogViewer(this);
        } else if (command == resetCommand) {
            // Reset Inbox
            UIController.resetInboxWithConfirmation();
        } else if (command == backgroundCommand ||
                   command == UIController.lowLevelBackCommand) {
            UIController.background();
        } else if (command == updateCommand) {
            UIController.update();
        } else if(command instanceof AccountCommand &&
                command.getCommandType() == AccountCommand.ACCOUNT_COMMAND_TYPE) {
            // Change the title right away, so the user understand what's going on
            // A new account has been selected
            setTitle(Localization.getMessages().INBOX_SWITCHING);
            UIController.setCurrentMailAccount(((AccountCommand)command).getAccount(), this);
        }  else {
            boolean handled = false;
            // Check if this is a folder/account command
            if (folderCommand != null) {
                for(int i=0;i<folderCommand.length;++i) {
                    if (command == folderCommand[i]) {
                        switchToFolder(i);
                        handled = true;
                        break;
                    }
                }
            }
            if (!handled) {
                super.commandAction(command, displayable);
            }
        }
    }


    private void switchToFolder(int folderIndex) {
        switch (folderIndex) {
            case 0:
                UIController.showOutboxScreen(this);
                break;
            case 1:
                UIController.showDraftScreen(this);
                break;
            case 2:
                UIController.showSentScreen(this);
                break;
        }
    }

    protected void initCommands() {
        Log.trace(TAG_LOG, "initCommands");
        
        //Command priority must give the following sequence:
        //View
        //Get Mail
        //---
        //Delete
        //Flag
        //Compose
        //---
        //Go to >
        //Settings >
        //---
        //About
        //---
        //Hide (where applicable is on the soft key and not in this menu)
        //Exit
       
        int screenCmdType = Command.BACK;
        //#ifdef UseOKCmd
        //#         screenCmdType = UIController.COMMAND_TYPE;
        //#endif

        openCommand =
                new Command(Localization.getMessages().OPEN_COMMAND_LABEL,
                UIController.COMMAND_TYPE, OPEN_CMD_PRIORITY);

        composeCommand =
                new Command(Localization.getMessages().COMPOSE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, COMPOSE_CMD_PRIORITY);

        aboutCommand =
                new Command(Localization.getMessages().ABOUT_COMMAND_LABEL,
                UIController.COMMAND_TYPE, ABOUT_CMD_PRIORITY);

        exitCommand =
                new Command(Localization.getMessages().EXIT_COMMAND_LABEL,
                UIController.COMMAND_TYPE, EXIT_CMD_PRIORITY);
        syncCommand = new Command(Localization.getMessages().SYNC_COMMAND_LABEL, 
                UIController.COMMAND_TYPE, SYNC_CMD_PRIORITY);

        deleteCommand =
                new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, DELETE_CMD_PRIORITY);
        deleteMenuItem = new MenuItem(deleteCommand,"[0]",null);
        deleteMenuItem.setComandInfoFgColor(GraphicalThemeFactory.getGraphicalTheme().getShortcutsFgColor());

        flagCommand =
                new Command(Localization.getMessages().FLAG_COMMAND_LABEL,
                UIController.COMMAND_TYPE, FLAG_CMD_PRIORITY);

        // For this command we need the corresponding menu item to change its
        // text and properties
        flagMenuItem = new MenuItem(flagCommand,"[*]",null);
        flagMenuItem.setComandInfoFgColor(GraphicalThemeFactory.getGraphicalTheme().getShortcutsFgColor());

        accountSettingsCommand =
                new Command(Localization.getMessages().ACCOUNT_COMMAND_LABEL,
                UIController.COMMAND_TYPE, ACCOUNT_SETTINGS_CMD_PRIORITY);
        generalSettingsCommand =
                new Command(Localization.getMessages().GENERAL_COMMAND_LABEL,
                UIController.COMMAND_TYPE, GENERAL_SETTINGS_CMD_PRIORITY);
        loggingCommand =
                new Command(Localization.getMessages().LOGGING_COMMAND_LABEL,
                UIController.COMMAND_TYPE, LOGGING_CMD_PRIORITY);
        resetCommand =
                new Command(Localization.getMessages().RESET_COMMAND_LABEL,
                UIController.COMMAND_TYPE, RESET_CMD_PRIORITY);

        updateCommand =
                new Command(Localization.getMessages().UPDATE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, UPDATE_CMD_PRIORITY);

        //#ifdef isBlackberry
        //# backgroundCommand = new Command(Localization.getMessages().STD_BACKGROUND_COMMAND, Command.BACK, HIDE_CMD_PRIORITY);
        //#else
        backgroundCommand = new Command(Localization.getMessages().STD_BACKGROUND_COMMAND, screenCmdType, HIDE_CMD_PRIORITY);
        //#endif
    }

    public void load() {
        Log.trace(TAG_LOG, "load");
        /* displayInboxMessageList(false); */

        // loading opened and closed image, they'll be probably needed later and
        // this minimize the empty inbox for devices like nokia 6630
        Theme.getImageFromFlags(MessageFlags.OPENED);
        Theme.getImageFromFlags(0);

        isLoadingMessages = true;
        try {
            String[] s = {Localization.getMessages().INBOX_LOADING, " "};
            UIController.midlet.setSplashPhase(s, 0);
            
            Folder[] inboxes = UIController.messageManager.getFoldersFromRole(FolderData.INBOX_ROLE);
            for(int j=0; j<inboxes.length; j++) {

                // Load the messages of the current inbox
                Vector messages = UIController.messageManager.getMessagesInFolder(inboxes[j], this);

                if(((AccountFolder)UIController.messageManager.getFolderParent(
                        inboxes[j])).equals(UIController.getCurrentMailAccount())) {
                    setMessageList(messages);
                    UIController.resortInboxMessages();
                }
                AccountFolder f = (AccountFolder)UIController.messageManager.getFolderParent(inboxes[j]);
                String emailAddress = f.getEmailAddress(); 
                allMessages.put(emailAddress, messages);
            }
            UIController.midlet.setSplashPhase("Opening Inbox", 0);
        } catch (Exception e) {
            // can happen if a message is deleted while we're in the  cycle
            Log.error(TAG_LOG, "Exception loading messages from inbox: " + e.toString());
            e.printStackTrace();
        } finally {
            setMinimumDisplayedMessageCount(COUNT_NONE);
            rewindList();
            UIController.display.setCurrent(this);
            isLoadingMessages = false;
        }
    }

    /**
     * @see MessagesLoaderListener#messageLoaded(int, int) 
     */
    public void messageLoaded(int count, int total) {
        String[] s = {Localization.getMessages().INBOX_LOADING,
            "(" + count + "/" + total + ")"};
        UIController.midlet.setSplashPhase(s, 0);
    }

    /**
     * @see UnreadMessagesListener#updateUnreadMessages(com.funambol.mail.Folder, int) 
     */
    public void updateUnreadMessages(Folder folder, int count) {
        if(folder.getRole().equals(FolderData.INBOX_ROLE) &&
                ((AccountFolder)UIController.messageManager.getFolderParent(folder)).equals(
                UIController.getCurrentMailAccount())) {
            if (!UIController.isSyncInProgress()) {
                setTitle(getDefaultTitle(count));
            }
        }
    }

    public void switchAccount() {
        // If we are already switching account, we ignore this event without
        // blocking the UI
        if (!accountSwitcher.isSwitching()) {
            // Perform the action in a separate thread so the UI does not hang
            // up. We need the switcher to pause at the very beginning for the
            // given amount of time, so that the commandAction can terminate and
            // the UI gets refreshed with the new title
            accountSwitcher = new AccountSwitcher(200);
            //accountSwitcher.setPriority(Thread.MIN_PRIORITY);
            accountSwitcher.start();
        }
    }
    
    public void setLastSelectedAccountMessage(String emailAddress, Message m) {
        SelectedMessage sm = new SelectedMessage(m);
        lastSelectedAccountMessage.put(emailAddress, sm);
    }

    private void setCurrentAccountSelectedMessagePosition(SelectedMessage sm) {
        //The Message must exist in the list
        int currentMessageIndex = sm.getCurrentMessageIndex();
        if (currentMessageIndex >= 0) {
            if (currentMessageIndex != sm.lastSelectedMessageIndex) {
                //if the message exists on the list but its position changed 
                //after the last switch account action, due to a sync add or
                //delete message action, then firstVisible and  lastVisible must
                //be recalculated.
                int delta = currentMessageIndex - sm.lastSelectedMessageIndex;

                firstVisible += delta;
                lastVisible += delta;

                if (firstVisible<0) {
                    firstVisible = 0;
                }

                if (lastVisible<0) {
                    lastVisible = 0;
                }

                if (getMessageList().size() > 0) {
                    setActiveElement(currentMessageIndex);
                }

            } else {
                firstVisible = sm.firstVisibleElement;
                lastVisible = sm.lastVisibleElement;
                if (getMessageList().size() > 0) {
                    setActiveElement(sm.lastSelectedMessageIndex);
                }
            }
        } else {
            rewindList();
        }
    }

    private class SelectedMessage {
        protected Message m = null;
        protected int firstVisibleElement = 0;
        protected int lastVisibleElement = 0;
        protected int lastSelectedMessageIndex = 0;

        public SelectedMessage(Message m) {
            this.m = m;
            this.firstVisibleElement = firstVisible;
            this.lastVisibleElement = lastVisible;
            this.lastSelectedMessageIndex = getMessageList().indexOf(m);
        }
        
        protected int getCurrentMessageIndex() {
            return getMessageList().indexOf(m);
        }
    }

    private MenuItem getGotoSubMenu() {
        AccountFolder[] accounts = UIController.messageManager.getAccounts();
        Menu gotoSubMenu = new Menu(Localization.getMessages().GOTO_COMMAND_LABEL);
        int priority = 0;
        if(accounts.length > 1) {
            for(int i=0; i<accounts.length; i++) {
                Image icon = null;
                if(UIController.getCurrentMailAccount().equals(accounts[i])){
                    icon = Theme.getSELECT_IMG();
                }
                gotoSubMenu.addMenuItem(new MenuItem(
                        new AccountCommand(accounts[i], priority),
                        "", icon));
                priority++;
            }
            // Add separator
            gotoSubMenu.addMenuItem(new MenuItem(priority));
        }

        // Add the standard folders goto commands
        folderCommand = new Command[3];
        folderCommand[0] =
                new Command(Localization.getMessages().OUTBOX_MESSAGE_LIST_TITLE,
                Command.SCREEN, priority+1);
        folderCommand[1] =
                new Command(Localization.getMessages().DRAFT_MESSAGE_LIST_TITLE,
                Command.SCREEN, priority+2);
        folderCommand[2] =
                new Command(Localization.getMessages().SENT_MESSAGES_LIST_TITLE,
                Command.SCREEN, priority+3);
        
        gotoSubMenu.addMenuItem(new MenuItem(folderCommand[0]));
        gotoSubMenu.addMenuItem(new MenuItem(folderCommand[1]));
        gotoSubMenu.addMenuItem(new MenuItem(folderCommand[2]));

        return new MenuItem(gotoSubMenu, GOTO_CMD_PRIORITY);
    }

    private MenuItem getSettingsSubMenu() {
        Menu settingsSubMenu = new Menu(Localization.getMessages().SETTINGS_COMMAND_LABEL);
        settingsSubMenu.addMenuItem(new MenuItem(accountSettingsCommand));
        settingsSubMenu.addMenuItem(new MenuItem(generalSettingsCommand));
        settingsSubMenu.addMenuItem(new MenuItem(GENERAL_SETTINGS_CMD_PRIORITY+1));
        settingsSubMenu.addMenuItem(new MenuItem(loggingCommand));
        settingsSubMenu.addMenuItem(new MenuItem(resetCommand));
        return new MenuItem(settingsSubMenu, SETTINGS_CMD_PRIORITY);
    }

    private void loadCurrentAccountMessages(AccountFolder currentAccount) {
        Folder currentInbox = UIController.messageManager.getAccountFolderFromRole(
                currentAccount, FolderData.INBOX_ROLE);
        Vector messages = UIController.messageManager.getMessagesInFolder(currentInbox, this);
        setMessageList(messages);
        UIController.resortInboxMessages();
        allMessages.put(((AccountFolder)UIController.messageManager.getFolderParent(currentInbox)).getEmailAddress(), messages);
    }

    protected void handleKey(int keyCode) {
        Log.trace(TAG_LOG, "handleKey");
        // handle the shortcut to flag a message with KEY_STAR key (*)
        Log.debug(TAG_LOG, "keyCode= " + keyCode + " - getGameAction(keyCode)= "
                           + getGameAction(keyCode));
        if (keyCode == KEY_STAR) {
            commandAction(flagCommand, this);
        } else if (UIController.isClearKey(keyCode)) {
            deleteMessage();
        } else if (keyCode == -7) {
//#ifdef isBlackberry
//#         commandAction(UIController.lowLevelBackCommand, this);
//#endif
        }
        int dir = getGameAction(keyCode);
        if (dir == Canvas.RIGHT) {
            UIController.nextMailAccount(this);
        } else if (dir == Canvas.LEFT) {
            UIController.previousMailAccount(this);
        }
        super.handleKey( keyCode );
    }

    protected void handleKeyPressed(int keyCode) {
        Log.trace(TAG_LOG, "[InboxMessageList.keyPressed]");
        if (!UIController.isSyncInProgress() && syncEnded == true) {
            resetTitle();
            syncEnded = false;
        }
        super.handleKeyPressed(keyCode);
    }

    /**
     * Tells the InboxMessageList that a sync session ended.
     */
    public void setSyncEnded() {
        syncEnded = true;
    }

    protected void activate(int activeElement) {
        Log.trace(TAG_LOG, "Active element: " + activeElement);
        super.activate(activeElement);
        boolean activeMessageFlag = false;

        Message m = getSelectedMessage();
        if (m != null) {
            activeMessageFlag = m.getFlags().isSet(MessageFlags.FLAGGED);
            if (activeMessageFlag) {
                flagMenuItem.setText(Localization.getMessages().UNFLAG_COMMAND_LABEL);
            } else {
                flagMenuItem.setText(Localization.getMessages().FLAG_COMMAND_LABEL);
            }
        }
    }

    public void updateMessageFlags(String msgId, MessageFlags newFlags) {
        int p = search(msgId);
        if (p >= 0) {
            Message msg = getMessageAt(p);
            updateMessageFlags(msgId, newFlags, msg.getFlags());
        }
    }

    public void updateMessageFlags(String msgId, MessageFlags newFlags,
            MessageFlags oldFlags) {
        int p = search(msgId);
        if (p >= 0) {
            Message msg = getMessageAt(p);
            // Log.debug(TAG_LOG, "updating message flags for message " + msgId);
            msg.setFlags(newFlags);
            repaint();
        } else {
            updateOnOtherAccounts(msgId, newFlags, oldFlags);
        }
    }

    private void updateOnOtherAccounts(String msgId, MessageFlags newFlags,
            MessageFlags oldFlags) {
        Enumeration accounts = allMessages.keys();
        while (accounts.hasMoreElements()) {
            String acc = (String) accounts.nextElement();
            if(!acc.equals(UIController.getCurrentMailAccount().getEmailAddress())){
                Vector emails = (Vector) allMessages.get(acc);
                for(int i =0; i<emails.size();i++){
                    if(((Message)emails.elementAt(i)).getKey().equals(msgId)){
                        Message msg = (Message)emails.elementAt(i);
                        msg.setFlags(newFlags);
                        emails.setElementAt(msg, i);
                        break;
                    }
                }
            }
        }
    }

    /**
     * Enable or disable the sync command and the reset inbox command
     * @param enable if true enable the command, if false disable the commands
     */
    public void enableSyncCommand(boolean enable) {
        Log.debug(TAG_LOG, "enableSyncCommand Value: " + enable);
        if (enable) {
            enableCommand(syncCommand);
            enableCommand(resetCommand);
        } else {
            disableCommand(syncCommand);
            disableCommand(resetCommand);
        }
    }

    public void enableUpdateCommand(boolean enable) {
        Log.debug(TAG_LOG, "enableUpdateCommand Value: " + enable);
        if (enable) {
            addCommand(updateCommand);
        } else {
            removeCommand(updateCommand);
        }
    }

    /**
     * @return true if the inbox is loading messages 
     */
    public boolean isLoadingMessages() {
        Log.trace(TAG_LOG, "isLoadingMessages");
        return isLoadingMessages;
    }

    protected void setElements(Vector messages) {
        Log.trace(TAG_LOG, "setElements");
        super.setElements(messages);
    }

    private void  addMessageFromRMS(Message msg) {
        Log.trace(TAG_LOG, "addMessage");
        super.addMessage(msg);
    }

    public void addMessage(Message msg) {
        Log.trace(TAG_LOG, "addMessage");
        AccountFolder currentAccount = UIController.getCurrentMailAccount();
        AccountFolder msgAccount = (AccountFolder)UIController.messageManager.getFolderParent(
                msg.getParent());
        
        addMsgToAccount(msg, msgAccount);
        if(msgAccount.equals(currentAccount)){
            super.addMessage(msg);
        }
    }

     private void addMsgToAccount(Message msg,AccountFolder msgAccount){
        Vector messages = (Vector) allMessages.get(msgAccount.getEmailAddress());
        int pos = -1;
        for (int i = 0; i < messages.size(); i++) {
            if (((Message)messages.elementAt(i)).getKey().equals(msg.getKey())) {
                pos = i;
                break;
            }
        }

        if (pos >= 0) {
            messages.setElementAt(msg, pos);

        }else{

            // if we're here we need to add the message to the list in the
            // right position
            int i = 0;
            while (i < messages.size() && (msg.getReceivedTime() <
                   ((Message) messages.elementAt(i)).getReceivedTime())) {
                i++;
            }

            // we've found the right position.
            if (i == messages.size()) {
                //it's an append
                messages.addElement(msg);
            } else {
                //it's an insert
                messages.insertElementAt(msg, i);
            }
        }
    }

     /**
     * Delete the message with the given msgID from the elements array, if present
     * this method does not delete the message from the rms!
     *
     * @param msgId the messageid of the message to be deleted
     */
    public void deleteMessage(String msgId) {
        // Remove the message from the UI
        int p = search(msgId);
        if(p>=0){
            deleteMessageAt(p);
        } else {
            deleteOnOtherAccounts(msgId);
        }
    }

    private void deleteOnOtherAccounts(String msgId) {
        Enumeration accounts = allMessages.keys();
        while (accounts.hasMoreElements()) {
            String acc = (String) accounts.nextElement();
            if(!acc.equals(UIController.getCurrentMailAccount().getEmailAddress())){
                Vector emails = (Vector) allMessages.get(acc);
                for(int i =0; i<emails.size();i++) {
                    if(((Message)emails.elementAt(i)).getKey().equals(msgId)) {
                        emails.removeElementAt(i);
                        return;
                    }
                }
            }
        }
    }

    protected void deleteMessageAt(int position) {
        Log.trace(TAG_LOG, "Deleting at position: " + position);
        // Remove the message from the all messages
        CanvasMessageItem msg = (CanvasMessageItem) elements.elementAt(position);
        String key = msg.message.getKey();

        Enumeration accounts = allMessages.keys();
        Vector emails = null;
        while (accounts.hasMoreElements()) {
            String acc = (String) accounts.nextElement();
            if(acc.equals(UIController.getCurrentMailAccount().getEmailAddress())){
                emails = (Vector) allMessages.get(acc);
                break;
            }
        }
        if (emails != null) {
            for(int i =0; i<emails.size();i++) {
                if(((Message)emails.elementAt(i)).getKey().equals(key)) {
                    emails.removeElementAt(i);
                    break;
                }
            }
        }

        // Finally update the UI
        int selected = super.getActiveElementId();
        if (position<selected && selected !=0) {
            super.setActiveElement(selected - 1);
        }
        super.deleteMessageAt(position);
    }

    public void paint(Graphics graphics) {
        Log.trace(TAG_LOG, "paint");
        // Since we are repainting the inbox, we consider messages as being read
        UserNotificationManager.getInstance().hideNewEmailIcon();
        super.paint(graphics);
    }

    /**
     * Get the number of unread messages belonging to the inbox of the given
     * account.
     * 
     * @param account
     * @return
     */
    private int getUnreadMessages(AccountFolder account) {
        try {
            return UIController.messageManager.getUnreadMessagesCount(account);
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot get unread messages count");
            return 0;
        }
    }

    /**
     * Get the number of unread messages belonging to the inbox of the default
     * account.
     *
     * @return
     */
    private int getUnreadMessages() {
        AccountFolder account = UIController.getCurrentMailAccount();
        if(account != null) {
            return getUnreadMessages(account);
        }
        return 0;
    }

    protected void openSubMenu(MenuItem item, OpenedMenuStatus parent) {

        if(item == gotoMenuItem){
            Menu menu = item.getSubMenu();
            // Update accounts menu items counter
            AccountFolder[] accounts = UIController.messageManager.getAccounts();
            if(accounts.length > 1) {
                for(int i = 0; i < accounts.length; i++) {
                    MenuItem mItem = menu.getMenuItem(i);
                    int unreadMsg = getUnreadMessages(accounts[i]);
                    String unreadString = "";
                    if(unreadMsg > 0) {
                        unreadString = "("+unreadMsg+")";
                    }
                    mItem.setCommandInfo(unreadString);
                }
            }
        }
        super.openSubMenu(item, parent);
    }

    private class AccountSwitcher extends Thread {

        private boolean switching = false;
        private int delay;

        public AccountSwitcher(int delay) {
            this.delay = delay;
        }

        public boolean isSwitching() {
            return switching;
        }

        public void run() {

            // Change the title right away, so the user understand what's going on
            setTitle(Localization.getMessages().INBOX_SWITCHING);
            
            try {
                switching = true;
                Thread.sleep(delay);
                AccountFolder currentAccount = UIController.getCurrentMailAccount();
                Vector messages = (Vector)allMessages.get(currentAccount.getEmailAddress());

                Log.timeStats("Starting to switch account");

                if(messages != null) {
                    // The messages are already in the messages cache
                    setMessageList(messages);
                    UIController.resortInboxMessages();
                } else {
                    // The messages for the new account are not preloaded in the cache
                    // so we have to load them and replace the cache
                    loadCurrentAccountMessages(currentAccount);
                }

                //Get the last active message for this account in order to keep track
                //of the user message selection before the last switch action
                if (lastSelectedAccountMessage.isEmpty()) {
                    rewindList();
                } else {
                    String emailAddress = UIController.getCurrentMailAccount().getEmailAddress();
                    SelectedMessage sm = (SelectedMessage) lastSelectedAccountMessage.get(emailAddress);
                    if (sm != null) {
                        setCurrentAccountSelectedMessagePosition(sm);
                    } else {
                        rewindList();
                    }
                }

                // Update go to submenu with the new account selected (move the checkmark)
                gotoMenuItem = getGotoSubMenu();
                updateMenuItem(gotoMenuItem);
                Log.timeStats("Account switched");
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot switch account " + e.toString());
            } finally {
                switching = false;
                // Update the title bar
                resetTitle();
            }
        }
    }

}
