/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.ui.view;

import java.util.Vector;

import com.funambol.util.Log;

class Menu {
    // A menu contains an ordered sequence of menu items
    private Vector items = new Vector();
    private String   title = null;

    public Menu(String title) {
        this.title = title;
    }

    public void addMenuItem(MenuItem item) {
        items.addElement(item);
    }

    // This is temporary. We shall be able to remove by item, but until the
    // MenuItem is public we cannot. It also does not work with separators
    public void removeMenuItem(MenuItem item) {
        for(int i=0;i<items.size();++i) {
            MenuItem otherItem = (MenuItem) items.elementAt(i);
            if (otherItem.getCommand() == item.getCommand() && item.getCommand() != null) {
                items.removeElementAt(i);
                break;
            }
        }
    }

    // This is temporary. We shall be able to remove by item, but until the
    // MenuItem is public we cannot. It also does not work with separators
    public void setEnabled(MenuItem item, boolean enabled) {
        for(int i=0;i<items.size();++i) {
            MenuItem otherItem = (MenuItem) items.elementAt(i);
            if (otherItem.getCommand() == item.getCommand() && item.getCommand() != null) {
                if (enabled) {
                    otherItem.enable();
                } else {
                    otherItem.disable();
                }
                break;
            }
        }
    }


    public int totalNumberOfItems() {
        return items.size();
    }

    public int numberOfSeparators() {
        int count = 0;
        for(int i=0;i<items.size();++i) {
            MenuItem item = (MenuItem) items.elementAt(i);
            if (item.isSeparator()) {
                count++;
            }
        }
        return count;
    }

    public MenuItem getMenuItem(int idx) {
        if (idx >= items.size()) {
            throw new IllegalArgumentException("Invalid menu item index");
        }
        return (MenuItem)items.elementAt(idx);
    }

    public String getTitle() {
        return title;
    }

}


