/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.ui.view;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Image;


class MenuItem {
    private boolean separator = false;
    private String text = null;
    private Command command = null;
    private boolean enabled = true;
    private Menu subMenu = null;
    private int priority = 0;
    private String commandInfo ="";
    private Image icon = null;
    private int commandInfoFgColor = GraphicalThemeFactory.getGraphicalTheme().getMenuItemFgColor();

    /**
     * Creates an item from the given command (the item priority is inherited
     * from the command priority)
     */
    public MenuItem(Command command) {
        this.command = command;
        priority = command.getPriority();
    }

    /**
     * Creates an item from the given command (the item priority is inherited
     * from the command priority) adding more info for the command (e.g
     * shortcut) and a icon.
     */
    public MenuItem(Command command, String commandInfo, Image icon) {
        this.command = command;
        priority = command.getPriority();
        this.commandInfo = commandInfo;
        this.icon = icon;

    }


    /**
     * Creates a separator
     */
    public MenuItem(int priority) {
        separator = true;
        enabled = false;
        this.priority = priority;
    }

    /**
     * Creates an item with a submenu. The test label associated to this item is
     * the menu title.
     */
    public MenuItem(Menu subMenu, int priority) {
        this.subMenu = subMenu;
        this.priority = priority;
    }

    public int getPriority() {
        return priority;
    }

    /**
     * Returns true for separators
     */
    public boolean isSeparator() {
        return separator;
    }

    /**
     * Returns true for sub menus
     */
    public boolean isSubMenu() {
        return subMenu != null;
    }

    /**
     * Get the text associated to this item. A menu item can have its text
     * specified in several ways:
     *
     * <li>
     *   <ul> setText sets the text for this item, regardless of its type
     *   <ul> commands label is used for commands if the text property is unset
     *   <ul> sub menus use the menu title if the text propery is unset
     *   <ul> separators have a default null text
     * </li>
     */
    public String getText() {
        if (text != null) {
            return text;
        } else if (command != null) {
            return command.getLabel();
        } else if (subMenu != null) {
            return subMenu.getTitle();
        } else {
            return null;
        }
    }

    public Image getIcon(){
        return icon;
    }

    public String getCommandInfo(){
        return commandInfo;
    }

    public void setCommandInfo(String info){
        commandInfo = info;
    }

    public void setText(String text) {
        this.text = text;
    }

    public Menu getSubMenu() {
        return subMenu;
    }

    public Command getCommand() {
        return command;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void enable() {
        enabled = true;
    }

    public void disable() {
        enabled = false;
    }

    public void setComandInfoFgColor(int color){
        commandInfoFgColor = color;
    }

    public int getComandInfoFgColor(){
        return commandInfoFgColor;
    }
}

