/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 */

package com.funambol.mailclient.ui.view;

import java.util.Vector;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import com.funambol.mail.Address;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.AccountFolder;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.syncml.FolderData;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;

/**
 * This class represents a generic list of Message (mail messages).
 */
public abstract class MessageList extends List implements CommandListener {

    private static final String TAG_LOG = "MessageList";

    /** The margin */
    private static final int MARGIN = 3;

    /** Constant for setting no minimal displayed message count */
    public static final int COUNT_NONE = -1;

    private static Font font;
    private static Font boldFont;
    private static Font titleFont;
    private static Drawer drawer;
    private int minimumDisplayedMessageCount = -1;
    public int screenWidth;

    /**
     * Represents the role of the folder to be displayed in this MessageList.
     * It must be set in all the subclasses
     */
    protected String role = null;

    /**
     * Create a new MessageList
     */
    public MessageList(Vector elements) {
        super(null);
        Log.trace(TAG_LOG, "[MessageList]");
        setMessageList(elements);
        this.drawer = UIController.getDrawer();
        this.font = drawer.getGraphicalTheme().getDefaultFont();
        this.boldFont = drawer.getGraphicalTheme().getBoldFont();
        this.titleFont = drawer.getGraphicalTheme().getTitleFont();
        resetTitle();
        setCommandListener(this);
        screenWidth = getWidth();
    }

    /**
     * Set the message list, and repaint the message list
     * @param msgList a vector of Message elements
     */
    public void setMessageList(Vector msgList) {
        Log.trace(TAG_LOG, "setMessageList");
        synchronized (elements) {
            elements.removeAllElements();
            setElements(createCanvasMessages(msgList));
        }
        repaint();
    }

    /**
     * @return the num of messages
     **/
    public int getNumMessages() {
        Log.trace(TAG_LOG, "getNumMessages");
        return elements.size();
    }

    /**
     * Set the minimum displayed message count.
     * if set to COUNT_NONE the number of elements in the message
     * list is displayed
     *
     * @param i the minimum message count to be returned by
     * getDisplayedMessageCount. i must be >=0 or COUNT_NONE
     *
     */
    public void setMinimumDisplayedMessageCount(int i) {
        Log.trace(TAG_LOG, "setMinimumDisplayedMessageCount");
        this.minimumDisplayedMessageCount = i < 0 ? COUNT_NONE : i;
    }

    /**
     * @return the selected message info
     */
    public Message getSelectedMessage() {
        Log.trace(TAG_LOG, "getSelectedMessage");
        // We need a lock here because the elements may be manipulated by others
        // and the active elements may no longer exist when we try to fetch it
        synchronized(elements) {
            if (getActiveElementId() < elements.size() && getActiveElementId() >= 0) {
                CanvasMessageItem item = (CanvasMessageItem)elements.elementAt(getActiveElementId());
                return item.message;
            } else {
                return null;
            }
        }
    }

    /**
     * Get the first visible element index
     * @return int the first visible list element
     */
    public int getFirstVisible() {
        return this.firstVisible;
    }

    /**
     * Get the last visible element index
     * @return int the last visible list element
     */
    public int getLastVisible() {
        return this.lastVisible;
    }

    /**
     * Set the current account selected message.
     * @param emailAddress the current account's String formatted email address
     * @param m the current account selected message reference
     */
    public void setLastSelectedAccountMessage(String emailAddress, Message m) {
    }

    /**
     * Handle command action
     */
    public void commandAction(Command command, Displayable displayable) {
        Log.trace(TAG_LOG, "commandAction");
        if (command == UIController.backCommand || command == UIController.lowLevelBackCommand) {
            UIController.showBackScreen();
        } else {
            super.commandAction(command, displayable);
        }
    }

    /**
     * Paint the title and return the height of the painted area
     */
    public int paintTitle(Graphics g) {
        String counterSubstring = null;

        if (title != null) {
            //Set the message counter
            title.enableCounterSubstring(true);
            title.setCounterSubstring(getCounterSubstring());

            return title.paint(g, getWidth(), getHeight());
        }

        Log.trace(TAG_LOG, "paintTitle");

        // Use method getFunTitle instead of getTitle
        // for double titlebar in Nokia S40
        String title = getFunTitle();

        // keep this because otherwhise some motorola phones may throw a
        // nullpointer exception
        if (title == null) {
            //Log.debug(TAG_LOG, "title null... setting to empty string");
            title = "";
        }

        counterSubstring = getCounterSubstring();

        int sWidh = titleFont.stringWidth(counterSubstring);

        String shortTitle = UiUtils.cutString(title, getWidth() - sWidh - 1, titleFont);
        drawer.drawTitleBackground(g, getWidth(), titleFont.getHeight());

        g.setFont(titleFont);
        g.setColor(drawer.getGraphicalTheme().getTitleColor());
        //not using drawTitle because we need to draw also the number of messages
        g.drawString(shortTitle, 1, 1, Graphics.TOP | Graphics.LEFT);
        g.drawString(counterSubstring, getWidth(), 0, Graphics.TOP | Graphics.RIGHT);
        return drawer.getGraphicalTheme().getTitleFont().getHeight();
    }

    private String getCounterSubstring() {
        String s = " ";
        if (elements.size() > 0) {
            s += (getActiveElementId() + 1) + "/";
        }
        if (this.getDisplayedMessageCount() > 0) {
            s += this.getDisplayedMessageCount();
        } else {
            s = " " + this.getDisplayedMessageCount();
        }
        return s;
    }

    /**
     * This method return the number of messages as shown in the title bar
     * if setminimumdisplayedmessagecount has been called, the returned
     * value is the max between minimumumDisplayedMessageCount (that can be
     * set using the setMinimumDisplayedMessageCount method) and the size of 
     * the message vector
     */
    public int getDisplayedMessageCount() {
        Log.trace(TAG_LOG, "getDisplayedMessageCount");
        return minimumDisplayedMessageCount < elements.size() ? elements.size()
                : minimumDisplayedMessageCount;
    }

    /**
     * Return the default title
     */
    public String getDefaultTitle() {
        Log.trace(TAG_LOG, "getDefaultTitle");
        return "";
    }

    /**
     * Reset the title to the default title
     */
    public void resetTitle() {
        Log.trace(TAG_LOG, "resetTitle");
        setTitle(this.getDefaultTitle());
    }

    /**
     * Add a message to the current list of messages
     * @param message the message to be added
     */
    public void addMessage(Message message) {
        Log.trace(TAG_LOG, "addMessage");
        synchronized(elements) {
            if (elements.size() == 0) {
                // This is the first message, we can use setMessageList
                Vector v = new Vector();
                v.addElement(message);
                setMessageList(v);
                //repaint();
                return;
            }
            int pos = search(message.getKey());
            if (pos >= 0) {
                // we overwrite previous messageinfo and canvasmessageitem's message
                ((CanvasMessageItem) elements.elementAt(pos)).message = message;
                repaint();
                return;
            }

            // if we're here we need to add the message to the list in the
            // right position
            int i = 0;
            while (i < elements.size() &&
                    (message.getReceivedTime() < getMessageAt(i).getReceivedTime()))
            {
                i++;
            }

            // we've found the right position.
            if (i == elements.size()) {
                //it's an append
                elements.addElement(new CanvasMessageItem(message, i));
            } else {
                //it's an insert
                increaseFirstVisibleIfNeeded(i);
                elements.insertElementAt(new CanvasMessageItem(message, i), i);

                // we change the id of each following item
                for (int j = i + 1; j < this.size(); j++) {
                    ((CanvasMessageItem) elements.elementAt(j)).setId(j);
                }
            }
            if (i <= getActiveElementId()) {
                setActiveElement(getActiveElementId()+1);
            }
            repaint();
            // this can happen sometimes, we need a double increase.
            if (getActiveElementId() > lastVisible +1 )  {   
                firstVisible++;
                repaint();
            }
        }
    }

    /**
     * Handle the account switch.
     * Default implementation updates the current message list by loading the
     * messages for the new account.
     */
    public void switchAccount() {
        UIController.updateMessageList(null, this.role);
    }

    /**
     * Paint the element
     */
    protected int paintElement(int elementId, Graphics g) {
        Log.trace(TAG_LOG, "paintElement");
        // this should not happen, but shomehow it happens on some
        // nokias (like 6111)
        if (elementId >= elements.size()) {
            return 0;
        }
        g.translate(0, current_paint_h);
        int h = ((CanvasMessageItem) elements.elementAt(elementId)).paint(g, elementId);
        g.translate(0, -current_paint_h);
        return h;
    }

    /**
     * This method is called by implementation when the commands need to be
     * initialized (in the constructor), should be overridden. After this element
     * the commands referenced in addCommands, addItemCommands, removeItemcommands
     * must be not null
     */
    protected void initCommands() {
        Log.trace(TAG_LOG, "initCommands");
    }

    /**
     * This method is called by implementation when the generic commands need to be
     * added (in the constructor), should be overridden.
     */
    protected void addCommands() {
        Log.trace(TAG_LOG, "addCommands");
        super.addCommands();
    }

    /**
     * @return the index of the messageinfo with the same messagid as
     * the given messageinfo, or -1 if not found.
     */
    protected int search(String msgId) {
        Log.trace(TAG_LOG, "search");
        int position = -1;
        for (int i = 0; i < elements.size(); i++) {
            if (getMessageAt(i).getKey().equals(msgId)) {
                position = i;
                break;
            }
        }
        return position;
    }

    /**
     * Handle the keypress events, should be overridden
     */
    protected void handleKey(int keyCode) {
        Log.trace(TAG_LOG, "handleKey");
    }

    protected void handleTouchEvent(FTouchEvent event) {
        Log.debug(TAG_LOG, "handleTouchEvent " + event);
        int type = event.getType();
        Log.debug(TAG_LOG, "type="+type);

        if(type==FTouchEvent.SCROLL_RIGHT){
            handleKey(Canvas.RIGHT);
        }else if(type==FTouchEvent.SCROLL_LEFT){
            handleKey(Canvas.LEFT);
        }else{
            super.handleTouchEvent(event);
        }
    }


 
    /**
     * This method is called by implementation when the item commands need to be
     * added (i.e. while setting a non empty elements array.  should be ovverridden
     */
    void removeItemCommands() {
        Log.trace(TAG_LOG, "removeItemCommands");
    }

    /**
     * This method is called by implementation when the item commands need to be
     * removed (i.e. while setting an empty elements array). should be ovverridden
     */
    void addItemCommands() {
        Log.trace(TAG_LOG, "addItemCommands");
    }

    /**
     * Paint the background
     */
    void paintBackground(Graphics graphics) {
        Log.trace(TAG_LOG, "paintBackground");
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
    }

    /**
     * Delete the message eventually after a popup
     */
    void deleteMessage() {
        Log.trace(TAG_LOG, "deleteMessage");
        Message msg = getSelectedMessage();
        if (msg != null) {
            UIController.deleteMessageWithConfirmation(msg);
        }
    }

    /**
     * Remove the message in the given position from the visualized list.
     * This function will NOT delete the message from the folder
     *
     * @param position the position of the messageinfo to be removed
     */
    protected void deleteMessageAt(int position) {
        Log.trace(TAG_LOG, "deleteMessageAt position: " + position);

        //#ifdef low_mem
//#         UIController.freeMemory();      //XXX
        //#endif

        synchronized(elements) {
            decreaseFirstVisibleIfNeeded(position);
            elements.removeElementAt(position);

            // Update messageList.
            for (int i = position; i < elements.size(); i++) {
                ((CanvasMessageItem) elements.elementAt(i)).setId(i);
            }

            // If the deleted message was the last, set the new last as
            // active.
            if (getActiveElementId() >= elements.size()) {
                setActiveElement(elements.size() - 1);
            }
        }
        repaint();
    }

    /**
     * Delete the message with the given msgID from the elements array, if present
     * this method does not delete the message from the rms!
     *
     * @param msgId the messageid of the message to be deleted
     */
    public void deleteMessage(String msgId) {
        Log.trace(TAG_LOG, "deleteMessage ID: " + msgId);
        synchronized(elements) {
            int p = search(msgId);
            if (p >= 0) {
                deleteMessageAt(p);
            }
        }
    }

    /**
     * Return the messageList as a vector of Messages
     */
    public Vector getMessageList() {
        Log.trace(TAG_LOG, "getMessageList");

        Vector res = new Vector();
        for(int i=0;i<elements.size();++i) {
            CanvasMessageItem item = (CanvasMessageItem)elements.elementAt(i);
            Message msg = item.message;
            res.addElement(msg);
        }
        return res;
    }

    /**
     * Return the message at the given position
     * @param position the position of the element to be returned
     */
    public Message getMessageAt(int position) {
        Log.trace(TAG_LOG, "getMessageAt position: " + position);
        return ((CanvasMessageItem) elements.elementAt(position)).message;
    }

    boolean isVisible(int elementId) {
        Log.trace(TAG_LOG, "isVisible element: " + elementId);
        return elements.elementAt(elementId) != null;
    }

    boolean isActivable(int elementId) {
        Log.trace(TAG_LOG, "isActivable element: " + elementId);
        return isVisible(elementId);
    }

    private Vector createCanvasMessages(Vector messages) {
        Vector res = new Vector();
        for (int i = 0; i < messages.size(); i++) {
            CanvasMessageItem item = new CanvasMessageItem((Message)messages.elementAt(i), i);
            res.addElement(item);
        }
        return res;
    }

    private void increaseFirstVisibleIfNeeded(int i) {
        Log.trace(TAG_LOG, "increaseFirstVisibleIfNeeded");
        // we need to move the first visible down if we add a message before the
        // currently active element and if we have no space left. this means that
        // we need to have the active element as the last visible element but we
        // are unable to show the entire list of messages in one screen
        
        boolean needToIncrease = false;
        needToIncrease = i< getActiveElementId() && !isEveryMessageVisible() ||
               (getActiveElementId() >= lastVisible  && lastVisible < elements.size()-1 );
          
        if (needToIncrease) {
            firstVisible++;
        }
    }
    
    private void decreaseFirstVisibleIfNeeded(int i) {
        Log.trace(TAG_LOG, "decreaseFirstVisibleIfNeeded");
        // check if we need to decrease the first visible.  
        // messages and we are not and we remove a message before the
        // currently active element and if we have no space left. this means that
        // we need to have the active element as the last visible element but we
        // are unable to show the entire list of messages in one screen
        boolean needToDecrease = false;
        needToDecrease = firstVisible > 0 &&
                !isEveryMessageVisible() &&  //this is a must-have
                (
                    // this happens if we're off-screen
                    getActiveElementId() < firstVisible ||
                    (
                        // "standard" case, we're removing a message that is
                        // currently visible and before the selected one
                        firstVisible <= i &&
                        lastVisible >=i &&
                        i <getActiveElementId()
                    ) || 
                    // if we're showing the last element we need to go up 
                    // or the screen will just show one message less and looks ugly
                    lastVisible == elements.size()-1
                );
        if (needToDecrease) {  
           firstVisible--;         
        }
    }
    
    /**
     * @return true if all the messages are visible
     */
    private boolean isEveryMessageVisible() {
        Log.trace(TAG_LOG, "isEveryMessageVisible");    
        return firstVisible == 0 && 
                lastVisible >= elements.size()-1;
    }

    /**
     * Put the given messageinfo in the right position in
     * elements and messagelist
     *
     *@param to the position where to put the messageinfo
     *@param msg the message to be added
     */
    private void assign(int to, Message msg) {
        elements.setElementAt(new CanvasMessageItem(msg, to), to);
    }




//------------------------ CanvasMessageItem ---------------------------------//

    /**
     * This class draws the message item
     */
    class CanvasMessageItem {

        private static final String TAG_LOG = "CanvasMessageItem";

        Message message;
        private int id;
        private String date;

        /** The subject that fits on a single line */
        private String shortSubject;
        
        /** The address that fits on a single line */
        private String shortAddress;
        
        /** The height of this element when not expanded */
        private int unexpanded_h;
        
        /** The height of this element when expanded */
        private int expanded_h;

        /**
         * Track if the last time we painted the item the read flag was
         * set or not (to recalculate the short strings if necessary)
         */
        private boolean lastReadFlag;
        private boolean lastFlagFlag;

        CanvasMessageItem(Message msg, int id) {
            Log.trace(TAG_LOG, "Constructor");
            this.message = msg;
            this.id = id;
        }

        private boolean isActive() {
            return id == getActiveElementId();
        }

        /**
         * Paint the messageitem
         */
        protected int paint(Graphics graphics, int elementId) {
            Log.trace(TAG_LOG, "[CanvasMessageItem.paint]");

            //we need to calculate the short strings
            int w = getWidth();
            int h = calculateHeight();
            calculateShortStrings();

            Image img = Theme.getImageFromFlags(getFlagValue());


            int mailImageH = 0;
            int textHeaderH = 0;
            int textSubjectH = 0;

            int fontH = 0;
            if (isRead()) {
                fontH = font.getHeight();
            } else {
                fontH = boldFont.getHeight();
            }

            if (isExpanded()) {
                mailImageH = h/4 - img.getHeight()/2 + MARGIN;
                textHeaderH = h/4 - fontH/2 + MARGIN;
                textSubjectH = 3*h/4 - fontH/2;
            } else {
                mailImageH = h/2 - img.getHeight()/2;
                textHeaderH = h/2 - fontH/2;
            }


            //we draw the background, image and date
            drawer.drawBackground(graphics, 0, 0, w, h, (elementId+1)%2, isActive());
            graphics.drawImage(img, MARGIN, mailImageH, Graphics.TOP | Graphics.LEFT);
            drawer.drawString(getDate(), graphics, w - MARGIN, textHeaderH, Graphics.TOP | Graphics.RIGHT, isActive());

            //if this is a read message we use normal font, otherwhise we use the bold font
            if (isRead()) {
                drawer.drawString(shortAddress, graphics, Theme.getIMAGE_PART_WIDTH() +
                        2 * MARGIN, textHeaderH, isActive());
            } else {
                drawer.drawBoldString(shortAddress, graphics, Theme.getIMAGE_PART_WIDTH() +
                        2 * MARGIN, textHeaderH, isActive());
            }
            int attachPos = 0;
            int stringWidth ;

            if (isRead()) {
                stringWidth = font.stringWidth(shortAddress);
            } else {
                stringWidth = boldFont.stringWidth(shortAddress);
            }
            attachPos = Theme.getIMAGE_PART_WIDTH() +
                    3 * MARGIN +
                    stringWidth;
            if (hasAttach()) {
                graphics.drawImage(Theme.getCLIP_IMG(), attachPos, 2*MARGIN, Graphics.TOP | Graphics.LEFT);
            }
            if (isFlagged()) {
                //draw image
                int flagPos = hasAttach() ? attachPos + Theme.getCLIP_IMG().getWidth() : attachPos;
                 graphics.drawImage(Theme.getFLAG_IMG(), flagPos  , 2*MARGIN, Graphics.TOP | Graphics.LEFT);
            }
            //if expanded we draw the second line
            if (isExpanded()) {
                drawer.drawString(shortSubject, graphics, MARGIN, textSubjectH, isActive());
            }
            return h;
        }

        /**
         * Return the flag value used to identify the image in the image array
         */
        private int getFlagValue() {
            Log.trace(TAG_LOG, "getFlagValue");
            int flags = message.getFlags().getFlags();
            //we mask the flag to be sure we have a return value <32
            flags = (flags & 0x0000001f);
            return flags;
        }

        /**
         * Calculate the short version of address and subject and
         * store them in shortAddress and shortSubject
         */
        private void calculateShortStrings() {
            Log.trace(TAG_LOG, "calculateShortStrings");

            if (shortAddress == null || shortSubject == null ||
                    lastReadFlag != isRead() || lastFlagFlag != isFlagged()) {

                int w = getWidth();
                int firstlineSpace = w - 4 * MARGIN - Theme.getIMAGE_PART_WIDTH() - font.stringWidth(getDate());
                if (hasAttach()) {
                    firstlineSpace = firstlineSpace - Theme.getCLIP_IMG().getWidth();
                }
                if (isFlagged()) {
                    firstlineSpace -= Theme.getFLAG_IMG().getWidth();
                }
                lastFlagFlag = isFlagged();

                int secondlineSpace = w - MARGIN;
                shortSubject = UiUtils.cutString(message.getSubject(), secondlineSpace, font);

                String address = null;
                if (isIncomingMessage()) {
                    Address from = message.getFrom();
                    if (from != null) {
                        address = from.getVisibleName();
                    }
                } else {
                    Address to[] = message.getTo();
                    Address cc[] = message.getCc();
                    if (to != null && to.length > 0) {
                        Address singleTo = to[0];
                        address = singleTo.getVisibleName();
                    } else if (cc != null && cc.length > 0) {
                        Address singleCc = cc[0];
                        address = singleCc.getVisibleName();
                    }
                }
                if (address == null) {
                    address = Localization.getMessages().EMPTY_RECIPIENTS;
                }
                if (isRead()) {
                    shortAddress = UiUtils.cutString(address, firstlineSpace, font);
                    lastReadFlag = true;
                } else {
                    shortAddress = UiUtils.cutString(address, firstlineSpace, boldFont);
                    lastReadFlag = false;
                }
            }
        }

        /**
         * @return true if message is in the inbox
         */
        private boolean isIncomingMessage() {
            Log.trace(TAG_LOG, "isIncomingMessage");
            return (FolderData.INBOX_ROLE.equals(message.getParent().getRole()));
        }

        /**
         * @return true if the message has been opened
         */
        private boolean isRead() {
            return message.getFlags().isSet(MessageFlags.OPENED);
        }

        /**
         * @return true if the message has one ore more attachments
         */
        private boolean hasAttach() {
            return message.isMultipart();

        }

        /**
         * @return true if the message is flagged
         */
        private boolean isFlagged() {
            return message.getFlags().isSet(MessageFlags.FLAGGED);
        }

        /**
         * @return true if this item is expanded, false otherwise
         */
        public boolean isExpanded() {
            boolean ret;
            //#ifdef MessageItemAlwaysExpanded
//#             ret = true;
            //#else
            ret = isActive();
            //#endif
            return ret;
        }

        /**
         * Compute height using information from the bean,
         * the drawer and the icon size
         */
        private int calculateHeight() {
            Log.trace(TAG_LOG, "calculateHeight");
            if (isExpanded()) {
                return getExpandedh();
            } else {
                return getUnexpandedh();
            }
        }

        /**
         * @return the height of this element not expanded
         */
        private int getUnexpandedh() {
            Log.trace(TAG_LOG, "getUnexpandedh");
            if (unexpanded_h == 0) {
                unexpanded_h = MARGIN * 2;
                unexpanded_h += Math.max(
                        Math.max(font.getHeight(), boldFont.getHeight()), Theme.getIMAGE_PART_HEIGHT());
            }
            return unexpanded_h;
        }

        /**
         * @return the height of this element expanded
         */
        private int getExpandedh() {
            Log.trace(TAG_LOG, "getExpandedh");
            if (expanded_h == 0) {
                expanded_h = getUnexpandedh();
                expanded_h += font.getHeight();
                expanded_h += MARGIN *2;
            }
            return expanded_h;
        }

        /**
         * return the date as a formatted string
         */
        private String getDate() {
            Log.trace(TAG_LOG, "getDate");
            if (date == null) {
                date = UiUtils.formatDate(message.getReceivedDate());
            }
            return date;
        }

        private void setId(int newId) {
            Log.trace(TAG_LOG, "setId");
            this.id = newId;
        }
    }
}
