/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.view.components.EditField;
import com.funambol.mailclient.ui.view.components.BodyItem;
import com.funambol.mailclient.ui.view.components.Title;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.mm.MessageManager;

import com.funambol.mail.Message;

import com.funambol.util.Log;

import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Font;

import java.util.Vector;

/**
 * Massage Screen container. Generalize the screen to view and edit messages  
 */
public abstract class MessageScreen extends CustomMenuCanvas implements CommandListener {

    /** Scroll amount for the scrollable items */
    protected static final int SCROLL_AMOUNT = 2;

    /**
     * MessageManager instance: must be always initialized with reference
     * to the UIController
     */
    protected MessageManager mm = null;

    /** The drawer for this canvas*/
    protected Drawer drawer = null;

    /** The Graphical theme to be used to draw this canvas*/
    protected GraphicalTheme theme = null;
    
    /** The message referenced by this screen*/
    protected Message message = null;

    /** The screen Title item*/
    protected Title title = null;

    /** Recipients custom item */
    public RecipientsItem recipients = null;
    
    /** Subject custom item */
    public EditField subject = null;
    
    /** Attachments custom item */
    public EditField attachments = null;

    /** Date Sent custom item */
    public EditField dateSent = null;

    /** Body custom item */
    public BodyItem body = null;

    //Custom items contraint:
    //- Recipients item must be limited to 2 lines for To: label and just 1 for
    //the others
    //- Subject item must be limited to 2 lines
    //- Body has not limitations on the lines number

    /** Recipients lines limit */
    int[] highLightedRecipientsMaxLines = {2, 1, 1};

    /** Subject Max lines limit */
    int subjectMaxLines = 2;


    /** CustomItemsVector */
    protected Vector items = new Vector();

    /** The current highlighted item on the screen */
    protected EditField currentItem = null;
    
    protected boolean isBodyScrolling = false;
    
    protected MessageScreen() {
        super();
        drawer = UIController.getDrawer();
        theme = GraphicalThemeFactory.getGraphicalTheme();

        if (message==null) {
            message = new Message();
        }

        mm = UIController.messageManager;
    }

    protected void paintContent(Graphics graphics) {
        
        int width = getWidth();
        int height = getHeight() - TITLE_Y_OFFSET;

        //Avoid the draw of the entire screen when the body is scrolled up or 
        //down
        if (isBodyScrolling) {
            body.setScreenWidth(width);
            body.setScreenHeight(height);
            body.paint(graphics, 0, body.getY(), width);
            isBodyScrolling = false;
            return;
        }
        
        drawer.drawCanvasBackground(graphics, width, height);
        
        //Paint Title
        int y = title.paint(graphics, width, height);

        for (int i = 0; i < items.size(); i++) {
            EditField item = (EditField) items.elementAt(i);
            //Needed by tiltable screens devices to manage the
            //width and height
            item.setScreenWidth(width);
            item.setScreenHeight(height);

            //Needs for the touch screen devices to understand
            //Where the item is initially painted
            item.setY(y+1);
            if (item.isVisible()) {
                y = item.paint(graphics, 0, y, width);
            }
        }
    }
    
    protected void setReadOnly(boolean readOnly) {
        
    }

    protected void setMessage(Message m) {
        this.message = m;
    }
    
    protected void addScreenElement(EditField ci) {
        items.addElement(ci);

        if (ci == attachments) {
            
            EditField[] tmp = new EditField[items.size()];
            items.copyInto(tmp);
            for (int i = 0; i < tmp.length; i++) {
                EditField c = tmp[i];
                items.setElementAt(c, c.getDefaultScreenIndex());
            }
        }
        items.trimToSize();
    }

    protected void addScreenElement(EditField ci, int pos) {
        
        items.insertElementAt(ci, pos);

        if (ci == attachments) {

            EditField[] tmp = new EditField[items.size()];
            items.copyInto(tmp);
            for (int i = 0; i < tmp.length; i++) {
                EditField c = tmp[i];
                items.setElementAt(c, c.getDefaultScreenIndex());
            }
        }
        items.trimToSize();
       
    }
    
    protected void removeScreenElement(EditField ci) {
        ci.setHighlighted(false);
        items.removeElement(ci);
        items.trimToSize();
    }

    protected boolean isItemOnScreen(EditField item) {
        for (int i = 0; i < items.size(); i++) {
            if (item == ((EditField)items.elementAt(i))) {
                return true;
            }
        }
        return false;
    }

    protected void handleKeyPressed(int keyCode) {
        int dir = getGameAction(keyCode);
        if (isDownKey(dir, keyCode)) {
            handleScrollUpEvent();
            
        } else if (isUpKey(dir, keyCode)) {
            handleScrollDownEvent();
        }
    }

    protected void getNextItem() {
        //find the highlighted item into the vector
        for (int i=0; i<items.size(); i++) {
            currentItem = (EditField) items.elementAt(i);
            if (currentItem.isHighlighted()) {
                if (i!=items.size()-1) {
                    currentItem.setHighlighted(false);
                    currentItem = (EditField)items.elementAt(i+1);
                    currentItem.setHighlighted(true);
                    title.setSubTitle(currentItem.getToolTipText());
                }
                
                updateScreenCommands();
                
                break;
            }
        }
    }

    protected void getPreviousItem() {
        //find the highlighted item into the vector
        for (int i=0; i<items.size(); i++) {
            currentItem = (EditField) items.elementAt(i);
            if (currentItem.isHighlighted()) {
                if (i!=0) {
                    currentItem.setHighlighted(false);
                    currentItem = (EditField)items.elementAt(i-1);
                    currentItem.setHighlighted(true);
                    title.setSubTitle(currentItem.getToolTipText());
                }

                updateScreenCommands();

                break;
            }
        }
    }
    
    public void activateItem(int y) {
        EditField item = null;
        for (int i = 0; i<items.size(); i++) {
            item = (EditField) items.elementAt(i);
            int startCoordinate = item.getY();
            if (y >= startCoordinate && y <= startCoordinate + item.getHeight()) {
                activateItem(item);
            }
        }
    }

    public void resetBodyTofirstLine(){
        body.setFirstVisibleRowIndex(0);
    }
    
    public void activateItem(EditField c) {
        for (int i = 0; i<items.size(); i++) {
            ((EditField) items.elementAt(i)).setHighlighted(false);
        }
        c.setHighlighted(true);
        currentItem = c;
        title.setSubTitle(c.getToolTipText());
        updateScreenCommands();
    }
    
    protected void handleTouchEvent(FTouchEvent event) {
        Log.debug("[List.handleTouchEvent]");
        int type     = event.getType();
        int tapCount = event.getTapCount();

        Log.debug("List.handleTouchEvent " + type + " -- " + tapCount);

        switch (type) {
            case FTouchEvent.PRESSED:
                Log.debug("Pressed");
                int y = event.getPressY(0);
                if (y >= 0) {
                    //activate the item drawn on that y
                    activateItem(y);
                }
                repaint();
                break;
            case FTouchEvent.CLICK:
                // Now we may need to open the msg
                if (tapCount > 1) {
                    // Multi taps are considered as double tap
                    fireCommand();
                }
                break;
            case FTouchEvent.DRAGGING_DOWN:
                if (isBodyScrolling) {
                    return;
                }
                handleScrollDownEvent();
                break;
            case FTouchEvent.DRAGGING_UP:
                if (isBodyScrolling) {
                    return;
                }
                handleScrollUpEvent();
                break;
            default:
                Log.trace("Dropping event from List");
        }
    }

    protected void handleScrollDownEvent() {
        //Implement here the scroll logic for the body item
        if (body.isHighlighted() && body.isScrollable()) {
            if (!body.isFirstLineReached()) {
                body.scrollUp(SCROLL_AMOUNT);
                isBodyScrolling = true;
                repaint();
            } else {
                getPreviousItem();
                repaint();
            }
        } else {
            getPreviousItem();
            repaint();
        }
    }

    protected void handleScrollUpEvent() {
        //Implement the scroll logic for the body item
        if (body.isHighlighted() && body.isScrollable()) {
            if (!body.isLastLineReached()) {
                body.scrollDown(SCROLL_AMOUNT);
                isBodyScrolling = true;
                repaint();
            }
        } else {
            getNextItem();
            repaint();
        }
    }


    protected abstract void addCommands();

    protected abstract void updateScreenCommands();

    protected abstract void fireCommand();

}
