/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.view.components.EditField;
import com.funambol.mail.Address;
import com.funambol.mail.Message;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Graphics;

public class RecipientsItem extends EditField {
    
    private Command addRecipientsCommand;
    
    private final String LABEL_TO = Localization.getMessages().TO_LABEL;
    private final String LABEL_CC = Localization.getMessages().CC_LABEL;
    private final String LABEL_BCC = Localization.getMessages().BCC_LABEL;
    
    private Message message;
    
    private StringBuffer to;
    private StringBuffer cc;
    private StringBuffer bcc;

    /**
     * Creates a new instance of RecipientsItem
     */
    public RecipientsItem(Message message, Drawer d, GraphicalTheme gt) {
        super(Localization.getMessages().SELECT_RECIPIENTS, "", d, gt);

        to = new StringBuffer();
        cc = new StringBuffer();
        bcc = new StringBuffer();
        
        this.message=message;
        
        addRecipientsCommand = new Command(Localization.getMessages().ADD_RECIPIENTS_COMMAND, UIController.COMMAND_TYPE, CUSTOM_ITEM_COMMAND_PRIORITY);

        this.setMessage(message);
    }
    
    private int getVisibleNames(StringBuffer s, Address[] list) throws Exception {
        if (list != null&&list.length>0) {
            for(int i=0; i<list.length; i++) {
                if (i > 0) {
                    s.append(", ");
                }
                s.append(list[i].getVisibleName());
            }
        }
        if (s.length()>0) {
            return 1;
        }
        return 0;
    }
    
    private void update(){
        
        to = new StringBuffer();
        cc = new StringBuffer();
        bcc = new StringBuffer();
        
        try {
            Address[] toList = message.getTo();
            Address[] ccList = message.getCc();
            Address[] bccList = message.getBcc();

            int counter = 0;
            int toCount = getVisibleNames(to, toList);
            int ccCount = getVisibleNames(cc, ccList);
            int bccCount = getVisibleNames(bcc, bccList);
            
            counter = toCount + ccCount + bccCount;
            
            if (counter == 0) {
                if(isReadOnly()) {
                    setLabel(LABEL_TO);
                    setText(Localization.getMessages().UNDISCLOSED_RECIPIENTS_LABEL);
                } else {
                    setLabel(Localization.getMessages().SELECT_RECIPIENTS);
                    setText("");
                }
                return;
            }

            String[] itemLabel = new String[counter];
            String[] itemText = new String[counter];

            int toIndex = 0;
            int ccIndex = 1;
            int bccIndex = 2;
            
            if (toCount>0) {
                itemLabel[toIndex] = LABEL_TO;
                itemText[toIndex] = to.toString();
            } else {
                ccIndex--;
                bccIndex--;
            }

            if (ccCount > 0) {
                itemLabel[ccIndex] = LABEL_CC;
                itemText[ccIndex] = cc.toString();
            } else {
                bccIndex--;
            }
            
            if (bccCount>0) {
                itemLabel[bccIndex] = LABEL_BCC;
                itemText[bccIndex] = bcc.toString();
            }
            
            setLabel(itemLabel);
            setText(itemText);

            //The constructor call produces a null pointer exception
            if(stateListener != null) {
                stateListener.notifyChanged();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            Log.error(this, "exception parsing message addresses: " + ex);
        }
    }
    
    /**
     * Set the message reference in order to recalculate the message recipients 
     * strings and draw them correctly.
     * @param message the Message to be set
     */
    public void setMessage(Message message) {
        this.message=message;
        
        update();
    }

    /**
     * Return the default command to edit recipients
     * @return Command the command object to add recipients to the recipients 
     * list
     */
    public Command getAddRecipientsCommand() {
        return addRecipientsCommand;
    }
    
    /**
     * Overrides the superclass method: if the label is "TO:" set the max
     * text lines to 2, 1 otherwise
     * @param y the absolute y coordinate of the item
     * @param g the Graphics object
     */
    protected void recalculate(int y, Graphics g) {
        int[] stringsPerLabel = {0, 1, 1};
        
        String[] labels = getLabel();
        
        for (int i = 0; i < labels.length; i++) {
           if (labels[0].equals(LABEL_TO)) {
               stringsPerLabel[0] = 2;
           } else {
               stringsPerLabel[0] = 1;
           } 
        }
        
        setMaxTextLinesPerLabel(stringsPerLabel);
        
        super.recalculate(y, g);
    }

    
}
