/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */


package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

/**
 * a drawer with rounded corners
 */
public class RoundedDrawer implements Drawer {
    
    protected  GraphicalTheme graphicalTheme;
    
    protected static final int LEFT_EDGE_MARGIN   = 0;
    protected static final int RIGHT_EDGE_MARGIN  = 0; 
    protected static final int TOP_EDGE_MARGIN    = 0;
    protected static final int BOTTOM_EDGE_MARGIN = 2; 
    protected static final int BKG_COLOR_START = 0x00dcee;
    protected static final int BKG_COLOR_END = 0x004a7e;
    protected static final int ARC_WIDTH=0;
    protected static final int ARC_HEIGHT=0;
    //private static final String CONTINUE_STRING="..";
    
    protected int titleBarHeight;
    
    private Font font;
    private int[] colors;
    

    
    
    
    /** Creates a new instance of Drawer */
    public RoundedDrawer() {
      this.graphicalTheme = GraphicalThemeFactory.getGraphicalTheme();
    }
   

    
    public void drawBackground(Graphics g, int w, int h, boolean selected) {

        drawBackground(g, 0, 0, w, h, selected);
    }

    public void drawBackground(Graphics g, int x, int y, int w, int h,
                               boolean selected) {
        
        if (selected) {
            g.setColor(graphicalTheme.getHighlightedBackgroundColor());

            g.fillRoundRect(x + LEFT_EDGE_MARGIN,  y + TOP_EDGE_MARGIN,
                            w - RIGHT_EDGE_MARGIN, h - BOTTOM_EDGE_MARGIN,
                            ARC_WIDTH,ARC_HEIGHT);

            g.setColor(graphicalTheme.getHighlightedBorderColor());

            g.drawRoundRect(x + LEFT_EDGE_MARGIN,      y + TOP_EDGE_MARGIN,
                            w - RIGHT_EDGE_MARGIN - 1, h - BOTTOM_EDGE_MARGIN - 1,
                            ARC_WIDTH,ARC_HEIGHT);

        } else {
            g.setColor(graphicalTheme.getBackgroundColor());

            g.fillRoundRect(x + LEFT_EDGE_MARGIN,  y + TOP_EDGE_MARGIN,
                            w - RIGHT_EDGE_MARGIN, h - TOP_EDGE_MARGIN,
                            ARC_WIDTH,ARC_HEIGHT);
        }
    }

    public void drawBackground(Graphics g, int x, int y, int w, int h,
                               int ratio, boolean isActive, int grayWidth,
                               int grayScaleIncrease) {

        drawBackground(g, x, y, w, h, ratio, isActive);

        // Increase the gray scale
        g.setGrayScale(g.getGrayScale() + grayScaleIncrease);
        g.fillRoundRect(x + LEFT_EDGE_MARGIN,  y + TOP_EDGE_MARGIN,
                        grayWidth - RIGHT_EDGE_MARGIN, h - TOP_EDGE_MARGIN,
                        ARC_WIDTH, ARC_HEIGHT);
    }

    public void drawBackground(Graphics g, int x, int y, int w, int h,
                               int ratio, boolean isActive) {

        int gradientFirstColor = 0xefefef;
        int gradientLastColor = 0xb2b3b4;

        int borderDarkColor = 0x999999;
        int borderLightColor = 0xfefefe;

        if (isActive) {

            //drawGradientBackground(g, x, y, w, h/2, 0xefefef, 0xe4e5e6);
            //drawGradientBackground(g, x, y + h / 2, w, h/2, 0xd4d8d9, 0xb2b3b4);
            drawGradientBackground(g, x, y, w, h, gradientFirstColor, gradientLastColor);

            g.setColor(borderDarkColor);
            g.drawRoundRect(x, y + 1, w - 1, h - 2, ARC_WIDTH, ARC_HEIGHT);

            g.setColor(borderLightColor);
            g.drawRoundRect(x + 1, y + 2, w - 3, h - 4, ARC_WIDTH, ARC_HEIGHT);

        } else {
            if (ratio==1) {
                g.setColor(254, 254, 254);

                g.fillRoundRect(x + LEFT_EDGE_MARGIN,  y + TOP_EDGE_MARGIN,
                                w - RIGHT_EDGE_MARGIN, h - BOTTOM_EDGE_MARGIN,
                                ARC_WIDTH,ARC_HEIGHT);

            } else {
                g.setColor(245, 245, 245);

                g.fillRoundRect(x + LEFT_EDGE_MARGIN,  y + TOP_EDGE_MARGIN,
                                w - RIGHT_EDGE_MARGIN, h - TOP_EDGE_MARGIN,
                                ARC_WIDTH,ARC_HEIGHT);
            }
        }
    }


    public GraphicalTheme getGraphicalTheme(){
        return graphicalTheme;
    }
    
    public void drawString(String string, Graphics graphics, int x, int y, boolean selected) {
        drawString(string, graphics, x, y, Graphics.TOP|Graphics.LEFT,selected);
    }
    public void drawBoldString(String string, Graphics graphics, int x, int y, boolean selected) {
        Font oldfont = graphics.getFont();
        graphics.setFont(getGraphicalTheme().getBoldFont());
        graphics.setColor(graphicalTheme.getForegroundColor());
        graphics.drawString(string, x,y,Graphics.TOP|Graphics.LEFT);
        graphics.setFont(oldfont);
    }
    
    
    public void drawString(String string, Graphics graphics, int x, int y, int layout, boolean selected) {
        graphics.setColor(graphicalTheme.getForegroundColor());
        graphics.setFont(graphicalTheme.getDefaultFont());
        graphics.drawString(string, x,y,layout);
    }
    
    public void drawCanvasBackground(Graphics g, int w, int h) {
        g.setColor(graphicalTheme.getBackgroundColor());
        g.fillRect(0,0,w,h);
    }
    
    public void drawTitleBackground(Graphics g, int w, int h) {
        
        this.titleBarHeight = h;
        colors = UiUtils.getGradient(BKG_COLOR_START, BKG_COLOR_END, h);

        for (int i = 0; i<colors.length; i++) {
            g.setColor(colors[i]);
            g.drawLine(0,i,w,i);
        }
        //g.setColor(graphicalTheme.getTitleBorderColor());
        //g.drawLine(0,h,w,h);
        
    }

    /**
     * Draws gradient background for a generic item
     * @param g the graphic object used to paint
     * @param x the start x coordinate
     * @param y the start y coordinate
     * @param w the width of the gradient region
     * @param h the heigth of the gradient region
     * @param startColor the gradient start color
     * @param endColor the gradient end color
     */
    public void drawGradientBackground(Graphics g, int x, int y, int w, int h, int startColor, int endColor) {
        colors = UiUtils.getGradient(startColor, endColor, h);
        
        for (int i = 0; i<colors.length; i++) {
            g.setColor(colors[i]);
            g.drawLine(x, i + y, w, i + y);
        }
    }
    
  /*  public int[] getGradient(int aStartColor, int aEndColor, int aLength){
        int r1 = aStartColor>>16;
        int g1 = aStartColor>>8;
        g1 = g1 & 0x0000FF;
        int b1 = aStartColor;
        b1 = b1 & 0x0000FF;
        
        int r2 = aEndColor>>16;
        int g2 = aEndColor>>8;
        g2 = g2 & 0x0000FF;
        int b2 = aEndColor;
        b2 = b2 & 0x0000FF;
        
        int res_r = 0;
        int res_g = 0;
        int res_b = 0;
        
        int gradient[] = new int[aLength];
        
        int counter = 0;
        int dif;
        while(counter < aLength){
            dif = (aLength-counter);
            res_r = (r1 * dif + r2*counter)/aLength ;
            res_g = (g1 * dif + g2*counter)/aLength ;
            res_b = (b1 * dif + b2*counter)/aLength ;
            
            res_r = res_r <<16;
            res_g = res_g <<8;
            
            gradient[counter] = (res_r | res_g | res_b);
            
            counter++;
        }
        return gradient;
    }
    */

    public void drawHighlightedCustomItemSquaredBackground(Graphics g, int x, int y, int w, int h) {
        g.setColor(graphicalTheme.getCustomItemHighlightedSquaredBrightColor());
        g.fillRect(x, y, w, h / 2);
        g.setColor(graphicalTheme.getCustomItemHighlightedSquaredDarkColor());
        g.fillRect(x, y + h / 2, w, h / 2);

        g.setColor(graphicalTheme.getCustomItemSquaredFrameBrightColor());
        g.drawRect(x + 1, y + 1, w - 3, h - 4);
        g.setColor(graphicalTheme.getCustomItemSquaredFrameDarkColor());
        g.drawRect(x, y, w - 1, h - 2);
    }
    
    public void drawHighlightedBodyItemBackground(Graphics g, int x, int y, int w, int totalItemH) {
        //Draw the body highlighted background
        int[] color = graphicalTheme.getBodyHighLightedTextColor();
        g.setColor(color[0], color[1], color[2]);
        g.fillRect(x, y, w, totalItemH);

        //Draw the body highlighted frame
        color = graphicalTheme.getBodyHighLightedFrameColor();
        g.setColor(color[6], color[7], color[8]);
        g.drawRect(x+2, y+2, w - 5, totalItemH - 5);
        g.setColor(color[3], color[4], color[5]);
        g.drawRect(x+1, y+1, w - 3, totalItemH - 3);
        g.setColor(color[0], color[1], color[2]);
        g.drawRect(x, y, w - 1, totalItemH - 1);
    }

    public void drawHighlightedCustomItemGradientBackground(Graphics g, int x, int y, int w, int h) {

        int initColor = graphicalTheme.getCustomItemHighlightedGradientBrightColor();
        int endColor = graphicalTheme.getCustomItemHighlightedGradientDarkColor();

        int[] gradient = UiUtils.getGradient(initColor, endColor, h);

        for (int i = 0; i < h; i++) {
            g.setColor(gradient[i]);
            g.drawLine(x, y + i, w, y + i);
        }
    
        g.setColor(graphicalTheme.getCustomItemGradientFrameBrightColor());
        g.drawRect(x + 1, y + 1, w - 3, h - 4);
        g.setColor(graphicalTheme.getCustomItemGradientFrameDarkColor());
        g.drawRect(x, y, w - 1, h - 2);
    }

    public void drawCustomItemLabel(String labelText, Font labelFont, Graphics g, int labelFrameX, int labelFrameY, int labelFrameW, int labelFrameH, int labelFrameArcWidth, int labelFrameArcHeight, int labelTextX, int labelTextY, int layout) {

        //draw the label frame
        g.setColor(graphicalTheme.getLabelBackgroundColor());
        g.fillRoundRect(labelFrameX, labelFrameY, labelFrameW, labelFrameH, labelFrameArcWidth, labelFrameArcHeight);

        //Draw the label text
        g.setFont(labelFont);
        g.setColor(graphicalTheme.getTitleColor());
        g.drawString(labelText, labelTextX, labelTextY, layout);

    }

    public void drawCustomItemTextString(String s, Graphics g, int x, int y, int layout, Font f, int color) {
        g.setColor(color);
        g.setFont(f);
        g.drawString(s, x, y, layout);
    }
}
