/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.view.components.EditField;
import com.funambol.mailclient.ui.view.components.BodyItem;
import com.funambol.mailclient.ui.view.components.Title;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;

import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.cm.rms.RMSContactManager;
import com.funambol.mailclient.loc.Localization;

import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.BodyPart;

import com.funambol.util.Log;
import com.funambol.util.MailDateFormatter;
import com.funambol.util.StringUtil;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.CommandListener;

import java.util.Vector;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;


public class ViewMessageScreen extends MessageScreen implements CommandListener, ItemStateListener {

    /** Next message increment*/
    private static final int NEXT_MESSAGE = 1;
    /** Previous message increment*/
    private static final int PREVIOUS_MESSAGE = -1;

    //Custom Items indexes:

    /** Recipients item index*/
    private static final int RECIPIENTS_ITEM_INDEX = 0;
    /** Date Sent item index*/
    private static final int DATE_SENT_ITEM_INDEX = 1;
    /** Attachments item index*/
    private static final int ATTACHMENTS_ITEM_INDEX = 2;
    /** Subject item index*/
    private static final int SUBJECT_ITEM_INDEX = 3;
    /** Body item index*/
    private static final int BODY_ITEM_INDEX = 4;

     //Commands
    private Command folderCommand;

    private Command nextCommand;
    private Command previousCommand;

    private Command fullMessageCommand;
    private AttachmentCommand attachmentCommand;
    

    private Command replyCommand;
    private Command replyAllCommand;
    private Command forwardCommand;

    private Command deleteCommand;
    private Command followUpCommand;
    private Command markAsReadCommand;

    //private Command detailsCommand;
    private Command recipientsCommand;
    private Command saveSenderCommand;
    private Command callCommand;
    private Command linksCommand;
    
    private Command editCommand;
    
    //Commands priority
    private static final int FOLDER_CMD_PRIORITY = 10;

    //Used viewing message from sent folder
    private static final int EDIT_CMD_PRIORITY = 15;

    private static final int NEXT_CMD_PRIORITY = 20;
    private static final int PREV_CMD_PRIORITY = 30;

    private static final int GETMORE_CMD_PRIORITY = 40;
    private static final int ATTACH_CMD_PRIORITY = 50;

    private static final int REPLY_CMD_PRIORITY = 60;
    private static final int REPLYALL_CMD_PRIORITY = 80;
    private static final int FORWARD_CMD_PRIORITY = 90;

    private static final int DELETE_CMD_PRIORITY = 100;
    private static final int FLAG_CMD_PRIORITY = 110;
    private static final int MARKREAD_CMD_PRIORITY = 120;
    private static final int MORE_CMD_PRIORITY = 130;
    
    private static final int DATAILS_CMD_PRIORITY = 130;
    private static final int RECIPIENTS_CMD_PRIORITY = 140;
    private static final int SAVESENDER_CMD_PRIORITY = 150;
    private static final int CALLSENDER_CMD_PRIORITY = 160;
    private static final int LINKS_CMD_PRIORITY = 170;

    private MenuItem flagMenuItem;
    private MenuItem markAsReadMenuItem;
    private MenuItem nextMenuItem;
    private MenuItem prevMenuItem;
    private MenuItem deleteMenuItem;
    private MenuItem detailsMenuItem;
    private MenuItem callMenuItem;
    private MenuItem attachMenuItem;

    private Menu moreSubMenu;
    private Menu attachMenu;

    private Contact contactToCall;

    private boolean fullMsgPerformed = false;
    private boolean attachReadOnly = false;
    private boolean showedDetails = false;

    public ViewMessageScreen (Message m, boolean attachReadOnly) {
        super();
        this.attachReadOnly = attachReadOnly;

        message = m;

        Address from = message.getFrom();
        String visible;
        if (from != null) {
            visible = from.getVisibleName();
        } else {
            visible = Localization.getMessages().DEFAULT_SENDER_VISIBLE_NAME;
        }
        title = new Title(theme, visible, null, getWidth());

        //Initialize the recipients custom item
        recipients = new RecipientsItem(message, drawer, theme);
        dateSent = new EditField(drawer, theme);
        subject = new EditField(drawer, theme);
        attachments = new EditField(drawer,theme);
        body = new BodyItem(drawer, theme);
        update(message, attachReadOnly);
    }

    public void setMessage(Message fullmsg) {
        if (fullmsg == null) {
            Log.info("The server didn't send the full message, ");
            this.enableFullMessageCommand(true);
            return;
        }

        if (this.message == null) {
            // cleanup has been called
            Log.info("User closed the viewmessagescreen, discarding full message");
            return;
        }

        try {
            fullmsg.setParent(message.getParent());
            //Fix for message flags - see 2772 bug
            fullmsg.setFlags(message.getFlags());
            message = fullmsg;
        } catch (MailException e) {
            this.setTitle(Localization.getMessages().MESSAGE_VIEW_FORM_TITLE);
            Log.error("Error getting VisibleName - " + e.getMessage());
            e.printStackTrace();
        } finally {
            this.removeCommand(fullMessageCommand);
            update(message, attachReadOnly);
        }

        this.message = fullmsg;
    }

    public void update(Message message, boolean attachReadOnly) {
        this.attachReadOnly = attachReadOnly;

        this.message = message;

        Address from = message.getFrom();
        String visible;
        if (from != null) {
            visible = from.getVisibleName();
        } else {
            visible = Localization.getMessages().DEFAULT_SENDER_VISIBLE_NAME;
        }
        title.setMainTitle(visible);
        title.enableMessageImage(false);

        title.enableFlagImage(message.getFlags().isSet(MessageFlags.FLAGGED));
        title.enableAttachmentsImage(message.hasAttachments());

        //Initialize the recipients custom item
        recipients.setItemStateListener(this);
        recipients.setMaxTextLinesPerLabel(highLightedRecipientsMaxLines);
        recipients.setMessage(message);
        recipients.setDefaultScreenIndex(RECIPIENTS_ITEM_INDEX);

        //Initialize the date sent custom item
        dateSent.setLabel(Localization.getMessages().DATE_SENT_LABEL);
        dateSent.setText(MailDateFormatter.formatLocalTime(message.getSentDate()));
        dateSent.setDefaultScreenIndex(DATE_SENT_ITEM_INDEX);

        //Initialize the subject custom item
        subject.setMaxTextLinesPerLabel(subjectMaxLines);
        subject.setLabel(Localization.getMessages().CMF_SUBJECT_TF_LABEL);
        subject.setText(message.getSubject());
        subject.setItemStateListener(this);
        subject.setVisible(true);
        subject.setReadOnly(true);
        subject.setDefaultScreenIndex(SUBJECT_ITEM_INDEX);
        subject.enableLabel(false);

        attachments.setItemStateListener(this);
        attachments.setDefaultScreenIndex(ATTACHMENTS_ITEM_INDEX);

        //Initialize the body custom item
        body.setLabel(Localization.getMessages().CMF_BODY_TF_LABEL);
        body.setText(message.getTextContent());
        body.setVisible(true);
        body.setReadOnly(true);
        body.enableScrollBar(true);
        body.setItemStateListener(this);
        body.setDefaultScreenIndex(BODY_ITEM_INDEX);
        body.enableBackground(false);
        body.setTextFont(Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL));

        if (!isItemOnScreen(recipients)) {
            addScreenElement(recipients);
        }

        if (!isItemOnScreen(dateSent)) {
            addScreenElement(dateSent);
        }

        if (!isItemOnScreen(subject)) {
            addScreenElement(subject);
        }

        if (!isItemOnScreen(body)) {
            addScreenElement(body);
        }

        body.setChanged(true);

        updateAttachments();
        
        currentItem = body;

        addCommands();

        activateItem(body);
        
        enableHeaderVisibility(false);
        repaint();
    }

    protected void addCommands() {

        //Command priority must give the following sequence:
        //Folder (where applicable is on the soft key and not in this menu)
        //Next
        //Previous
        //---
        //Get More
        //Attachments
        //---
        //Reply...
        //Reply All...
        //Forward...
        //---
        //Delete
        //Clear Flag
        //Mark Unread
        //---
        //More --> View Recipients
                 //Show Details
                 //Save Sender
                 //Call Sender
                 //Browse Lnk...

        if(folderCommand != null){
            removeCommand(folderCommand);
        }

        if (nextCommand==null) {
            nextCommand = new Command(Localization.getMessages().MESSAGE_VIEW_NEXT_MESSAGE, UIController.COMMAND_TYPE,
                                        NEXT_CMD_PRIORITY);
            nextMenuItem = new MenuItem(nextCommand,"[>]",null);
            nextMenuItem.setComandInfoFgColor(GraphicalThemeFactory.getGraphicalTheme().getShortcutsFgColor());
        } else {
            removeCommand(nextCommand);
        }

        if (previousCommand==null) {
            previousCommand = new Command(Localization.getMessages().MESSAGE_VIEW_PREVIOUS_MESSAGE, UIController.COMMAND_TYPE,
                                        PREV_CMD_PRIORITY);
            prevMenuItem = new MenuItem(previousCommand,"[<]",null);
            prevMenuItem.setComandInfoFgColor(GraphicalThemeFactory.getGraphicalTheme().getShortcutsFgColor());
        } else {
            removeCommand(previousCommand);
        }
        if (fullMessageCommand == null) {
            fullMessageCommand = new Command(Localization.getMessages().MVF_FULL_MSG_CMD, UIController.COMMAND_TYPE,
                                            GETMORE_CMD_PRIORITY);
        } else {
            removeCommand(fullMessageCommand);
        }

        if (replyCommand == null) {
            replyCommand = new Command(Localization.getMessages().MVF_REPLY_COMMAND, UIController.COMMAND_TYPE,
                                            REPLY_CMD_PRIORITY);
        } else {
            removeCommand(replyCommand);
        }

        if (replyAllCommand == null) {
            replyAllCommand = new Command(Localization.getMessages().MVF_REPLY_ALL_COMMAND, UIController.COMMAND_TYPE,
                                            REPLYALL_CMD_PRIORITY);
        } else {
            removeCommand(replyAllCommand);
        }

        if (forwardCommand == null) {
            forwardCommand = new Command(Localization.getMessages().MVF_FORWARD_COMMAND, UIController.COMMAND_TYPE,
                                            FORWARD_CMD_PRIORITY);
        } else {
            removeCommand(forwardCommand);
        }
        
        if (deleteCommand == null) {
            deleteCommand = new Command(Localization.getMessages().DELETE_COMMAND_LABEL, UIController.COMMAND_TYPE,
                                            DELETE_CMD_PRIORITY);
            deleteMenuItem = new MenuItem(deleteCommand, "[0]", null);
            deleteMenuItem.setComandInfoFgColor(GraphicalThemeFactory.getGraphicalTheme().getShortcutsFgColor());

        } else {
            removeCommand(deleteCommand);
        }

        if (followUpCommand == null) {
            followUpCommand = new Command(Localization.getMessages().UNFLAG_COMMAND_LABEL, UIController.COMMAND_TYPE,
                                                     FLAG_CMD_PRIORITY);
             // For this command we need the corresponding menu item to change its
            // text
            flagMenuItem = new MenuItem(followUpCommand,"[*]",null);
            flagMenuItem.setComandInfoFgColor(GraphicalThemeFactory.getGraphicalTheme().getShortcutsFgColor());
        }
       
        
        if (markAsReadCommand == null) {
            markAsReadCommand = new Command(Localization.getMessages().UNREAD_COMMAND_LABEL, UIController.COMMAND_TYPE,
                                                 MARKREAD_CMD_PRIORITY);
            markAsReadMenuItem = new MenuItem(markAsReadCommand);

            
        }
       

        
        if (linksCommand == null) {
            linksCommand = new Command(Localization.getMessages().MVF_LINKS_COMMAND, UIController.COMMAND_TYPE,
                                            LINKS_CMD_PRIORITY);
        } else {
            removeCommand(linksCommand);
        }

        

        if (callCommand == null) {
            callCommand = new Command(Localization.getMessages().CALL_SENDER_COMMAND, UIController.COMMAND_TYPE,
                                            CALLSENDER_CMD_PRIORITY);
            callMenuItem = new MenuItem(callCommand);
        } else {
            removeCommand(callCommand);
        }

        

        if (saveSenderCommand == null) {
            saveSenderCommand = new Command(Localization.getMessages().MVF_SAVE_SENDER_COMMAND, UIController.COMMAND_TYPE,
                                            SAVESENDER_CMD_PRIORITY);
        } else {
            removeCommand(saveSenderCommand);
        }

        /**
        if (detailsCommand == null) {
            detailsCommand = new Command(Localization.getMessages().SHOWDETAILS_COMMAND_LABEL, UIController.COMMAND_TYPE,
                                            DATAILS_CMD_PRIORITY);
            detailsMenuItem = new MenuItem(detailsCommand);
            
        } else {
            removeCommand(detailsCommand);
        }
        **/

        if (recipientsCommand == null) {
            recipientsCommand = new Command(Localization.getMessages().MVF_RECIPIENTS_COMMAND, UIController.COMMAND_TYPE,
                                            RECIPIENTS_CMD_PRIORITY);
        } else {
            removeCommand(recipientsCommand);
        }

        if (editCommand == null) {
            editCommand = new Command(Localization.getMessages().EDIT_COMMAND_LABEL, UIController.COMMAND_TYPE,
                                            EDIT_CMD_PRIORITY);
        } else {
            removeCommand(editCommand);
        }

        this.addMenuItem(nextMenuItem);
        this.addMenuItem(prevMenuItem);
        this.addSeparator(PREV_CMD_PRIORITY + 1);

        boolean needSeparator = false;
        //Adds the Get More command if the SyncClient is idle
        if (message.getFlags().isSet(MessageFlags.PARTIAL)) {
            if (UIController.isOkToAddOnlineCommands()) {
                addCommand(fullMessageCommand);
                needSeparator = true;
            }
        }
        if(attachMenuItem!=null){
            removeMenuItem(attachMenuItem);
        }
        if(message.hasAttachments()) {
            populateAttachMenu();
            addMenuItem(attachMenuItem);
            needSeparator = true;
        }
        
        if(needSeparator){
           this.addSeparator(ATTACH_CMD_PRIORITY + 1);
        }
        addMenuItem(flagMenuItem);
        addMenuItem(markAsReadMenuItem);
        //detailsMenuItem.setText(Localization.getMessages().SHOWDETAILS_COMMAND_LABEL);
        
        if(moreSubMenu == null){
            addMenuItem(getMoreSubMenu());
        }

        if (!UiUtils.isOutgoing(message)) {
            this.addCommand(forwardCommand);
        }


        this.addMenuItem(deleteMenuItem);

        //Folders commands management: the available commands are belonging by
        //the kind of message folder beacause different behaviors are defined
        //rispectively for Inbox, Oubox/Draft and Sent Folders.
        if (UiUtils.isIncoming(message)) {
            //Inbox folder commands management

            this.addCommand(replyCommand);
            int numRec = message.getNumberOfRecipients();

            //TODO: verify why the code below is needed
            Address[] to = message.getTo();
            for (int i = 0; i < to.length; i++) {
                if (!StringUtil.equalsIgnoreCase(to[i].getEmail(), UIController.getMyAddress())) {
                    numRec++;
                }
            }

            if (numRec > 1) {
                this.addCommand(replyAllCommand);
            }

            this.addSeparator(FORWARD_CMD_PRIORITY+ 1);

           
            setFollowUpCommand();
            setMarkAsReadCommand();
            addSeparator(MARKREAD_CMD_PRIORITY+ 1);
            //#ifndef isBlackberry
                folderCommand = new Command(Localization.getMessages().INBOX_COMMAND, UIController.COMMAND_TYPE, 0);
                this.addCommand(folderCommand);
            //#endif
            

        } else if (UiUtils.isOutgoing(message)) {
            //#ifndef isBlackberry
                folderCommand = new Command(Localization.getMessages().OUTBOX_COMMAND, UIController.COMMAND_TYPE, 0);
                this.addCommand(folderCommand);
            //#endif

        } else if (UiUtils.isSent(message)) {
            //Sent folder: the edit command is available in this folder to
            //Reproduce the "Edit as new" behavior
            //#ifndef isBlackberry
                folderCommand = new Command(Localization.getMessages().SENT_COMMAND, UIController.COMMAND_TYPE, 0);
                this.addCommand(folderCommand);
            //#endif
            this.addCommand(editCommand);
        }

        //Display a non-autodismiss alert if the contact list is not ready in
        //memory.
        if (!UIController.getContactsLoader().isLocked()) {
            setCallCommand();
        }

        this.setCommandListener(this);
    }

    protected void updateScreenCommands() {
        //There are no dependent screen commands to be associated with screen
        //items on this screen
    }

    protected void fireCommand() {
        //There are not commands to be fired with the fire key
    }

    protected void handleKeyRepeated(int keyCode) {
        keyPressed(keyCode);
    }

    public void notifyChanged() {
        recipients.setChanged(true);
        dateSent.setChanged(true);
        attachments.setChanged(true);
        subject.setChanged(true);
        body.setChanged(true);
    }

    /**
     * Enable the "Call Sender" command if the telephone number is in the client
     * contact list
     */
    public void enableCallCommand() {
        setCallCommand();
    }

    /**
     * enable or disable the fullMessageCommand command
     * @param enable if true enable the command, if false disable the command
     */
    public void enableFullMessageCommand(boolean enable) {
        Log.debug(this, "Enable sync commands: " + enable);
        if (enable && !fullMsgPerformed) {
            addCommand(fullMessageCommand);
        } else {
            removeCommand(fullMessageCommand);
        }
    }

    public void commandAction(Command command, Displayable displayable) {
       if (command == UIController.lowLevelBackCommand) {
            try {
                if (!UIController.isSyncInProgress()) {
                    UIController.getInboxMessageList().resetTitle();
                }
                UIController.showBackScreen();
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error("ViewMessage: " + ex);
            }
        }else if (command == folderCommand) {
            try {
                if(UiUtils.isSent(message)){
                    UIController.showSentScreen(null);
                }else if(UiUtils.isOutgoing(message)){
                    UIController.showOutboxScreen(null);
                }else{
                    if (!UIController.isSyncInProgress()) {
                        UIController.getInboxMessageList().resetTitle();
                    }
                    UIController.showInboxScreen();
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error("ViewMessage: " + ex);
            }
        } else if (command == nextCommand) {
            try {
                title.setMainTitle(Localization.getMessages().MESSAGE_VIEW_NEXT_MESSAGE);
                repaint();
                getNextMessage(NEXT_MESSAGE);
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "Exception: " + ex);
            }
        } else if (command == previousCommand) {
            try {
                title.setMainTitle(Localization.getMessages().MESSAGE_VIEW_PREVIOUS_MESSAGE);
                repaint();
                getNextMessage(PREVIOUS_MESSAGE);
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "Exception: " + ex);
            }
        } else if (command == replyCommand) {
            UIController.replyMessage(message);
        } else if (command == forwardCommand) {
            UIController.forwardMessage(message);
        } else if (command == deleteCommand) {
            deleteMessage();
        } else if (command == saveSenderCommand) {
            if (UIController.checkContactListFull()) {
                UIController.showErrorAlert(
                        Localization.getMessages().AB_LIMIT_ERROR_MSG);
                return;
            }
            Contact contact;
            try {
                // resetting the message in contact list
                UIController.getContactList().setMessage(new Message());

                if (message.getFrom() != null) {
                    contact = new Contact(
                            message.getFrom().getName(),
                            message.getFrom().getEmail());
                    UIController.showAddContactScreen(
                            UIController.display.getCurrent(), contact);
                } else {
                    UIController.showAlert(Localization.getMessages().FROM_IS_NULL);
                }

            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "Mail Exception while adding Contact" + ex);
                UIController.showAddContactScreen(UIController.display.getCurrent());
                UIController.showErrorAlert(
                        Localization.getMessages().MESSAGE_VIEW_UNABLE_TO_CREATE_CONTACT);
            }

        } else if (command == linksCommand) {
            String content = message.getTextContent();
            UIController.showLinkList(UIController.display.getCurrent(), content);
        } else if (command instanceof AttachmentCommand) {
            BodyPart bp = ((AttachmentCommand)command).getBodyPart();
            String msg;
            //#ifdef isBlackberry
            //# UIController.openUrl(bp.getAttachUrl());
            //#else
            msg = Localization.getMessages().OPEN_ATTACHMENT_1 +
                    " " + bp.getFileName() +
                    " " + Localization.getMessages().OPEN_ATTACHMENT_2;

            UIController.openAttachmentWithConfirmation(bp);
        //#endif
        } else if (command == recipientsCommand) {
            Log.info("showing recipients for message " + message.getSubject());
            UIController.showRecipientsList(UIController.display.getCurrent(), message);
        } else if (command == editCommand) {
            UIController.editAsNew(message);
        } else if (command == replyAllCommand) {
            UIController.replyAll(message);
        } else if (command == markAsReadCommand) {
            UIController.changeReadFlag(message);
            setMarkAsReadCommand();
        } else if (command == followUpCommand) {
            int currentLine = body.getFirstVisibleRowIndex();
            boolean headerVisible = recipients.isVisible();
            UIController.changeFollowUpFlag(message);
            setFollowUpCommand();
            UIController.getInboxMessageList().activate(UIController.getInboxMessageList().getActiveElementId());
            Log.debug("Current Line: " + currentLine);

            update(message, attachReadOnly);
            Log.debug("Bodyinstance = " + body);

            //header was visible before flag action
            if (currentLine == 0 && headerVisible) {
                handleGameActionKey(Canvas.UP);
                repaint();
            }/* else {
                handleGameActionKey(Canvas.DOWN);
                handleGameActionKey(Canvas.UP);
            }*/
        } else if (command == fullMessageCommand) {
            this.removeCommand(fullMessageCommand);
            fullMsgPerformed = true;
            title.setMainTitle(Localization.getMessages().MVF_FULL_MESSAGE_LOADING);
            repaint();
            UIController.setSyncCaller(UIController.USER);
            UIController.getFullMessage(message);
        } else if (command == callCommand) {
            UIController.phoneCall(contactToCall, this);
        } else {
            super.commandAction(command, this);
        }
    }

    private void deleteMessage() {

        if (UIController.getConfig().getDeleteConfirmation()) {
            UIController.deleteMessageWithConfirmation(message);
        } else {
            UIController.deleteMessage(message);

            if(UiUtils.isOutgoing(message)){
                UIController.showOutboxScreen(null);
            } else if (UiUtils.isSent(message)) {
                UIController.showSentScreen(null);
            } else {
                returnFromDelete();
            }
        }

    }

    public void returnFromDelete() {
        int choice = UIController.mailClientConfig.getDeleteOpenMessage();

        int shift = 0;

        if (UIController.getInboxMessageList().getNumMessages()==0) {
            UIController.showInboxScreen();
        }

        switch(choice) {
            case GeneralSettingsForm.GO_TO_INBOX:

                UIController.showInboxScreen();
                break;
            case GeneralSettingsForm.GO_TO_NEXT_MSG:
                if (UIController.getInboxMessageList().getActiveElementId()==
                        UIController.getInboxMessageList().getNumMessages()-1) {
                    shift = 1;
                }

                getNextMessage(shift);
                break;

            case GeneralSettingsForm.GO_TO_NEXT_UR_MSG:
                shift = calculateNextUnreadMessagesPosition(NEXT_MESSAGE);
                getNextMessage(shift);
                break;

            case GeneralSettingsForm.GO_TO_PREV_MSG:
                getNextMessage(PREVIOUS_MESSAGE);
                break;

            case GeneralSettingsForm.GO_TO_PREV_UR_MSG:
                shift = calculateNextUnreadMessagesPosition(PREVIOUS_MESSAGE);
                getNextMessage(shift);
                break;
            default:
                break;
        }
    }

    private int calculateNextUnreadMessagesPosition(int direction) {

        Vector sortedMessages = UIController.getSortedMessages(message.getParent());
        int ret = 0;

        Message[] ml = new Message[sortedMessages.size()];
        int[] msgIndexes = new int[sortedMessages.size()];
        sortedMessages.copyInto(ml);

        int activeElement = UIController.getInboxMessageList().getActiveElementId();
        Log.debug("Active Element: " + activeElement);

        if (direction<0) {
            for (int i=activeElement; i<=0; i--) {
                Message m = ml[i];

                if (!m.getFlags().isSet(MessageFlags.OPENED)) {
                    ret=i-activeElement-1;
                    break;
                } else {
                    msgIndexes[i]=-1;
                }
            }
        } else {
            for (int i=activeElement; i<ml.length; i++) {
                Message m = ml[i];

                if (!m.getFlags().isSet(MessageFlags.OPENED)) {
                    ret=i-activeElement;
                    break;
                } else {
                    msgIndexes[i]=-1;
                }
            }

        }

        for (int i=0; i<msgIndexes.length; i++) {
            Log.debug("Msg " + (i+1) + ": " + msgIndexes[i]);
        }

        Log.debug("Shifting " + ret + " Messages");
        if (direction<0) {
            return -ret;
        } else {
            return ret;
        }
    }


    private void setCallCommand() {
        Address from = message.getFrom();
        // check in case of missing from like for messages in Sent Folder
        if (from != null) {
            try {
                contactToCall = ((RMSContactManager) UIController.getContactManager()).searchContactFromMail(from.getEmail());
            } catch (MailException ex) {
                Log.error("Error searching contact from email address in ViewMessage setting CallCommand ");
                ex.printStackTrace();
            } catch (ContactManagerException ex) {
                Log.error("Error searching contact from email address in ViewMessage setting CallCommand ");
                ex.printStackTrace();
            }
            if (isValidForCall(contactToCall)) {
                callMenuItem.enable();
            } else {
                callMenuItem.disable();
            }
        } else {
            callMenuItem.disable();
        }
    }

    private boolean isValidForCall(Contact contactToCall) {
        boolean result = false;
        if (contactToCall != null) {
            if (!StringUtil.isNullOrEmpty(contactToCall.getHomePhone()) ||
                !StringUtil.isNullOrEmpty(contactToCall.getJobPhone())  ||
                !StringUtil.isNullOrEmpty(contactToCall.getMobilePhone()))
            {
                result = true;
            }
        }
        return result;
    }

    private void setMarkAsReadCommand() {
        if (message.getFlags().isSet(MessageFlags.OPENED)) {
            markAsReadMenuItem.setText(Localization.getMessages().UNREAD_COMMAND_LABEL);
        } else {
             markAsReadMenuItem.setText(Localization.getMessages().READ_COMMAND_LABEL);
        }

    }

    private void setFollowUpCommand() {

        if (message.getFlags().isSet(MessageFlags.FLAGGED)) {
            flagMenuItem.setText(Localization.getMessages().UNFLAG_COMMAND_LABEL);
        } else {
            flagMenuItem.setText(Localization.getMessages().FLAG_COMMAND_LABEL);
        }
        
    }

    private void updateAttachments() {

        if(message.hasAttachments()) {
            //Initializes the attachment item on the screen
            attachments.setLabel(Localization.getMessages().CMF_ATTACH_LABEL);
            if (!isItemOnScreen(attachments)) {
                addScreenElement(attachments);
            }
            Vector files = UiUtils.getAttachments(message);

            //more than 1 attachment found
            if (message.attachmentsNumber() > 1) {
                attachments.setText(" " + message.attachmentsNumber() + " " + Localization.getMessages().CMF_ATTACH_FILES);
            //1 attachment found
            } else {
                attachments.setText(" " + ((BodyPart) files.elementAt(0)).getFileName());
            }

        //NO attachment found
        } else {
            removeScreenElement(attachments);
        }
    }

    /**
     * Sets the correct message when left/rigth navigation is performed
     * @param shift is the
     */
    public void getNextMessage(int shift) {
        MessageList msgList = UIController.getMessageList(message);
        int msgNumber = msgList.getNumMessages();
        int messageToSet = msgList.getActiveElementId() + shift;
        if (messageToSet >= msgNumber) {
            messageToSet = 0;
            msgList.rewindList();
            shift = 0;
        } else if (messageToSet < 0) {
            messageToSet = msgNumber - 1;
            shift = msgList.getNumMessages();
        }

        if (Math.abs(shift) > 1) {
            for (int i = 0; i < shift; i++) {
                msgList.scroll(1);
                msgList.repaint();
            }
        } else {
            msgList.scroll(shift);
        }
        msgList.setActiveElement(messageToSet);
        Log.debug("getNextMessage has found message: " + msgList.getSelectedMessage());

        resetBodyTofirstLine();

        UIController.showViewMessageScreen(
                msgList, msgList.getSelectedMessage(),attachReadOnly);
    }

    /**
     * Handle the key navigation
     * @param keyCode is the key that has been pressed
     */
    protected void handleKeyPressed(int keyCode) {

        int dir = getGameAction(keyCode);
        if (keyCode == Canvas.KEY_STAR) {
            commandAction(followUpCommand, this);
        }

        if (isFireKey(dir, keyCode)) {
            // No fire action in this screen, but on BB we want to open the menu
//#ifdef isBlackberry
//#         handleFireKeyBB();
//#endif
        } else if (UIController.isClearKey(keyCode)) {
            deleteMessage();
        } else if (keyCode == -10) {
            // green key
            if (isValidForCall(contactToCall)) {
                UIController.phoneCall(contactToCall, this);
            }
        } else if (keyCode == -7) {
//#ifdef isBlackberry
//#         commandAction(UIController.lowLevelBackCommand, this);
//#endif
        } else {
            handleGameActionKey(dir);
        }
    }

    protected void handleGameActionKey(int dir) {

        if (dir == Canvas.DOWN) {
            if (recipients.isVisible()&&body.isFirstLineReached()) {
                enableHeaderVisibility(false);
                notifyChanged();
            } else {
                body.scrollDown(SCROLL_AMOUNT);
            }
            repaint();
        } else if (dir == Canvas.UP) {
            if (!recipients.isVisible()&&body.isFirstLineReached()) {
                enableHeaderVisibility(true);
                notifyChanged();
            } else {
                body.scrollUp(SCROLL_AMOUNT);
            }
            repaint();
        } else if (dir == Canvas.RIGHT) {
            try {
                title.setMainTitle(Localization.getMessages().MESSAGE_VIEW_NEXT_MESSAGE);
                repaint();
                getNextMessage(NEXT_MESSAGE);
            } catch (NullPointerException nex) {
                Log.error(this, nex.getMessage());
            }
        } else if (dir == Canvas.LEFT) {
            try {
                title.setMainTitle(Localization.getMessages().MESSAGE_VIEW_PREVIOUS_MESSAGE);
                repaint();
                getNextMessage(PREVIOUS_MESSAGE);
            } catch (NullPointerException nex) {
                Log.error(this, nex.getMessage());
            }
        }
    }

    private void handleFireKeyBB() {
        // Show the menu when the user presses the fire button
        openMainMenu();
    }

    private void enableHeaderVisibility(boolean enable) {
        recipients.setVisible(enable);
        dateSent.setVisible(enable);
        attachments.setVisible(enable);
        subject.enableLabel(enable);
    }

    private MenuItem getMoreSubMenu() {
        moreSubMenu  = new Menu(Localization.getMessages().MORE_COMMAND_LABEL);
        //moreSubMenu.addMenuItem(detailsMenuItem);
        moreSubMenu.addMenuItem(new MenuItem(recipientsCommand));
        moreSubMenu.addMenuItem(new MenuItem(saveSenderCommand));
        moreSubMenu.addMenuItem(callMenuItem);
        
        
        if (!UiUtils.isOutgoing(message)) {
            moreSubMenu.addMenuItem(new MenuItem(linksCommand));
        }
        return new MenuItem(moreSubMenu, MORE_CMD_PRIORITY);
    }

     private void populateAttachMenu(){
        attachMenu  = new Menu(Localization.getMessages().ATTACHMENT_LIST_CMD);
        
        Vector attachments = UiUtils.getAttachments(message);
        for(int i = 0; i < attachments.size(); ++i) {
            attachmentCommand = new AttachmentCommand(((BodyPart)attachments.elementAt(i)), i);
            attachMenu.addMenuItem(new MenuItem(attachmentCommand));
        }
        attachMenuItem = new MenuItem(attachMenu, ATTACH_CMD_PRIORITY);
    }

    protected void handleScrollDownEvent() {
        handleGameActionKey(Canvas.DOWN);
    }

    protected void handleScrollUpEvent() {
        handleGameActionKey(Canvas.UP);
    }

    /**
     * In VieMessageScreen the only activable item is the BodyItem. This method
     * overrides the MessageScreen implementation in order to filter items
     * activation.
     * 
     * @param y
     */
    public void activateItem(int y) {
        EditField item = null;
        for (int i = 0; i<items.size(); i++) {
            item = (EditField) items.elementAt(i);
            if(item instanceof BodyItem) {
                int startCoordinate = item.getY();
                if (y >= startCoordinate && y <= startCoordinate + item.getHeight()) {
                    activateItem(item);
                }
            }
        }
    }

    private class AttachmentCommand extends Command {

    private BodyPart bp = null;

    public AttachmentCommand(BodyPart bodyPart, int priority) {
        super(bodyPart.getFileName(), Command.SCREEN, priority);
        this.bp = bodyPart;
    }

    public BodyPart getBodyPart() {
        return bp;
    }


}
}
