/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.ui.view.components;

import com.funambol.mailclient.ui.view.*;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.TextBox;
import javax.microedition.lcdui.TextField;

public class EditField extends Item {

    //#ifdef composeLongMessage
    //# protected static final int MAX_EDITOR_SIZE = 50000;
    //#else
        protected static final int MAX_EDITOR_SIZE = 1000;
    //#endif
    protected static int CUSTOM_ITEM_COMMAND_PRIORITY = 15;
    protected Drawer d;
    protected GraphicalTheme gt;    //set the item visible or not

    private boolean isReadOnly = true;    //The item has a help message initially set to empty string

    private Command editCommand = null;    //The editor related to this item - see the inner private class
    private EditForm editor = null;    //The editor's title - see the inner private class
    private String editFormTitle = "";    //Item's labels array
    private String[] label = null;    //Item's labels related contect text array
    private String[] text = null;    //Item state listener

    protected int labelWidth = 0;
    protected Font labelFont = null;
    protected Font textFont = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_SMALL);
    protected int maxSize = 0;

    private boolean alignLabels = false;
    private int defaultScreenIndex = 0;
    //Total height of the custom item is proportional to the 
    //number of labels
    protected boolean isBackGroundNeeded = true;
    protected boolean isLabelNeeded = true;

    protected int indentX = 0;    //height of a single line of text
    protected int textHeight = 0;
    //protected int labelItemH = 0;    //Single Label frame padding and values
    protected int[] labelStringWidth = null;
    protected int[] labelFrameW = null;
    protected int labelRatio = 4;
    protected int labelFrameH = 0;    //Text after label coordinates
    protected int[] textAfterLabelX = null;    //calculates the y coordinate for every label
    protected int labelX = 0;    //Calculate the text rows wrapped after the labels
    //When the text strings are too long for the device display width
    protected Vector rows = null;
    protected int counter = 0;
    protected int textStrings = 0;
    protected int totalItemH = 0;
    protected int deltaFrame = 0;
    protected String[] textRows = null;
    protected int[] labelY = null;
    protected int[] textAfterLabelY = null;

    protected int layout = Graphics.BOTTOM | Graphics.LEFT;
    protected int labelFrameArcWidth = 10;
    protected int labelFrameArcEight = 10;
    protected int labelFrameX = 0;
    protected int labelTextX = 0;    //Set the max number of lines after wich the text will be truncated
    private int[] maxTextLinesPerLabel;    //The Edit Command label string
    private String editCommandLabel = "Edit";

    public void setEditCommand(String editCommandLabel) {
        this.editCommandLabel = editCommandLabel;
        addCommand(getEditCommand());
    }

    public EditField(Drawer d, GraphicalTheme gt) {
        this.d = d;
        this.gt = gt;
    }

    public EditField(String label, String text, Drawer d, GraphicalTheme gt) {
        this(d, gt);
        this.label = new String[1];
        this.label[0] = label;

        this.text = new String[1];
        this.text[0] = text;
    }

    public EditField(String[] label, String[] text, Drawer d, GraphicalTheme gt) {
        this(d, gt);
        this.label = new String[label.length];
        for (int i = 0; i < label.length; i++) {
            this.label[i] = label[i];
        }
        this.text = new String[text.length];
        for (int i = 0; i < label.length; i++) {
            this.text[i] = text[i];
        }
    }

    public void setDefaultScreenIndex(int defaultScreenIndex) {
        this.defaultScreenIndex = defaultScreenIndex;
    } 

    public int getDefaultScreenIndex() {
        return this.defaultScreenIndex;
    } 
    
    public Command getEditCommand() {
        if (this.editCommand == null) {
            this.editCommand = new Command(editCommandLabel, UIController.COMMAND_TYPE, CUSTOM_ITEM_COMMAND_PRIORITY);
        }
        return this.editCommand;
    }

    public EditForm getItemContentEditor() {
        if (this.editor == null) {
            this.editor = new EditForm();
        }
        this.editor.setString(getItemContent());
        return this.editor;
    }

    public void setChanged(boolean isChanged) {
        this.isChanged = isChanged;
    }

    public boolean isChanged() {
        return this.isChanged;
    }

    public void enablealignLabels(boolean alignLabels) {
        this.alignLabels = alignLabels;
    }

    public void setReadOnly(boolean isReadOnly) {
        this.isReadOnly = isReadOnly;
    }

    public boolean isReadOnly() {
        return this.isReadOnly;
    }

    public void setLabelFont(Font f) {
        this.labelFont = f;
    }

    public void setTextFont(Font f) {
        this.textFont = f;
    }

    public void setLabel(String[] label) {
        this.label = label;
        isChanged = true;
    }

    public void setLabel(String label) {

        this.label = new String[1];
        if (label != null) {
            this.label[0] = label;
        } else {
            this.label[0] = "";
        }
        isChanged = true;
    }

    public void setText(String[] text) {
        this.text = text;
        isChanged = true;
    }

    public void setMaxTextLinesPerLabel(int[] maxTextLinesPerLabel) {
        this.maxTextLinesPerLabel = maxTextLinesPerLabel;
    }

    public void setMaxTextLinesPerLabel(int maxTextLinesPerLabel) {
        this.maxTextLinesPerLabel = new int[1];
        this.maxTextLinesPerLabel[0] = maxTextLinesPerLabel;
    }

    public void setText(String text) {
        this.text = new String[1];
        if (text != null) {
            this.text[0] = text;
        } else {
            this.text[0] = "";
        }
        isChanged = true;
    }

    public boolean isBackGroundNeeded() {
        return this.isBackGroundNeeded;
    }

    public void enableBackground(boolean enable) {
        this.isBackGroundNeeded = enable;
    }

    public void enableLabel(boolean enable) {
        this.isLabelNeeded = enable;
    }

    /**
     * Get the content of this item except the label
     * @return the String representing the content of this item's text field 
     */
    public String getItemContent() {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < text.length; i++) {
            if (text[i] != null) {
                sb.append(text[i]);
            } else {
                sb.append("");
            }
        }
        return sb.toString();
    }

    /**
     * Give information about the item's content. 
     * @return boolean true if this item's text is empty, false otherwise.
     */
    public boolean isItemContentEmpty() {
        int count = 0;
        for (int i = 0; i < text.length; i++) {
            count += text[i].length();
        }
        return count == 0 ? true : false;
    }

    protected String[] getLabel() {
        return label;
    }

    public String[] getText() {
        return text;
    }

    protected void recalculate(int y, Graphics g) {
        //Set the label font:
        if (this.labelFont == null) {
            this.labelFont = gt.getLabelFont();
        }

        //Calculate the text height for every label
        indentX = labelFont.stringWidth(" ");

        //The height of the item is proportional to the
        //dimension of the frame and depends by the row of text
        textHeight = textFont.getHeight();// + textFont.getHeight()/6;

        int labelFontEight = labelFont.getHeight();
        

        //Space Between 2 labels or between a label and the previous text line
        deltaFrame = textHeight/labelRatio;

        //Single Label frame height
        labelFrameH = labelFontEight;

        //Calculate the text rows wrapped after the labels
        //When the text strings are too long for the device display width
        rows = new Vector();

        labelY = new int[label.length];

        textAfterLabelY = new int[label.length];

        counter = 0;

        textStrings = 0;

        labelStringWidth = new int[label.length];
        labelFrameW = new int[label.length];
        textAfterLabelX = new int[label.length];

        //the label frame x indentation
        labelFrameX = indentX;
        
        //calculates the y coordinate for every label
        labelX = indentX;

        //the label text x indentation
        labelTextX = 2 * indentX;

        //Calculate max size of a label in case of alignment
        int size = getMaxLabelWidth(labelFont);
        
        //Calculate coordinates for all objects
        for (int i = 0; i < label.length; i++) {
            //Calculate the x ordinate for all text lines and labels
            recalculateObjectsX(size, i);

            //The first label is drawn at a fixed y
            if (i == 0) {
                labelY[i] = y + deltaFrame;
            }

            if (i > 0) {
                //Count the number of text lines of the previous label
                int textLinesBefore = ((String[]) rows.elementAt(i - 1)).length;
                
                textStrings += textLinesBefore;

                labelY[i] = labelY[i-1] + (deltaFrame) + (textLinesBefore * textFont.getHeight());
            }

            //The text must have the same alignment of the label text
            //drawn by the drawer object
            textAfterLabelY[i] = labelY[i] + textHeight;

        }

        totalItemH = textAfterLabelY[label.length-1] - y + deltaFrame + textHeight * (((String[])rows.elementAt(label.length-1)).length-1);

        isChanged = false;
    }

    public int paint(Graphics g, int x, int y, int w) {
        
        //recalculate the objects popsitions if the content has changed
        if (isChanged) {
            recalculate(y, g);
        }

        //Draws the backgound if needed
        drawItemBackGround(g, x, y, w, totalItemH);
        
        for (int i = 0; i < label.length; i++) {
            drawItemContent(i, g);
        }

        return totalItemH + y;
    }

    public void scroll(int amount) {
    }

    public int getMaxContentSize() {
        return maxSize;
    }

    public void setMaxContentSize(int maxSize) {
        this.maxSize = maxSize;
    }

    protected void drawItemBackGround(Graphics g, int x, int y, int w, int totalItemH) {

        //draw item background
        //a two gray values orizontal bar if it is highlighted, a gradient otherwise
        if (this.isHighlighted) {
            d.drawHighlightedBodyItemBackground(g, x, y, w, totalItemH);
            //d.drawHighlightedCustomItemGradientBackground(g, x, y, w, totalItemH);
        } else {
            int startColor = this.isLighter ? gt.getCustomItemLighterBgColorStart(): gt.getCustomItemBgColorStart();
            int endColor = this.isLighter ? gt.getCustomItemLighterBgColorEnd() : gt.getCustomItemBgColorEnd();
            d.drawGradientBackground(g, x, y, w, totalItemH, startColor, endColor);
        }
    }

    protected void drawItemContent(int i, Graphics g) {
        String[] strings = (String[]) rows.elementAt(i);

        //draw the labeled text line
        //Draw the label with the text
        if (isLabelNeeded) {
            d.drawCustomItemLabel(label[i], labelFont, g, labelFrameX, labelY[i], labelFrameW[i], labelFrameH, labelFrameArcWidth, labelFrameArcEight, labelTextX, textAfterLabelY[i], layout);
            labelTextX = 2 * indentX;
        } else {
            labelTextX = indentX;
        }

        //Draws the text after the label
        d.drawCustomItemTextString(strings[0], g, textAfterLabelX[i], textAfterLabelY[i], layout, textFont, gt.getForegroundColor());
        //Draws other item strings
        if (strings.length > 1) {
            for (int j = 1; j < strings.length; j++) {
                d.drawCustomItemTextString(strings[j], g, labelTextX, textAfterLabelY[i] + j * textHeight, layout, textFont, gt.getForegroundColor());
            }
        }
    }

    protected void recalculateObjectsX(int size, int i) {

        if (alignLabels) {
            labelStringWidth[i] = size;
        } else {
            labelStringWidth[i] = labelFont.stringWidth(label[i]);
        }

        labelFrameW[i] = 2 * indentX + labelStringWidth[i];
        //Text after label coordinates
        textAfterLabelX[i] = indentX + labelFrameW[i] + indentX;

        if (!isLabelNeeded) {
            textAfterLabelX[i] = indentX;
        }

        if (textFont.stringWidth(text[i]) > screenWidth - textAfterLabelX[i]) {

            if (maxTextLinesPerLabel == null) {
                textRows = UiUtils.getCustomItemWrappedText(text[i], screenWidth - textAfterLabelX[i] - indentX, screenWidth - 3 * indentX, textFont);
            } else {
                textRows = UiUtils.getCustomItemWrappedTextGivenRowNumber(text[i], maxTextLinesPerLabel[i], screenWidth - textAfterLabelX[i] - indentX, screenWidth - 3 * indentX, textFont);
            }
            rows.addElement(textRows);
        } else {
            textRows = new String[1];
            textRows[0] = text[i];
            rows.addElement(textRows);
        }
    }

    private int getMaxLabelWidth(Font f) {
        int max = 0;
        int current = 0;
        for (int i = 0; i < label.length; i++) {
            current = f.stringWidth(label[i]);
            if (current >= max) {
                max = current;
            }
        }

        return max;
    }

    public void setEditFormTitle(String editFormTitle) {
        this.editFormTitle = editFormTitle;
    }

    public String getEditFormTitle() {
        return editFormTitle;
    }

    /**
     * @see Item#getHeight() 
     */
    public int getHeight() {
        return this.totalItemH;
    }

    /**
     * @see Item#commandAction(javax.microedition.lcdui.Command, javax.microedition.lcdui.Displayable) 
     */
    public void commandAction(Command command, Displayable displayable) {
        if(command == editCommand) {
            UIController.showScreen(getItemContentEditor(), displayable);
        }
    }

    protected class EditForm extends TextBox implements CommandListener, TestableScreen {

        private static final int OK_CMD_PRIORITY = 10;
        private static final int CANCEL_CMD_PRIORITY = 20;
        private Command okCommand = new Command(Localization.getMessages().GENERIC_OK, Command.OK, OK_CMD_PRIORITY);
        private Command cancelCommand = new Command(Localization.getMessages().CANCEL_MENU_OPTION, Command.CANCEL, CANCEL_CMD_PRIORITY);

        protected EditForm() {
            super(editFormTitle, getItemContent(), MAX_EDITOR_SIZE, TextField.ANY);
            this.setCommandListener(this);
            this.addCommands();
        }

        protected void addCommands() {
            addCommand(okCommand);
            addCommand(cancelCommand);
        }

        public void commandAction(Command c, Displayable d) {
            if (c == okCommand) {
                setText(getString());
                stateListener.notifyChanged();
                UIController.showBackScreen();
            } else if (c == cancelCommand) {
                UIController.showBackScreen();
            } else {
                Log.error("Unrecognized command");
            }
        }

        /* Automatic testing support */
//#ifdef TEST_MODE
//#     public Vector getTextFields() {
//#         Vector res = new Vector();
//#         BasicTestableInputField field = new BasicTestableInputField(this);
//#         res.addElement(field);
//#         return res;
//#     }
//#endif

//#ifdef TEST_MODE
//#     public void simulateKeySinglePress(int keycode) {
//#         // Cannot be simulated...
//#         throw new IllegalArgumentException("Cannot simulate key press");
//#     }
//#endif

//#ifdef TEST_MODE
//#     public Vector getCommands() {
//#         Vector res = new Vector();
//#         res.addElement(okCommand);
//#         res.addElement(cancelCommand);
//#         return res;
//#     }
//#endif

    }
}
