/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.ui.view.components;

import javax.microedition.lcdui.Graphics;

/**
 * A blueprint for the scrollbar. This class Abstract the scrollbar concept
 * in order it to be reusable both in screens and custom items.
 */
public class ScrollBar {

    private static final int ARC_HEIGHT = 3;
    private static final int MIN_BAR_LENGTH = 3;

    private int x = 0;
    private int y = 0;
    private int w = 0;
    private int h = 0;

    private int visibleElements = 0;
    private int totalElements = 0;

    private int relativeBarLength;

    private int relativeY = 0;

    private boolean isVisible = false;

    /**
     * Public constructor:
     * @param x the x absolute position of the scrollbar
     * @param y the y position absolute position of the scrollbar
     * @param w the width of the scrollbar
     * @param h the absolute height of the scrollbar
     * @param totalElements the number of total elements contained into the
     * scrollbar container
     * @param visibleElements the number of visibleElements contained into the
     * scrollbar container
     */
    public ScrollBar(int x, int y, int w, int h, int totalElements, int visibleElements) {
        updateScrollBar(x, y, w, h, totalElements, visibleElements);
    }

    /**
     * Update the absolute padding sizes of the scrollbar
     * Useful when the implementation uses a single instance
     * for the scrollbar object
     * @param x the x absolute position of the scrollbar
     * @param y the y position absolute position of the scrollbar
     * @param w the width of the scrollbar
     * @param h the absolute height of the scrollbar
     * @param totalElements the number of total elements contained into the
     * scrollbar container
     * @param visibleElements the number of visibleElements contained into the
     * scrollbar container
     */
    public void updateScrollBar(int x, int y, int w, int h, int totalElements, int visibleElements) {
        this.x = x;
        this.y = y;
        this.w = w;
        this.h = h;
        if (totalElements > 0) {
            relativeBarLength = (h * visibleElements) / totalElements;
        }
        if (relativeBarLength < MIN_BAR_LENGTH) {
            relativeBarLength = MIN_BAR_LENGTH;
        }
        this.totalElements = totalElements;
        this.visibleElements = visibleElements;
        this.relativeY = y;
    }

    /**
     * Update the relative bar element position given the relative item or
     * screen relative position
     * @param firstCurrentElement is the index of the first visible element
     */
    public void updateBarPosition(int firstCurrentElement) {
        int delta;
        if (totalElements > 0) {
            delta = (h * firstCurrentElement)/totalElements;
        } else {
            delta = 0;
        }
        this.relativeY = y + delta;
    }

    /**
     * If the scrollbar is visible paints it
     * @param g the graphics object
     */
    public void paint(Graphics g) {
        if (isVisible) {
            drawBarBackground(g);
            drawBar(g);
        }
    }

    /**
     * Draw the scrtollbar background
     */
    public void drawBarBackground(Graphics g) {
        g.setColor(197, 197, 197);

        int roundedAmount = 3;
        for (int j = 0; j < w; j++) {
            if (j == 0 ||j == w - 1) {
                roundedAmount = 1;
            } else if (j == 1 || j == w - 2) {
                roundedAmount = 0;
            }

            g.drawLine(x + j, y + roundedAmount, x + j, y + h - roundedAmount);
        }

    }

    /**
     * Draw the scrtollbar bar element
     */
    public void drawBar(Graphics g) {
        g.setColor(0, 128, 200);

        int roundedAmount = 3;
        for (int j = 0; j < w; j++) {
            if (j == 0 ||j == w - 1) {
                roundedAmount = 1;
            } else if (j == 1 || j == w - 2) {
                roundedAmount = 0;
            }

            int lastY = relativeY + relativeBarLength - roundedAmount;
            int maxY  = y + h - roundedAmount;
            if (lastY > maxY) {
                lastY = maxY;
            }

            g.drawLine(x + j, relativeY + roundedAmount, x + j, lastY);
        }

    }

    public void setVisible(boolean isVisible) {
        this.isVisible = isVisible;
    }
}

