/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.view.components;

import com.funambol.mailclient.ui.view.*;
import com.funambol.mailclient.ui.controller.Theme;

import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

/**
 * Title item. This element is a blueprint useful to draw a screen's title
 * It supports both a main title and a subtitle.
 */
public class Title {

    private static final int TEXT_INDENT = 2;
    private GraphicalTheme theme = null;
    private String mainTitle = null;
    private String subTitle = null;

    private int h = 0;
    private int w = 0;

    protected Image flagImage = Theme.getFLAG_IMG();
    protected Image attachmentsImage = Theme.getCLIP_IMG();
    protected Image messageImage = Theme.getOPEN_PART_IMG();

    private boolean isCounterSubstringEnabled = false;
    private boolean isMessageImageEnabled = false;
    private boolean isAttachmentsImageNeeded = false;
    private boolean isFlagImageNeeded = false;
    private String counterSubstring;

    public Title(GraphicalTheme theme, String mainTitle, String subTitle, int w) {
        this.theme = theme;
        this.w = w;
        this.mainTitle = mainTitle;
        this.subTitle = subTitle;
    }

    public void setMainTitle(String mainTitle) {
        this.mainTitle = mainTitle;
    }

    public String getMainTitle() {
        return this.mainTitle;
    }

    public void setSubTitle(String subTitle) {
        this.subTitle = subTitle;
    }

    public String getSubTitle() {
        return this.subTitle;
    }

    public int paint(Graphics g, int w, int h) {
        this.w = w;
        this.h = h;
        int imagesBeforeMainTitleW = 0;
        int imageAtTheEndOftitleW =0;
        
        if (isMessageImageEnabled) {
            imagesBeforeMainTitleW += messageImage.getWidth();
        }

        Font titleFont = theme.getTitleFont();
        Font subTitleFont = theme.getDefaultFont();

        int titleH = titleFont.getHeight() + 2 * TEXT_INDENT;

        titleH = isMessageImageEnabled ? Math.max(titleH, messageImage.getHeight()) : titleH;
        titleH = isFlagImageNeeded ? Math.max(titleH, flagImage.getHeight()) : titleH;
        titleH = isAttachmentsImageNeeded ? Math.max(titleH, attachmentsImage.getHeight()) : titleH;

        int subTitleH = subTitleFont.getHeight() + TEXT_INDENT;
        int frameLineH = TEXT_INDENT;

        //If the subtitle is needed
        h = subTitle == null? titleH + 2 * frameLineH : titleH + subTitleH + 2 * frameLineH;

        int[] color = theme.getTitleBackgroundRgbColor();
        g.setColor(color[0], color[1], color[2]);
        g.fillRect(0, 0, w, h/2);

        color = theme.getSubtitleBackgroundRGBColor();
        g.setColor(color[0], color[1], color[2]);
        g.fillRect(0, h/2, w, h/2);

        g.setColor(theme.getTitleColor());
        g.setFont(titleFont);

        if (isMessageImageEnabled) {
            int imgY = (h - messageImage.getHeight())/2;
            g.drawImage(messageImage, frameLineH + TEXT_INDENT, imgY, g.TOP|g.LEFT);
        }

        int titleY = (h - titleFont.getHeight())/2;
        if (!isCounterSubstringEnabled) {
            g.drawString(this.mainTitle, frameLineH + 2 * TEXT_INDENT + imagesBeforeMainTitleW, subTitle == null ? titleY : frameLineH, g.TOP|g.LEFT);
        } else {
            if (counterSubstring != null) {
            int sWidh = titleFont.stringWidth(counterSubstring) + frameLineH + 2 * TEXT_INDENT + imagesBeforeMainTitleW;
            String shortTitle = UiUtils.cutString(mainTitle, w - sWidh - 1, titleFont);
            g.drawString(shortTitle, frameLineH + 2 * TEXT_INDENT + imagesBeforeMainTitleW, subTitle == null ? titleY : frameLineH, g.TOP|g.LEFT);
            g.drawString(counterSubstring, w - frameLineH - 2 * TEXT_INDENT - imagesBeforeMainTitleW, subTitle == null ? titleY : frameLineH, Graphics.TOP | Graphics.RIGHT);

            }
        }

        if (isFlagImageNeeded)  {
            int imgY = (h - flagImage.getHeight())/2;
            g.drawImage(flagImage, w - flagImage.getWidth() - frameLineH - TEXT_INDENT, imgY, g.TOP|g.LEFT);
            imageAtTheEndOftitleW += flagImage.getWidth();
        }

        if (isAttachmentsImageNeeded)  {
            int imgY = (h - attachmentsImage.getHeight())/2;
            g.drawImage(attachmentsImage, w - attachmentsImage.getWidth() - frameLineH - TEXT_INDENT - imageAtTheEndOftitleW, imgY, g.TOP|g.LEFT);
        }

        if (subTitle!=null) {
            g.setColor(theme.getTitleColor());
            g.setFont(subTitleFont);
            g.drawString(this.subTitle, frameLineH + TEXT_INDENT, h/2-1 + TEXT_INDENT, g.TOP|g.LEFT);
        }

        //draw the title frame that surrounds the title
        //Internal frame
        g.setColor(theme.getTitleFrameColor()[0]);
        g.drawRect(1, 1, w-3, h-4);

        //External frame
        g.setColor(theme.getTitleFrameColor()[1]);
        g.drawRect(0, 0, w-1, h-2);

        return h;
    }

    public void enableCounterSubstring(boolean enabled) {
        this.isCounterSubstringEnabled = enabled;
    }

    public void setCounterSubstring(String counterSubstring) {
        this.counterSubstring = counterSubstring;
    }

    public void enableMessageImage(boolean enabled) {
        this.isMessageImageEnabled = enabled;
    }

    public void enableFlagImage(boolean enabled) {
        this.isFlagImageNeeded = enabled;
    }

    public void enableAttachmentsImage(boolean enabled) {
        this.isAttachmentsImageNeeded = enabled;
    }
}

