/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient;

import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.syncml.FolderData;
import com.funambol.mail.Folder;
import com.funambol.mail.AccountFolder;

import com.funambol.util.Log;

import java.util.Date;

/**
 * The MailAccount collects the information of a single mail account:
 *  - the visible name;
 *  - the email address.
 */
public class MailAccount {

    private static final String TAG_LOG = "MailAccount";
    
    public static final String DEFAULT_ACCOUNT_NAME = "DefaultAccount";

    public static final String DEFAULT_INBOX_NAME   = "Inbox";
    public static final String DEFAULT_OUTBOX_NAME  = "Outbox";
    public static final String DEFAULT_DRAFTS_NAME  = "Drafts";
    public static final String DEFAULT_SENT_NAME    = "Sent";

    public static final String DEFAULT_ACCOUNT_PATH = Folder.ROOT_FOLDER_PATH +
            DEFAULT_ACCOUNT_NAME;
    public static final String DEFAULT_INBOX_PATH  = DEFAULT_ACCOUNT_PATH +
            Folder.FOLDER_SEPARATOR + DEFAULT_INBOX_NAME;
    public static final String DEFAULT_OUTBOX_PATH = DEFAULT_ACCOUNT_PATH +
            Folder.FOLDER_SEPARATOR + DEFAULT_OUTBOX_NAME;
    public static final String DEFAULT_DRAFTS_PATH = DEFAULT_ACCOUNT_PATH +
            Folder.FOLDER_SEPARATOR + DEFAULT_DRAFTS_NAME;
    public static final String DEFAULT_SENT_PATH   = DEFAULT_ACCOUNT_PATH +
            Folder.FOLDER_SEPARATOR + DEFAULT_SENT_NAME;
    
    //------------------------------------------------------------- Private Data

    /** The account name */
    private String name;

    /** The account mail address */
    private String address;

    //------------------------------------------------------------- Constructors

    /**
     * Creates a new MailAccount, with default values.
     */
    public MailAccount() {
        name = "";
        address = "";
    }

    /**
     * Creates a new MailAccount, with the given values.
     */
    public MailAccount(String visibleName, String emailAddress) {
        name = visibleName;
        address = emailAddress;
    }
    
    //----------------------------------------------------------- Public Methods
    
    /** Returns the visible name of this MailAccount. */
    public String getVisibleName() {
        return this.name;
    }

    /** Returns the visible name of this MailAccount. */
    public String getEmailAddress() {
        return this.address;
    }

    /** Set the visible name for this MailAccount. */
    public void setVisibleName(String aName) {
        this.name = aName;
    }

    /** Set the visible name for this MailAccount. */
    public void setEmailAddress(String aAddress) {
        this.address = aAddress;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("Visible name: ").append(name)
          .append("\nEmail address: ").append(address);
        return sb.toString();
    }

    /**
     * Create a default account using the client configuration
     */
    public static void createDefaultAccount() {

        Log.debug(TAG_LOG, "Creating default account");

        MessageManager mm = MessageManager.getInstance();

        MailAccount account = UIController.getConfig().getMailAccount(0);
        
        String accountVisibleName  = account != null ? account.getVisibleName() : "";
        String accountEmailAddress = account != null ? account.getEmailAddress() : "";

        accountVisibleName  = accountVisibleName != null ? accountVisibleName : "";
        accountEmailAddress = accountEmailAddress != null ? accountEmailAddress : "";

        // Add the default account&folders
        AccountFolder defaultAccount = mm.createAccountFolder(DEFAULT_ACCOUNT_NAME, 
                new Date(), accountVisibleName, accountEmailAddress);

        // Notify the inbox of the new account created
        UIController.getInboxMessageList().newAccountAdded(defaultAccount);

        mm.createFolder(defaultAccount, DEFAULT_INBOX_NAME, FolderData.INBOX_ROLE, new Date());
        mm.createFolder(defaultAccount, DEFAULT_OUTBOX_NAME, FolderData.OUTBOX_ROLE, new Date());
        mm.createFolder(defaultAccount, DEFAULT_SENT_NAME, FolderData.SENT_ROLE, new Date());
        mm.createFolder(defaultAccount, DEFAULT_DRAFTS_NAME, FolderData.DRAFTS_ROLE, new Date());
    }
}

