/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.mm;

import java.util.Vector;

import java.io.IOException;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import java.io.InputStream;

import com.funambol.mail.Message;
import com.funambol.mail.BodyPart;
import com.funambol.storage.ComplexSerializer;
import com.funambol.storage.Serializable;

import com.funambol.util.Log;

/**
 * This class represents a formatted email with auxiliary information on
 * detached attachments. A message can be formatted using the Detached mime
 * formatter which generates a FormattedEmail object. Such an object has the
 * email body and links to the attachments, together with the position in the
 * formatted email where such attachments shall start.
 */
public class FormattedEmail implements Serializable {

    private Vector attachmentsPosition = new Vector();
    private Vector bodyParts = new Vector();
    private byte value[];

    public FormattedEmail() {
    }

    public void setPreformattedValue(byte value[]) {
        this.value = value;
    }

    public byte[] getPreformattedValue() {
        return value;
    }

    public void addAttachment(BodyPart part, int position) {
        bodyParts.addElement(part);
        attachmentsPosition.addElement(new Integer(position));
    }

    public int getNumberOfAttachmentParts() {
        return bodyParts.size();
    }

    public BodyPart getBodyPart(int idx) {
        return (BodyPart)bodyParts.elementAt(idx);
    }

    public int getAttachmentPosition(int idx) {
        Integer val = (Integer)attachmentsPosition.elementAt(idx);
        return val.intValue();
    }


    public void serialize(DataOutputStream dout) throws IOException {
        ComplexSerializer.serializeObject(dout, value);
        ComplexSerializer.serializeObject(dout, attachmentsPosition);
        ComplexSerializer.serializeObject(dout, bodyParts);
    }

    public void deserialize(DataInputStream din) throws IOException {
        value = (byte[])ComplexSerializer.deserializeObject(din);
        attachmentsPosition = (Vector)ComplexSerializer.deserializeObject(din);
        bodyParts = (Vector)ComplexSerializer.deserializeObject(din);
    }
}


