/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.mm;

import java.io.IOException;
import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;

import com.funambol.util.Log;
import com.funambol.mail.MIMEFormatter;
import com.funambol.mail.BodyPart;


public class MIMEFormatterDetachedAttachments extends MIMEFormatter {

    FormattedEmail email;

    public MIMEFormatterDetachedAttachments(FormattedEmail email) {
        this.email = email;
    }

    protected void formatBodyPartContent(BodyPart bodypart, OutputStream os, boolean encode)
    throws IOException
    {
        // Since attachments are detached, we don't format their content here,
        // but we simply keep track of the position where the content shall be
        // written
        if (os instanceof ByteArrayOutputStream) {
            ByteArrayOutputStream baos = (ByteArrayOutputStream)os;
            int position = baos.size();

            // We only track attachments that are stored in files
            if (bodypart.getAttachUrl() != null &&
                bodypart.getFileName() != null) {

                String fileName = "file:///" + bodypart.getAttachUrl() +
                    bodypart.getFileName();

                // Now we can save the attachment descriptor
                email.addAttachment(bodypart, position);
            } else {
                super.formatBodyPartContent(bodypart, os, encode);
            }
        }
    }
}

