/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.syncml;

import com.funambol.mailclient.ui.controller.ServerCapStore;
import com.funambol.syncml.protocol.Ext;
import com.funambol.mailclient.MailAccount;
import com.funambol.util.Log;

/**
 * Helper methods container. It is primarily used to mantain backward
 * compatibility issues between client and server.
 */
public class MailSyncItemHelper {

    private static final String TAG_LOG = "MailSyncItemHelper";

    /**
     * Filter the default account string from the mail items key and parents
     * MailAccount.DEFAULT_ACCOUNT_NAME. It is useful to manage messages when a
     * non MEA server is used.
     * @param key the String to be filtered
     * @param remove the boolean to indicate if the default account name related
     * string must removed or added
     * @return the given key with (remove = false) or without (remove = true)
     * the default acccount substring information
     */
    public static String filterDefaultAccount(String key, boolean remove) {

        // If the given folder key is coming from an old server, there's no need
        // to filter
        if(MailSyncSource.isOldFolderKey(key)) {
            return key;
        }

        String ret = key;
        Log.debug(TAG_LOG, "Original key: " + ret);
        boolean isMeaSupported = ServerCapStore.getInstance().isExtSupported(Ext.X_FUNAMBOL_MULTIPLE_EMAIL_ACCOUNT);
        if (isMeaSupported) {
            Log.debug(TAG_LOG, "SUPPORTED: Original key left");
            return ret;
        }

        if (!isMeaSupported && !remove){
            if (ret.indexOf(MailAccount.DEFAULT_ACCOUNT_NAME)<0) {
                Log.debug(TAG_LOG, "ADDED: Final key: " + "/" + MailAccount.DEFAULT_ACCOUNT_NAME + ret);
                return "/" + MailAccount.DEFAULT_ACCOUNT_NAME + ret;
            } else {
                Log.debug(TAG_LOG, "ALREADY EXISTS: Original key left");
                return ret;
            }
        }

        if (!isMeaSupported && remove) {
            int index = ret.indexOf(MailAccount.DEFAULT_ACCOUNT_NAME) + MailAccount.DEFAULT_ACCOUNT_NAME.length();
            if (index >= 0) {
                Log.debug(TAG_LOG, "REMOVED: Final key: " + ret.substring(index));
                return ret.substring(index);
            }
        }

        Log.debug(TAG_LOG, "LAST: Final key: " + ret);
        return ret;
    }

}
