/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.cm;

import junit.framework.*;

import com.funambol.util.Log;
import com.funambol.util.ConsoleAppender;

public class ContactTest extends TestCase {

    public ContactTest(String name) {
        super(name);
        Log.initLog(new ConsoleAppender(), Log.DEBUG);
    }
 
    public void testParseContact() throws Exception {
        Log.info("Parsing contact");

        StringBuffer vCard = new StringBuffer();
        vCard.append("BEGIN:VCARD\r\n")
             .append("VERSION:2.1\r\n")
             .append("FN:., .\r\n")
             .append("N:.;.;;;\r\n")
             .append("NAME:\r\n")
             .append("NICKNAME:\r\n")
             .append("CATEGORIES:\r\n")
             .append("CLASS:\r\n")
             .append("ADR;HOME:;;;;;;\r\n")
             .append("ADR;WORK:;;;;;;\r\n")
             .append("EMAIL;INTERNET:bt1@mobiletest.com\r\n")
             .append("GEO:0;0\r\n")
             .append("KEY:\r\n")
             .append("NOTE;ENCODING=QUOTED-PRINTABLE;CHARSET=UTF-8:\r\n")
             .append("PROFILE:VCARD\r\n")
             .append("REV:\r\n")
             .append("ROLE:\r\n")
             .append("SORT-STRING:\r\n")
             .append("SOURCE:\r\n")
             .append("ORG:;;\r\n")
             .append("TITLE:\r\n")
             .append("TEL;WORK;VOICE:.\r\n")
             .append("TZ:\r\n")
             .append("UID:10000000265109\r\n")
             .append("URL;TYPE=WORK:\r\n")
             .append("END:VCARD\r\n");

        Contact c = new Contact();
        try {
            c.parse(vCard.toString());
        } catch (Exception e) {
            Log.error("Error parsing contact: " + e.toString());
            throw e;
        }

        vCard = new StringBuffer();
        vCard.append("BEGIN:VCARD\r\n")
             .append("VERSION:2.1\r\n")
             .append("FN:Cantin, Tim\r\n")
             .append("N:Cantin;Tim;;;\r\n")
             .append("NAME:\r\n")
             .append("NICKNAME:\r\n")
             .append("CATEGORIES:\r\n")
             .append("CLASS:\r\n")
             .append("ADR;HOME:;;;;;;\r\n")
             .append("ADR;WORK:;;;;;;\r\n")
             .append("GEO:0;0\r\n")
             .append("KEY:\r\n")
             .append("NOTE;ENCODING=QUOTED-PRINTABLE;CHARSET=UTF-8:\r\n")
             .append("PROFILE:VCARD\r\n")
             .append("REV:\r\n")
             .append("ROLE:\r\n")
             .append("SORT-STRING:\r\n")
             .append("SOURCE:\r\n")
             .append("TEL;CELL:15853096483\r\n")
             .append("ORG:Bluetie;;\r\n")
             .append("TITLE:\r\n")
             .append("TZ:\r\n")
             .append("UID:\r\n")
             .append("URL;TYPE=WORK:\r\n")
             .append("END:VCARD\r\n");

        c = new Contact();
        try {
            c.parse(vCard.toString());
        } catch (Exception e) {
            Log.error("Error parsing contact: " + e.toString());
            throw e;
        }
    }
}
    
