/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.mm;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import junit.framework.*;

import com.funambol.mail.AccountFolder;
import com.funambol.mail.Address;
import com.funambol.mail.Folder;
import com.funambol.mail.MIMEFormatter;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mail.Multipart;
import com.funambol.mail.BodyPart;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Store;
import com.funambol.mail.StoreFactory;
import com.funambol.mailclient.UserAccount;
import com.funambol.mailclient.config.ConfigException;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.config.MailClientConfig;
import com.funambol.mailclient.syncml.FlagQueue;
import com.funambol.mailclient.syncml.FolderData;
import com.funambol.storage.ObjectStore;
import com.funambol.util.ConsoleAppender;
import com.funambol.util.Log;
import com.funambol.platform.FileAdapter;

/**
 * Test case for the FormattedEmailStore class.
 */
public class FormattedEmailStoreTest extends TestCase {

    private FormattedEmailStore fes;

    /**
     * Default constructor. Set the log level for all tests
     */
    public FormattedEmailStoreTest(String name) {
        super(name);
        Log.initLog(new ConsoleAppender());
        Log.setLogLevel(Log.INFO);
    }

    /**
     * Set up the tests:
     *  - Create the MessageManager instance 
     *  - Retrieve the Store instance 
     *  - Initializes a default account with default folders
     *  - Use the ConfigManager class to initialize the default configuration 
     *    creating a default account with default folders 
     *    (inbox-outbox-sent-drafts)
     */
    public void setUp() {
        fes = FormattedEmailStore.getInstance();
    }

    /**
     * Reset the system folders removing them from the system
     */
    public void tearDown() {
    }

    /**
     * Tests that the stream returned by the store has an initial available
     * value which is consistent with the number of bytes that can be read from
     * the store.
     */
    public void testAvailableSize() throws Exception {

        Message message = new Message();
        message.setKey("testGetInstanceMsg0");
        message.setSubject("AAA");
        Multipart multipart;

        if(!message.isMultipart()){
            multipart = new Multipart();

            // the first bodypart content is the text bodyof the mail
            BodyPart bodyPart= new BodyPart();
            bodyPart.addHeader(BodyPart.CONTENT_TRANSFER_ENCODING, "8bit");
            bodyPart.setContent("Body");
            multipart.addBodyPart(bodyPart);
        }else{
            multipart = (Multipart)message.getContent();
        }

        // Create a file for the attachment
        String fileDir = System.getProperty("fileconn.dir.photos");
        if (fileDir == null) {
            fileDir = "file:///root1/";
        }
        String fileUrl = fileDir + "testatt.txt";
        FileAdapter fa = new FileAdapter(fileUrl);
        OutputStream os = fa.openOutputStream();
        os.write("TestAttachment.txt".getBytes());
        os.close();
        fa.close();

        BodyPart bodyPart = new BodyPart();
        bodyPart.setDisposition(BodyPart.CD_ATTACHMENT);
        bodyPart.addHeader(BodyPart.CONTENT_TRANSFER_ENCODING, "base64");
        bodyPart.setSize("");
        bodyPart.setFileName("testatt.txt");
        // Remove the protocol if necessary
        if (fileDir.startsWith("file:///")) {
            bodyPart.setAttachUrl(fileDir.substring(8));
        } else {
            bodyPart.setAttachUrl(fileDir);
        }
        bodyPart.setContentType("text/plain");

        multipart.addBodyPart(bodyPart);
        message.setContent(multipart);

        for(int i=0;i<50;i++) {
            StringBuffer subj = new StringBuffer();
            for(int j=0;j<i;j++) {
                subj.append("A");
            }
            message.setSubject(subj.toString());
            message.setKey("testGetInstanceMsg"+i);

            // Add the message to the store
            fes.addMessage(message);
            InputStream is = fes.getMessageInputStream(message.getKey());
            int available = is.available();
            int ch = is.read();
            int len = 0;
            while(ch != -1) {
                len++;
                ch = is.read();
            }
            Log.info("Declared length: " + available + " effective length: " + len);
            assertTrue(len == available);
        }
    }
}
