/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.syncml;

import javax.microedition.lcdui.Display;

import java.io.IOException;
import java.util.Vector;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;

import com.funambol.mail.Folder;
import com.funambol.mail.AccountFolder;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Store;
import com.funambol.mail.StoreFactory;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.config.MailClientConfig;
import com.funambol.mailclient.UserAccount;
import com.funambol.mailclient.MailAccount;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.protocol.SyncMLStatus;
import com.funambol.syncml.protocol.DevInf;
import com.funambol.syncml.protocol.Ext;
import com.funambol.util.ConsoleAppender;
import com.funambol.util.MailDateFormatter;
import com.funambol.util.HttpTransportAgent;
import com.funambol.util.TransportAgent;
import com.funambol.util.CodedException;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.controller.ServerCapStore;

import junit.framework.*;

import com.funambol.util.Log;
import java.util.Date;

public class MailSyncSourceTest extends TestCase {
    
    SourceConfig sourceConfig = null;
    MailSyncSource mss = null;
    MessageManager mm = null;
    Store msgStore = null;
    
    /**
     * Creates a new instance of this testcase
     */
    public MailSyncSourceTest(String name) {
        super(name);
        Log.initLog(new ConsoleAppender());
        Log.setLogLevel(Log.INFO);
        FunTestRunner runner = FunTestRunner.getInstance();
        // We need to initialize this
        UIController.display = Display.getDisplay(runner);
        UIController.initAppProperties(runner);
    }
    
    /**
     * TestCase setUp default method
     */
    public void setUp() throws Exception {

        // Get the Store
        msgStore = StoreFactory.getStore();

        // By default all tests are with a MEA server here
        createMEAServerCapStore();

        // Init the MessageManager&Store
        mm = MessageManager.getInstance();
        mm.initStore(true);

        //
        // Prepare the Store
        // - add a default account
        // - add the related folders
        //
        mm.addFolder(new AccountFolder("/DefaultAccount", new Date(), msgStore,
                "Funambol Mail", "funambol@mail.com"));
        mm.addFolder(new Folder("/DefaultAccount/Inbox", FolderData.INBOX_ROLE,
                new Date(), msgStore));
        mm.addFolder(new Folder("/DefaultAccount/Outbox", FolderData.OUTBOX_ROLE,
                new Date(), msgStore));
        mm.addFolder(new Folder("/DefaultAccount/Sent", FolderData.SENT_ROLE,
                new Date(), msgStore));
        mm.addFolder(new Folder("/DefaultAccount/Drafts", FolderData.DRAFTS_ROLE,
                new Date(), msgStore));

        //Creates the syncSource
        sourceConfig = new SourceConfig(
                SourceConfig.MAIL, 
                SourceConfig.EMAIL_OBJECT_TYPE, 
                SourceConfig.EMAIL_OBJECT_NAME);
        
        mss = new MailSyncSource(sourceConfig);
    }
    
    /**
     * TestCase tearDown default method
     */
    public void tearDown() {
        //Delete the syncSource
        sourceConfig = null;
        mss = null;
        mm.initStore(true);
    }
    
    /**
     * Emulate all of the operations that the sync source performs on a 
     * message that is sending to the server as a reply and in the same time is 
     * flagged by the user 
     */
    public void testUpdateMessageFlagsWhileSyncing() throws Exception {
        Log.info("[MailSyncSourceTest.testUpdateMessageFlagsWhileSyncing]");
        Message m = createNewMessage("Test Message");
        mm.appendMessageToFolder(m, mm.getFolder("/DefaultAccount/Inbox"));

        String key = m.getKey();

        //set the open flag
        MessageFlags mf = new MessageFlags();
        mf.setFlag(MessageFlags.OPENED, true);
        mm.updateMessageFlags(m, mf);
        //the operation of sync initialize the updated items Hashtable
        mss.initUpdItems();
        
        //the emulated sync process updates the item as replied
        mf.setFlag(MessageFlags.ANSWERED, true);
        mm.updateMessageFlags(m, mf);
        
        //the user flags the message while sync'ing 
        mf.setFlag(MessageFlags.FLAGGED, true);
        mm.updateMessageFlags(m, mf);
        
        //emulation of the setItemStatu with the message key and the success 
        //status code
        mss.setItemStatus(key, 200);
        
        //The setItemStatus method MUST NOT remove the message from the 
        //its flags were updated during the sync. They must be sent in the next 
        //sync
        MessageFlags flags = (MessageFlags) FlagQueue.getUpdatedItemsList().get(key);
        
        flags = (MessageFlags) FlagQueue.getUpdatedItemsList().get(key);
        
        assertTrue(flags.isSet(MessageFlags.FLAGGED));
        Log.info("[MailSyncSourceTest.testUpdateMessageFlagsWhileSyncing]Successful");
    }
    
    public void testInitAllItems() throws Throwable {
        Log.info("[MailSyncSourceTest.testInitAllItems]");

        class MyMailSyncSource extends MailSyncSource {
            public MyMailSyncSource(SourceConfig config) {
                super(config);
            }
            public SyncItem[] getAllItems() { return allItems; }
        }

        Message m1 = createNewMessage("Message #1");
        Message m2 = createNewMessage("Message #2");

        mm.appendMessageToFolder(m1, mm.getFolder("/DefaultAccount/Inbox"));
        mm.appendMessageToFolder(m2, mm.getFolder("/DefaultAccount/Inbox"));

        mss = new MyMailSyncSource(sourceConfig);
        mss.initAllItems();
        SyncItem[] allItems = ((MyMailSyncSource)mss).getAllItems();

        //
        // Expected items
        // 1 AccountFolder: "/DefaultAccount"
        // 2 Folder "/DefaultAccount/Inbox" and "/DefaultAccount/Outbox"
        // 2 Message "Message #1" and "Message #2"
        //

        //
        // Since backward compatibility is not handled, the client doesn't send
        // the account folder and the folders parent is "/"
        //
        //assertTrue(allItems.length == 5);
        assertTrue(allItems.length == 4);

        //assertTrue(allItems[0].getKey().equals("/DefaultAccount") &&
        //        allItems[0].getParent().equals("/") &&
        //        allItems[0].getType().equals(MailSyncSource.FOLDER_TYPE));

        assertTrue(allItems[0].getKey().equals("/DefaultAccount/Inbox") &&
                allItems[0].getParent().equals("/") &&
                allItems[0].getType().equals(MailSyncSource.FOLDER_TYPE));

        assertTrue(allItems[1].getKey().equals("/DefaultAccount/Outbox") &&
                allItems[1].getParent().equals("/") &&
                allItems[1].getType().equals(MailSyncSource.FOLDER_TYPE));

        assertTrue(allItems[2].getKey().equals("/DefaultAccount/Inbox/2") &&
                allItems[2].getParent().equals("/DefaultAccount/Inbox") &&
                allItems[2].getType().equals(MailSyncSource.EMAIL_TYPE));

        assertTrue(allItems[3].getKey().equals("/DefaultAccount/Inbox/1") &&
                allItems[3].getParent().equals("/DefaultAccount/Inbox") &&
                allItems[3].getType().equals(MailSyncSource.EMAIL_TYPE));
        Log.info("[MailSyncSourceTest.testInitAllItems]Successful");
    }

    public void testAddItem_Message() throws Throwable {
        Log.info("[MailSyncSourceTest.testAddItem_Message]");
        SyncItem messageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Inbox", getSampleEmailData().getBytes());
        
        assertTrue(mss.addItem(messageItem) == SyncMLStatus.SUCCESS);
        Message newMessage = mm.getMessage(messageItem.getKey());

        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Inbox")) == 1);
        assertTrue(newMessage.getParent().getFullName().equals("/DefaultAccount/Inbox"));
        assertTrue(newMessage.getSubject().equals("This is the mail subject"));
        Log.info("[MailSyncSourceTest.testAddItem_Message]Successful");
    }

    public void testAddItem_Folder() throws Throwable {
        Log.info("[MailSyncSourceTest.testAddItem_Folder]");
        SyncItem folderItem = new SyncItem("guid_key",
                            MailSyncSource.FOLDER_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount", getSampleFolderData().getBytes());

        mm.deleteFolder("/DefaultAccount/Inbox");
        assertTrue(mm.getFoldersFromRole(FolderData.INBOX_ROLE).length == 0);

        assertTrue(mss.addItem(folderItem) == SyncMLStatus.SUCCESS);
        Folder newFolder = mm.getFolder(folderItem.getKey());

        assertTrue(mm.getFoldersFromRole(FolderData.INBOX_ROLE).length == 1);
        assertTrue(newFolder.getParent().getFullName().equals("/DefaultAccount"));
        assertTrue(newFolder.getRole().equals(FolderData.INBOX_ROLE));
        Log.info("[MailSyncSourceTest.testAddItem_Folder]Successful");
    }

    public void testAddItem_Account() throws Throwable {
        Log.info("[MailSyncSourceTest.testAddItem_Account]");
        SyncItem accountItem = new SyncItem("guid_key",
                            MailSyncSource.FOLDER_TYPE, SyncItem.STATE_NEW,
                            "/", getSampleAccountFolderData().getBytes());

        mm.deleteFolder("/DefaultAccount");
        assertTrue(mm.getAccounts().length == 0);

        assertTrue(mss.addItem(accountItem) == SyncMLStatus.SUCCESS);
        AccountFolder newAccount = mm.getAccounts()[0];

        assertTrue(newAccount.getParent().getFullName().equals("/"));
        assertTrue(newAccount.getRole().equals("account"));
        assertTrue(newAccount.getVisibleName().equals("Name Surname"));
        assertTrue(newAccount.getEmailAddress().equals("Name.Surname@gmail.com"));
        Log.info("[MailSyncSourceTest.testAddItem_Account]Successful");
    }
    
    public void testUpdateItem_Message() throws Throwable {
        Log.info("[MailSyncSourceTest.testUpdateItem_Message]");
        SyncItem messageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Inbox", getSampleEmailData().getBytes());

        assertTrue(mss.addItem(messageItem) == SyncMLStatus.SUCCESS);

        SyncItem updMessageItem = new SyncItem(messageItem.getKey(),
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_UPDATED,
                            messageItem.getParent(), getUpdatedEmailData().getBytes());

        assertTrue(mss.updateItem(updMessageItem) == SyncMLStatus.SUCCESS);

        Message newMessage = mm.getMessage(messageItem.getKey());

        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Inbox")) == 1);
        assertTrue(newMessage.getParent().getFullName().equals("/DefaultAccount/Inbox"));
        assertTrue((newMessage.getFlags().getFlags() & MessageFlags.OPENED) == MessageFlags.OPENED);
        assertTrue((newMessage.getFlags().getFlags() & MessageFlags.FORWARDED) == 0);
        assertTrue((newMessage.getFlags().getFlags() & MessageFlags.ANSWERED) == MessageFlags.ANSWERED);
        assertTrue((newMessage.getFlags().getFlags() & MessageFlags.DELETED) == 0);
        assertTrue((newMessage.getFlags().getFlags() & MessageFlags.FLAGGED) == 0);
        assertTrue(newMessage.getSubject().equals("This is the mail subject"));
        Log.info("[MailSyncSourceTest.testUpdateItem_Message]Successful");
    }

    public void testUpdateItem_Folder() throws Throwable {
        Log.info("[MailSyncSourceTest.testUpdateItem_Folder]");
        SyncItem updFolderItem = new SyncItem("/DefaultAccount/Inbox",
                            MailSyncSource.FOLDER_TYPE, SyncItem.STATE_UPDATED,
                            "/DefaultAccount", getUpdatedFolderData().getBytes());

        assertTrue(mss.updateItem(updFolderItem) == SyncMLStatus.SUCCESS);
        Folder folder = mm.getFolder(updFolderItem.getKey());

        assertTrue(folder.getParent().getFullName().equals("/DefaultAccount"));
        assertTrue(folder.getRole().equals(FolderData.INBOX_ROLE));
        assertTrue(MailDateFormatter.dateToUTC(
                folder.getCreated()).equals("20090428T180000Z"));
        Log.info("[MailSyncSourceTest.testUpdateItem_Folder]Successful");
    }

    public void testUpdateItem_Account() throws Throwable {
        Log.info("[MailSyncSourceTest.testUpdateItem_Account]");
        SyncItem accountItem = new SyncItem("/DefaultAccount",
                            MailSyncSource.FOLDER_TYPE, SyncItem.STATE_UPDATED,
                            "/", getUpdatedAccountFolderData().getBytes());

        assertTrue(mss.updateItem(accountItem) == SyncMLStatus.SUCCESS);
        AccountFolder updAccount = mm.getAccounts()[0];

        assertTrue(updAccount.getParent().getFullName().equals("/"));
        assertTrue(updAccount.getRole().equals("account"));
        assertTrue(updAccount.getVisibleName().equals("MyName MySurname"));
        assertTrue(updAccount.getEmailAddress().equals("email@address.com"));
        Log.info("[MailSyncSourceTest.testUpdateItem_Account]Successful");
    }

    public void testDeleteItem_Message() throws Throwable {
        Log.info("[MailSyncSourceTest.testDeleteItem_Message]");
        SyncItem messageItem1 = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Outbox", getSampleEmailData().getBytes());
        SyncItem messageItem2 = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Outbox", getSampleEmailData().getBytes());
        assertTrue(mss.addItem(messageItem1) == SyncMLStatus.SUCCESS);
        assertTrue(mss.addItem(messageItem2) == SyncMLStatus.SUCCESS);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Outbox")) == 2);
        assertTrue(mss.deleteItem(messageItem1.getKey()) == SyncMLStatus.SUCCESS);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Outbox")) == 1);
        assertTrue(mss.deleteItem(messageItem2.getKey()) == SyncMLStatus.SUCCESS);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Outbox")) == 0);
        Log.info("[MailSyncSourceTest.testDeleteItem_Message]Successful");
    }

    public void testDeleteItem_Folder() throws Throwable {
        Log.info("[MailSyncSourceTest.testDeleteItem_Folder]");
        assertTrue(mm.getFoldersFromRole(FolderData.DRAFTS_ROLE).length == 1);
        assertTrue(mss.deleteItem("/DefaultAccount/Drafts") == SyncMLStatus.SUCCESS);
        assertTrue(mm.getFoldersFromRole(FolderData.DRAFTS_ROLE).length == 0);
        Log.info("[MailSyncSourceTest.testDeleteItem_Folder]");
    }

    public void testDeleteItem_Account() throws Throwable {
        Log.info("[MailSyncSourceTest.testDeleteItem_Account]");
        assertTrue(mm.getAccounts().length == 1);
        assertTrue(mss.deleteItem("/DefaultAccount") == SyncMLStatus.SUCCESS);
        assertTrue(mm.getAccounts().length == 0);
        assertTrue(mm.getFoldersFromRole(FolderData.INBOX_ROLE).length == 0);
        assertTrue(mm.getFoldersFromRole(FolderData.OUTBOX_ROLE).length == 0);
        assertTrue(mm.getFoldersFromRole(FolderData.SENT_ROLE).length == 0);
        assertTrue(mm.getFoldersFromRole(FolderData.DRAFTS_ROLE).length == 0);
        Log.info("[MailSyncSourceTest.testDeleteItem_Account]Successful");
    }

    public void testAddFolder_Inbox_fromOldServer() throws Exception {
        Log.info("[MailSyncSourceTest.testAddFolder_Inbox_fromOldServer]");
        createSEAServerCapStore();
        SyncItem folderItem = new SyncItem("ROOT/I",
                            MailSyncSource.FOLDER_TYPE, SyncItem.STATE_NEW,
                            "ROOT/I", getSampleFolderData().getBytes());
        assertTrue(mss.addItem(folderItem) == SyncMLStatus.SUCCESS);
        assertTrue(folderItem.getKey().equals("/Inbox"));
        Log.info("[MailSyncSourceTest.testAddFolder_Inbox_fromOldServer]Successful");
    }

    public void testAddFolder_Outbox_fromOldServer() throws Exception {
        Log.info("[MailSyncSourceTest.testAddFolder_Outbox_fromOldServer]");
        createSEAServerCapStore();
        SyncItem folderItem = new SyncItem("ROOT/O",
                            MailSyncSource.FOLDER_TYPE, SyncItem.STATE_NEW,
                            "ROOT/O", getSampleFolderData().getBytes());
        assertTrue(mss.addItem(folderItem) == SyncMLStatus.SUCCESS);
        assertTrue(folderItem.getKey().equals("/Outbox"));
        Log.info("[MailSyncSourceTest.testAddFolder_Outbox_fromOldServer]Successful");
    }

    public void testAddMessage_fromOldServer1() throws Exception {
        Log.info("[MailSyncSourceTest.testAddMessage_fromOldServer1]");
        createSEAServerCapStore();
        SyncItem messageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/Inbox", getSampleEmailData().getBytes());
        assertTrue(mss.addItem(messageItem) == SyncMLStatus.SUCCESS);
        assertTrue(messageItem.getKey().equals("/Inbox/1"));

        messageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/Outbox", getSampleEmailData().getBytes());
        assertTrue(mss.addItem(messageItem) == SyncMLStatus.SUCCESS);
        assertTrue(messageItem.getKey().equals("/Outbox/1"));
        Log.info("[MailSyncSourceTest.testAddMessage_fromOldServer1]Successful");
    }
    
    public void testAddMessage_fromOldServer2() throws Exception {
        Log.info("[MailSyncSourceTest.testAddMessage_fromOldServer2]");
        createSEAServerCapStore();
        SyncItem messageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "ROOT/I", getSampleEmailData().getBytes());
        assertTrue(mss.addItem(messageItem) == SyncMLStatus.SUCCESS);
        assertTrue(messageItem.getKey().equals("/Inbox/1"));

        messageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "ROOT/O", getSampleEmailData().getBytes());
        assertTrue(mss.addItem(messageItem) == SyncMLStatus.SUCCESS);
        assertTrue(messageItem.getKey().equals("/Outbox/1"));
        Log.info("[MailSyncSourceTest.testAddMessage_fromOldServer2]Successful");
    }
    
    public void testAddMessage_fromOldServer3() throws Exception {
        Log.info("[MailSyncSourceTest.testAddMessage_fromOldServer3]");
        createSEAServerCapStore();
        SyncItem messageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "Inbox", getSampleEmailData().getBytes());
        assertTrue(mss.addItem(messageItem) == SyncMLStatus.SUCCESS);
        assertTrue(messageItem.getKey().equals("/Inbox/1"));

        messageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "Outbox", getSampleEmailData().getBytes());
        assertTrue(mss.addItem(messageItem) == SyncMLStatus.SUCCESS);
        assertTrue(messageItem.getKey().equals("/Outbox/1"));
        Log.info("[MailSyncSourceTest.testAddMessage_fromOldServer3]Successful");
    }

    /**
     * Test the beginSync method when a default account has to be created
     */
    public void testBeginSync_createDefaultAccount() throws Exception {
        Log.info("[MailSyncSourceTest.testBeginSync_createDefaultAccount]");

        MailClientConfig config = new MailClientConfig();
        MailAccount account = new MailAccount();
        account.setVisibleName("Funambol Mail");
        account.setEmailAddress("funambol@mail.com");
        config.setMailAccount(account, false);
        ConfigManager.setMailClientConfig(config);

        DevInf devInf = new DevInf();
        String xval[] = new String[0];
        Ext ext = new Ext(Ext.X_FUNAMBOL_SMARTSLOW, xval);
        Vector exts = new Vector();
        exts.addElement(ext);
        devInf.setExts(exts);
        ServerCapStore.getInstance().save(devInf);

        mm.initStore(true);

        assertTrue(mm.getAccounts().length == 0);
        //fakeTa.setResponse("Ext=X-funambol-multiple-email-account");
        // Simulate an empty response
        //fakeTa.setResponse("");
        mss.beginSync(SyncML.ALERT_CODE_SLOW);
        assertTrue(mm.getAccounts().length == 1);

        AccountFolder defaultAccount = mm.getAccounts()[0];
        assertTrue(defaultAccount.getParent().getFullName().equals("/"));
        assertTrue(defaultAccount.getRole().equals("account"));
        assertTrue(defaultAccount.getVisibleName().equals("Funambol Mail"));
        assertTrue(defaultAccount.getEmailAddress().equals("funambol@mail.com"));

        assertTrue(mm.getFoldersFromRole(FolderData.INBOX_ROLE).length == 1);
        assertTrue(mm.getFoldersFromRole(FolderData.OUTBOX_ROLE).length == 1);
        assertTrue(mm.getFoldersFromRole(FolderData.DRAFTS_ROLE).length == 1);
        assertTrue(mm.getFoldersFromRole(FolderData.SENT_ROLE).length == 1);

        Log.info("[MailSyncSourceTest.testBeginSync_createDefaultAccount]Successful");
    }

    /**
     * Test the beginSync method when a refresh from server is required: test
     * if the account inbox is reset correctly
     */
    public void testBeginSync_RefreshFromServer_SingleInbox() throws Exception {
        Log.info("[MailSyncSourceTest.testBeginSync_RefreshFromServer_SingleInbox]");

        SyncItem inboxMessageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Inbox", getSampleEmailData().getBytes());
        SyncItem outboxMessageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Outbox", getSampleEmailData().getBytes());
        SyncItem draftsMessageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Drafts", getSampleEmailData().getBytes());
        SyncItem sentMessageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Sent", getSampleEmailData().getBytes());

        // Adding messages to all the folders
        for(int i=0; i<10; i++) {
            mss.addItem(inboxMessageItem);
            mss.addItem(outboxMessageItem);
            mss.addItem(draftsMessageItem);
            mss.addItem(sentMessageItem);
        }
        
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Inbox")) == 10);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Outbox")) == 10);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Drafts")) == 10);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Sent")) == 10);

        mss.beginSync(SyncML.ALERT_CODE_REFRESH_FROM_SERVER);

        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Inbox")) == 0);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Outbox")) == 10);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Drafts")) == 10);
        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Sent")) == 10);
        
        Log.info("[MailSyncSourceTest.testBeginSync_RefreshFromServer_SingleInbox]Successful");
    }

    /**
     * Test the beginSync method when a refresh from server is required: test
     * if the accounts inboxes are reset correctly
     */
    public void testBeginSync_RefreshFromServer_MultipleInbox() throws Exception {
        Log.info("[MailSyncSourceTest.testBeginSync_RefreshFromServer_MultipleInbox]");

        // Add other 3 accounts with relative folders
        mm.addFolder(new AccountFolder("/Account1", new Date(), msgStore, "Funambol Mail1", "funambol@mail1.com"));
        mm.addFolder(new Folder("/Account1/Inbox", FolderData.INBOX_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account1/Outbox", FolderData.OUTBOX_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account1/Sent", FolderData.SENT_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account1/Drafts", FolderData.DRAFTS_ROLE, new Date(), msgStore));

        mm.addFolder(new AccountFolder("/Account2", new Date(), msgStore, "Funambol Mail2", "funambol@mail2.com"));
        mm.addFolder(new Folder("/Account2/Inbox", FolderData.INBOX_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account2/Outbox", FolderData.OUTBOX_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account2/Sent", FolderData.SENT_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account2/Drafts", FolderData.DRAFTS_ROLE, new Date(), msgStore));

        mm.addFolder(new AccountFolder("/Account3", new Date(), msgStore, "Funambol Mail2", "funambol@mail2.com"));
        mm.addFolder(new Folder("/Account3/Inbox", FolderData.INBOX_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account3/Outbox", FolderData.OUTBOX_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account3/Sent", FolderData.SENT_ROLE, new Date(), msgStore));
        mm.addFolder(new Folder("/Account3/Drafts", FolderData.DRAFTS_ROLE, new Date(), msgStore));

        SyncItem defaultAccountMessageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/DefaultAccount/Inbox", getSampleEmailData().getBytes());
        SyncItem account1MessageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/Account1/Inbox", getSampleEmailData().getBytes());
        SyncItem account2MessageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/Account2/Inbox", getSampleEmailData().getBytes());
        SyncItem account3MessageItem = new SyncItem("guid_key",
                            MailSyncSource.EMAIL_TYPE, SyncItem.STATE_NEW,
                            "/Account3/Inbox", getSampleEmailData().getBytes());

        // Adding messages to all the folders
        for(int i=0; i<25; i++) {
            mss.addItem(defaultAccountMessageItem);
            mss.addItem(account1MessageItem);
            mss.addItem(account2MessageItem);
            mss.addItem(account3MessageItem);
        }

        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Inbox")) == 25);
        assertTrue(mm.getMessagesCount(mm.getFolder("/Account1/Inbox")) == 25);
        assertTrue(mm.getMessagesCount(mm.getFolder("/Account2/Inbox")) == 25);
        assertTrue(mm.getMessagesCount(mm.getFolder("/Account3/Inbox")) == 25);

        mss.beginSync(SyncML.ALERT_CODE_REFRESH_FROM_SERVER);

        assertTrue(mm.getMessagesCount(mm.getFolder("/DefaultAccount/Inbox")) == 0);
        assertTrue(mm.getMessagesCount(mm.getFolder("/Account1/Inbox")) == 0);
        assertTrue(mm.getMessagesCount(mm.getFolder("/Account2/Inbox")) == 0);
        assertTrue(mm.getMessagesCount(mm.getFolder("/Account3/Inbox")) == 0);

        Log.info("[MailSyncSourceTest.testBeginSync_RefreshFromServer_MultipleInbox]Successful");
    }

    private Message createNewMessage(String subject) {
        Message m = new Message();
        Address from = new Address(Address.FROM, "from@funambol.com");
        Address[] to = new Address[1];
        to[0] = new Address(Address.TO, "to@funambol.com");
        Address[] cc = new Address[1];
        cc[0] = new Address(Address.CC, "cc@funambol.com");
        Address[] bcc = new Address[1];
        bcc[0] = new Address(Address.BCC, "bcc@funambol.com");
        Address[] replyTo = new Address[1];
        replyTo[0] = new Address(Address.REPLYTO, "reply@funambol.com");

        m.setFrom(from);
        m.setTo(to);
        m.setCc(cc);
        m.setBcc(bcc);
        m.setReplyTo(replyTo);

        String content = "message content";
        m.setContent(content);

        m.setSubject(subject);

        return m;
    }

    private void createMEAServerCapStore() throws IOException {
        DevInf devInf = new DevInf();

        Vector exts = new Vector();
        Ext smart = new Ext();
        smart.setXNam(Ext.X_FUNAMBOL_SMARTSLOW);
        Ext mea   = new Ext();
        mea.setXNam(Ext.X_FUNAMBOL_MULTIPLE_EMAIL_ACCOUNT);
        exts.addElement(smart);
        exts.addElement(mea);
        devInf.setExts(exts);

        ServerCapStore.getInstance().save(devInf);
    }

    private void createSEAServerCapStore() throws IOException {
        DevInf devInf = new DevInf();

        Vector exts = new Vector();
        Ext smart = new Ext();
        smart.setXNam(Ext.X_FUNAMBOL_SMARTSLOW);
        exts.addElement(smart);
        devInf.setExts(exts);

        ServerCapStore.getInstance().save(devInf);
    }


    private String getSampleEmailData() {
        return "<Email>" +
               "<read>false</read><forwarded>false</forwarded>" +
               "<replied>false</replied>" +
               "<created>20090423T073147Z</created>" +
               "<modified>20090423T073147Z</modified>" +
               "<deleted>false</deleted>" +
               "<flagged>false</flagged>" +
               "<emailitem>" +
               "Received: by 192.168.1.19 with SMTP id x19cs44999fgb;" +
               "Thu, 23 Apr 2009 00:42:38 -0700 (PDT) \r\n" +
               "Date: Thu, 23 Apr 2009 09:31:47 +0200 (CEST) \r\n" +
               "From: news@newsletter.com \r\n" +
               "Reply-To: noreply@newsletter.com \r\n" +
               "To: Name.Surname@gmail.com \r\n" +
               "Message-ID: <g081de34g0000000kb72435569.0._.magnews@smtpted> \r\n" +
               "Subject: This is the mail subject \r\n" +
               "MIME-Version: 1.0 \r\n" +
               "Content-Type: text/plain; \r\n" +
               "Content-Transfer-Encoding: 8bit \r\n\r\n " +
               "This is the mail body.\r\n" +
               "]]>" +
               "</emailitem>" +
               "</Email>";
    }

    // Get the updated message (read&replied flags changed)
    private String getUpdatedEmailData() {
        return "<Email>" +
               "<read>true</read><forwarded>false</forwarded>" +
               "<replied>true</replied>" +
               "<created>20090423T073147Z</created>" +
               "<modified>20090423T073147Z</modified>" +
               "<deleted>false</deleted>" +
               "<flagged>false</flagged>" +
               "</Email>";
    }

    private String getSampleFolderData() {
        return  "<Folder>" +
                "<name>Inbox</name>" +
                "<created>20090428T162654Z</created>" +
                "<role>inbox</role>" +
                "</Folder>";
    }

    private String getUpdatedFolderData() {
        return  "<Folder>" +
                "<name>Inbox</name>" +
                "<created>20090428T180000Z</created>" +
                "<role>inbox</role>" +
                "</Folder>";
    }

    private String getSampleAccountFolderData() {
        return  "<Folder>" +
                "<name>Gmail Home</name>" +
                "<created>20090428T162654Z</created>" +
                "<role>account</role>" +
                "<Ext>" +
                "<XNam>VisibleName</XNam>" +
                "<XVal>Name Surname</XVal>" +
                "</Ext>" +
                "<Ext>" +
                "<XNam>EmailAddress</XNam>" +
                "<XVal>Name.Surname@gmail.com</XVal>" +
                "</Ext>" +
                "</Folder>";
    }

    private String getUpdatedAccountFolderData() {
        return  "<Folder>" +
                "<name>DefaultAccount</name>" +
                "<created>20090428T162654Z</created>" +
                "<role>account</role>" +
                "<Ext>" +
                "<XNam>VisibleName</XNam>" +
                "<XVal>MyName MySurname</XVal>" +
                "</Ext>" +
                "<Ext>" +
                "<XNam>EmailAddress</XNam>" +
                "<XVal>email@address.com</XVal>" +
                "</Ext>" +
                "</Folder>";
    }
}
