/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient;

import com.funambol.mail.MailException;
import com.funambol.mailclient.ui.controller.UIController;

import com.funambol.mailclient.ui.view.SplashScreen;
import com.funambol.mailclient.ui.controller.LifeCycleController;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManager;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.syncml.FolderData;
import com.funambol.util.Log;
import com.funambol.util.ConsoleAppender;
import com.funambol.storage.AbstractRecordStore;
import com.funambol.storage.ObjectStore;
import com.funambol.mail.AccountFolder;
import com.funambol.mail.Folder;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncSource;
import javax.microedition.midlet.MIDletStateChangeException;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;
import javax.microedition.lcdui.Display;

/**
 * Engine run the Email Client Integration tests. Extends the Funambol MIDlet
 * and runs it before starting the test. This class is the core of the client
 * test framework.
 */
public class FunambolTest extends Funambol {

    public static final int TEST_SCREEN_TIME = 2000;

    private static final boolean SYNC_AT_STARTUP = false;

    private Vector testCases = new Vector();

    private String[] testAccountNames = {"DefaultAccount", "Funambol", "Gmail", "Yahoo" };
    
    private static SplashScreen ss = null;

    MessageManager mm = null;

    ContactManager cm = null;

    long initTime = 0;

    private static FunambolTest instance = null;
    
    public FunambolTest() {
        super();
        mm = MessageManager.getInstance();
        testCases = TestFactory.getTestList();
        ss = new SplashScreen();
    }

    private static Funambol funambolApp;
    private static boolean  startSuper = false;
    private static boolean  superStarted = false;

    /**
     * Overrides the parent startApp method: launches the Funambol Application
     * and perform the tests
     */
    protected void startApp() {

        if (startSuper) {
            if (!superStarted) {
                Display.getDisplay(this).setCurrent(splashScreen);
                superStarted = true;
                super.startApp();
            }
        } else {
            // Set the spalsh screen
            instance = this;
            ss.setInitPhase("Running automatic tests", 0);
            Display.getDisplay(this).setCurrent(ss);

            Log.initLog(new ConsoleAppender());
            Log.setLogLevel(Log.TRACE);
            funambolApp = this;
            IntegrationTest it = new IntegrationTest();
            it.setPriority(Thread.MAX_PRIORITY);
            it.start();
        }
    }

    protected void pauseApp() {
        // Ignore this event
    }

    public static void startMainApp() {
        startSuper = true;
        funambolApp.startApp();
    }

    public static String getProperty(String name) {
        return funambolApp.getAppProperty(name);
    }

    public static void updateSplashScreen(String msg) {
        ss.setInitPhase(msg, 0);
    }

    public static FunambolTest getInstance() {
        return instance;
    }

    /**
     * Exit the application. The exit method is invoked by the Controller
     * to exit the MIDlet
     */
     public void exit() throws MIDletStateChangeException {
         exit(false);
     }


    private class IntegrationTest extends Thread {

        public void run() {
            // Run test suite
            runTestSuite();
            // Create the report and display it
            TestReport report = new TestReport("Test Report", generateTestReport(), UIController.midlet);
            UIController.display.setCurrent(report);
            // Sleep here for some time, so that the report is left visible and
            // the sync for sending it has time to terminate
            // TODO FIXME: in autotest mode the midlet is restarted at the
            // end.... so don't let it end for the time being
            while(true) {
                try {
                    Thread.sleep(120 * 1000);
                } catch (Exception e) {
                }
            }
        }
    }
    
    private void runTestSuite() {
        Log.info("runTestSuite");

        initTime = System.currentTimeMillis();
        TestCase tc = null;

        for (int i = 0; i < testCases.size(); i++) {
            tc = (TestCase) testCases.elementAt(i);
            tc.runTest();
        }
        Log.info("runTestSuite end");
    }

    private String[] generateTestReport() {
        String[] report = new String[testCases.size()+1];
        for (int i=0; i< testCases.size(); i++) {
            TestCase test = (TestCase) testCases.elementAt(i);
            String value = test.getTestResult() == TestCase.RESULT_OK? " - Passed" : " - Failed";
            report[i] = test.getTestName() + value;
        }
        report[testCases.size()] = "Execution Time: " + getCompletedTime(System.currentTimeMillis() - initTime);
        return report;
    }

    private static String getCompletedTime(long delay) {
        
        final int H = 3600000;
        final int M = 60000;
        final int S = 1000;

        int hours   = (int)(delay/H);
        int minutes = (int)((delay-hours*H)/M);
        int seconds = (int)((delay-hours*H-minutes*M)/S);

        StringBuffer time = new StringBuffer();
        time.append(hours).append("H ").append(minutes).append("M ")
                .append(seconds).append("S ");
        return time.toString();
    }
}

