/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient;

import java.io.IOException;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Display;

import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.view.TestableScreen;
import com.funambol.mailclient.ui.view.TestableInputField;
import com.funambol.mailclient.ui.view.ContactList;
import com.funambol.mailclient.ui.view.MessageList;
import com.funambol.mailclient.ui.view.ContactItem;
import com.funambol.mailclient.ui.view.AlertScreen;
import com.funambol.mailclient.ui.view.ComposeMessageScreen;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.config.MailClientConfig;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.UserAccount;

import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.mail.AccountFolder;
import com.funambol.mail.StoreFactory;
import com.funambol.mail.Store;
import com.funambol.mail.Folder;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Address;

import com.funambol.mailclient.ui.controller.AttachFileListener;
import com.funambol.platform.FileAdapter;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Date;
import java.util.Enumeration;

import java.io.InputStream;
import java.io.OutputStream;

/**
 */
public class Robot {
   
    private static final String TAG_LOG = "Robot";
    private static Robot instance = null;

    private static final int COMMAND_ACTION_DELAY = 1000;
    private static final int WAIT_SCREEN_DELAY    = 1000;
    private static final int WAIT_SYNC_DELAY      = 100;
    
    private Hashtable textFieldMap = new Hashtable();
    
    private Robot() {
    }
    
    public static Robot getInstance() {
        if (instance==null) {
            instance = new Robot();
        }
        return instance;
    }

    private void addScreenTextFields(TestableScreen screen) {
        // Replace if it existed already
        String fullName = screen.getClass().getName();
        String components[] = StringUtil.split(fullName, ".");
        String name = components[components.length - 1];
        Log.debug(TAG_LOG, "Adding commands for screen: " + name);
        textFieldMap.put(name, screen.getTextFields());
    }

    public void simulateCommandAction(String label) throws Throwable {
        //Get the current screen
        Log.debug(TAG_LOG, "simulating command action: " + label);
        
        Displayable c = (Displayable) UIController.display.getCurrent();

        String fullName = c.getClass().getName();
        String components[] = StringUtil.split(fullName, ".");
        String name = components[components.length - 1];

        Vector commands = null;
        if (c instanceof TestableScreen) {
            TestableScreen ts = (TestableScreen)c;
            commands = ts.getCommands();
        } else {
            String msg = "Cannot simulate a command on a non testable screen";
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        
        Command cmd = null;
        
        //Get the cmd labeleb as the input argument
        for (int i = 0; i<commands.size(); i++) {
            Command tmp = (Command) commands.elementAt(i);
            Log.debug(TAG_LOG, "Found command: " + tmp.getLabel());
            if (tmp.getLabel().equals(label)) {
                cmd = tmp;
                break;
            }
        }
        
        if (cmd != null) {
            Log.trace(TAG_LOG, "invoking command " + cmd.getLabel());
            performCommandAction((CommandListener)c, cmd);
        } else {
            String msg = "Cannot run the \"" + label + "\" command on " + c.getClass();
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
    }

    public void waitForScreen(String screenName, int delay) throws Throwable {
        TestableScreen screen = waitForTestableScreen(screenName, delay);
    }

    public void waitForSyncToComplete(int minStart, int max) throws Throwable {
        SyncClient syncClient = SyncClient.getSyncClient();
        // We wait no more than minStart for sync client to start
        while(!syncClient.isBusy()) {
            Thread.sleep(WAIT_SYNC_DELAY);
            minStart -= WAIT_SYNC_DELAY;
            if (minStart < 0) {
                throw new ClientTestException("Sync did not start within time limit");
            }
        }

        // Now wait until the busy is in progress for a max amount of time
        while(syncClient.isBusy()) {
            Thread.sleep(WAIT_SYNC_DELAY);
            max -= WAIT_SYNC_DELAY;
            if (max < 0) {
                throw new ClientTestException("Sync did not complete before timeout");
            }
        }
    }

    public void setTextField(String fieldName, String value) throws Throwable {
        Log.debug(TAG_LOG, "simulating text field entering: " + fieldName + "," + value);
        Displayable c = (Displayable) UIController.display.getCurrent();

        String fullName = c.getClass().getName();
        String components[] = StringUtil.split(fullName, ".");
        String name = components[components.length - 1];

        Vector fields = null;
        if (c instanceof TestableScreen) {
            TestableScreen ts = (TestableScreen)c;
            fields = ts.getTextFields();
        } else {
            String msg = "Cannot simulate a text field setting on a non testable screen";
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        TestableInputField field = null;
        
        //Get the cmd label as the input argument
        for (int i = 0; i<fields.size(); i++) {
            TestableInputField tmp = (TestableInputField) fields.elementAt(i);
            if (tmp.getLabel().equals(fieldName)) {
                field = tmp;
                break;
            }
        }
        
        if (field != null) {
            Log.trace(TAG_LOG, "setting text field " + field.getLabel());
            field.setText(value);
        } else {
            String msg = "Cannot set the \"" + fieldName + "\" field on " + c.getClass();
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
    }

    public void saveAccount(String url, String user, String pwd) throws Throwable {
        Log.debug(TAG_LOG, "saveAccount " + url + "," + user + "," + pwd);

        MailClientConfig config = null;
        try {
            config = ConfigManager.getConfig(FunambolTest.getInstance());
        } catch (Exception e) {
            Log.error("Invalid mail client configuration: " +
                    "creating a default configuration");
            config = new MailClientConfig();
            e.printStackTrace();
        } finally {
            UIController.mailClientConfig = config;
        }
        UserAccount account = new UserAccount();
        account.setUrl(url);
        account.setUser(user);
        account.setPassword(pwd);
        config.setUserAccount(account);
        ConfigManager.saveConfig(config);
    }

    public void checkListCount(String listName, int count) throws Throwable {
        Log.debug(TAG_LOG, "checkListCount " + listName + "," + count);
        Displayable c = (Displayable) UIController.display.getCurrent();
        if (c instanceof ContactList) {
            ContactList list = (ContactList)c;
            // TODO: check this is the right list
            if (list.size() != count) {
                throw new ClientTestException("Wrong size for list " + listName
                                              + "(" + list.size() + "," + count + ")");
            }
        } else if (c instanceof MessageList) {
            MessageList list = (MessageList)c;
            // TODO: check this is the right list
            if (list.size() != count) {
                throw new ClientTestException("Wrong size for list " + listName
                                              + "(" + list.size() + "," + count + ")");
            }
        } else {
            throw new ClientTestException("Cannot count items for a non List object");
        }
    }

    public void checkActiveContactProperty(String propName, String value) throws Throwable {
        Log.debug(TAG_LOG, "checkActiveContactProperty " + propName + "," + value);
        Displayable c = (Displayable) UIController.display.getCurrent();
        if (c instanceof ContactList) {
            ContactList list = (ContactList)c;
            ContactItem item = list.getActiveContactItem();
            Contact     contact = item.getContact();
            String      property = null;
            if (propName.equals("FirstName")) {
                property = contact.getFirstName();
            } else if (propName.equals("LastName")) {
                property = contact.getLastName();
            } else if (propName.equals("VisibleName")) {
                property = contact.getVisibleName();
            } else if (propName.equals("DefaultEmail")) {
                property = contact.getDefaultEmail();
            }

            if (property == null) {
                throw new ClientTestException("Unknown contact field name " + propName);
            }
            Log.debug(TAG_LOG, "property=" + property + " value=" + value);
            // TODO: check this is the right list
            if (!value.equals(property)) {
                throw new ClientTestException("Wrong value in contact for " +
                                              propName + " is " + property + " expected " + value);
            }
        } else {
            throw new ClientTestException("Contact list is not on display " + propName);
        }
    }

    public void checkActiveMessageProperty(String propName, String value) throws Throwable {
        Log.debug(TAG_LOG, "checkActiveMessageProperty " + propName + "," + value);
        Displayable c = (Displayable) UIController.display.getCurrent();
        if (c instanceof MessageList) {
            MessageList list = (MessageList)c;
            Message message = list.getSelectedMessage();
            if (message == null) {
                throw new ClientTestException("No active message found");
            }
            String property = null;
            if (propName.equals("From")) {
                Address from = message.getFrom();
                property = from.getEmail();
            } else if (propName.equals("DisplayName")) {
                Address from = message.getFrom();
                property = from.getName();
            } else {
                throw new ClientTestException("Unknown property " + value);
            }

            if (property == null) {
                throw new ClientTestException("Unknown message field name " + propName);
            }
            Log.debug(TAG_LOG, "property=" + property + " value=" + value);
            // TODO: check this is the right list
            if (!value.equals(property)) {
                throw new ClientTestException("Wrong value in message for " +
                                              propName + " is " + property + " expected " + value);
            }
        } else {
            throw new ClientTestException("Message list is not on display " + propName);
        }
    }

    public void keySinglePress(String keyName, String keyChar) throws Throwable {

        int keyCode = 0;
        boolean code = false;
        Log.debug(TAG_LOG, "keySinglePress: " + keyName + "," + keyChar);

        if (ScriptRunner.ARROW_DOWN_KEY_NAME.equals(keyName)) {
            keyCode = Canvas.DOWN;
        } else if (ScriptRunner.ARROW_UP_KEY_NAME.equals(keyName)) {
            keyCode = Canvas.UP;
        } else if (ScriptRunner.ARROW_LEFT_KEY_NAME.equals(keyName)) {
            keyCode = Canvas.LEFT;
        } else if (ScriptRunner.ARROW_RIGHT_KEY_NAME.equals(keyName)) {
            keyCode = Canvas.RIGHT;
        } else if (ScriptRunner.FIRE_KEY_NAME.equals(keyName)) {
            keyCode = Canvas.FIRE;
        } else if (ScriptRunner.KEY_CHAR_KEY_NAME.equals(keyName)) {
            if (keyChar == null || keyChar.length() != 1) {
                throw new ClientTestException("Invalid key character " + keyChar);
            }
            keyCode = (int)keyChar.charAt(0);
            code = true;
        } else {
            Log.error(TAG_LOG, "unknown key name: " + keyName);
            throw new ClientTestException("unknown key name");
        }

        //Get the current screen
        Log.debug(TAG_LOG, "simulating single key press: " + keyCode);
        
        Displayable c = (Displayable) UIController.display.getCurrent();

        String fullName = c.getClass().getName();
        String components[] = StringUtil.split(fullName, ".");
        String name = components[components.length - 1];

        if (c instanceof TestableScreen) {
            TestableScreen ts = (TestableScreen)c;
            if (ts instanceof Canvas) {
                if (!code) {
                    keyCode = ((Canvas)ts).getKeyCode(keyCode);
                }
                Log.debug(TAG_LOG, "simulating single key press: " + keyCode);
            }
            ts.simulateKeySinglePress(keyCode);
        } else {
            Log.error(TAG_LOG, "Cannot send key press event to non testable scren: " + name);
        }
    }

    public void checkAlertTitle(String title) throws Throwable {
        Displayable c = (Displayable) UIController.display.getCurrent();

        if (c instanceof AlertScreen) {
            AlertScreen screen = (AlertScreen)c;
            if (!title.equals(screen.getTitle())) {
                throw new ClientTestException("AlertScreen has title " + screen.getTitle() + ", expected " + title);
            }
        } else {
            throw new ClientTestException("Cannot check title for screen which is not an alert");
        }
    }

    public void checkAlertMessage(String message) throws Throwable {
        Displayable c = (Displayable) UIController.display.getCurrent();

        if (c instanceof AlertScreen) {
            AlertScreen screen = (AlertScreen)c;
            if (!message.equals(screen.getMessage())) {
                throw new ClientTestException("AlertScreen has message " + screen.getMessage() + ", expected " + message);
            }
        } else {
            throw new ClientTestException("Cannot check message for screen which is not an alert");
        }
    }

    public void createAccount(String accountName, String emailAddress, String displayName) throws Throwable {
        Date now = new Date();
        Store store = StoreFactory.getStore();
        store.init(false);
        AccountFolder af = new AccountFolder(Folder.ROOT_FOLDER_PATH + accountName, now, store, displayName, emailAddress);
        MessageManager.getInstance().addFolder(af);
    }

    public void resetDir(String dir) throws Throwable {
        FileAdapter fa = new FileAdapter(dir);
        Enumeration files = fa.list(false);
        while(files.hasMoreElements()) {
            String file = (String)files.nextElement();
            file = dir + "/" + file;
            Log.debug(TAG_LOG, "Removing file: " + file);
            FileAdapter f = new FileAdapter(file);
            f.delete();
        }
    }

    public void createFile(String dir, String fileName, byte[] content) throws IOException {
        Log.debug(TAG_LOG, "Creating file: " + dir + "/" + fileName);
        FileAdapter targetFile = new FileAdapter(dir + "/" + fileName);
        OutputStream os = targetFile.openOutputStream();
        os.write(content);
        os.close();
        targetFile.close();
    }

    public byte[] readFile(String dir, String fileName) throws IOException {
        InputStream is = getClass().getResourceAsStream(dir + "/" + fileName);
        StringBuffer result = new StringBuffer();
        byte[] buffer = new byte[4096];
        int length = 0;
        do {
            length = is.read(buffer);
            if (length > 0) {
                result.append(buffer);
            }
        } while(length > 0);

        return result.toString().getBytes();
    }

    public void deleteFile(String dir, String fileName) throws IOException {
        FileAdapter targetFile = new FileAdapter(dir + "/" + fileName);
        if (targetFile.exists()) {
            targetFile.delete();
        }
    }

    public void addMessageAttachment(String dir, String file, int size) {
        ComposeMessageScreen screen = UIController.getComposeMessageScreen();
        Message m = screen.getMessage();
        if (m.getTextContent() == null) {
            m.setTextBody("");
        }
        AttachFileListener afl = new AttachFileListener(screen, m);
        afl.fileSelected(dir.substring(8), file, "text/plain", size, dir);
        afl.attachFile();
    }

    public void copyFileToDir(String fileDir, String fileName, String dir) throws Throwable {

        InputStream is = getClass().getResourceAsStream(fileDir + "/" + fileName);
        FileAdapter targetFile = new FileAdapter(dir + "/" + fileName);
        OutputStream os = targetFile.openOutputStream();
        byte buf[] = new byte[4096];
        int len = is.read(buf);
        while(len > 0) {
            os.write(buf);
            len = is.read(buf);
        }
        os.close();
        targetFile.close();
        is.close();
    }

    public void createFolder(String accountName, String path, String role) throws Throwable {
        Date now = new Date();
        Store store = StoreFactory.getStore();
        store.init(false);
        String fullName = Folder.FOLDER_SEPARATOR + accountName + Folder.FOLDER_SEPARATOR + path;
        Folder folder = new Folder(fullName, role, now, store);
        MessageManager.getInstance().addFolder(folder);
    }

    public void createMessage(String account, String folder, String from,
                              String subject, String body, boolean read,
                              int count) throws Throwable
    {
        String folderName = Folder.ROOT_FOLDER_PATH + account + Folder.FOLDER_SEPARATOR + folder;
        Folder f = MessageManager.getInstance().getFolder(folderName, true);
        for(int i=0;i<count;++i) {
            Address fromAddress = new Address(Address.FROM, from);
            Message msg = new Message();
            msg.setFrom(fromAddress);
            msg.setSubject(subject);
            msg.setContent(body);
            MessageFlags messageFlags = new MessageFlags();
            messageFlags.setFlag(MessageFlags.OPENED, read);
            msg.setFlags(messageFlags);
            f.appendMessage(msg);
            msg.setKey(MessageManager.makeLuid(msg));
        }
    }


    private void performCommandAction(CommandListener screen, Command command, Displayable display) {
        Log.debug(TAG_LOG, "command: " + command.getLabel());
        screen.commandAction(command, display);
        waitDelay();
    }
    
    private void performCommandAction(CommandListener screen, Command command) {
        Log.debug(TAG_LOG, "command: " + command.getLabel());
        screen.commandAction(command, (Displayable) screen);
        waitDelay();
    }

    private void performCommandAction(CommandListener screen, Command command, int delay) {
        Log.debug(TAG_LOG, "command: " + command.getLabel());
        screen.commandAction(command, (Displayable) screen);
        waitGivenDelay(delay);
    }

    private void waitDelay() {
        //Wait some secs to shows the commands action result on the screen
        try {
            Thread.sleep(COMMAND_ACTION_DELAY);
        } catch(Exception ex) { 
            Log.error(TAG_LOG, "Exception Occurred: " + ex);
        }
    }

    private void waitGivenDelay(int delay) {
        //Wait some secs to shows the commands action result on the screen
        try {
            Thread.sleep(delay);
        } catch(Exception ex) { 
            Log.error(TAG_LOG, "Exception Occurred: " + ex);
        }
    }


    private TestableScreen waitForTestableScreen(String screenName, int milliseconds) throws Exception {
        Log.debug(TAG_LOG, "screen: " + screenName);

        // If the app has not started yet, then we shall wait for the
        // UIContoller to start
        Display display = UIController.display;
        while(display == null) {
            Thread.sleep(WAIT_SCREEN_DELAY);
            milliseconds -= WAIT_SCREEN_DELAY;
            if(milliseconds < 0) {
                Log.error(TAG_LOG, "display is null in UIController");
                throw new ClientTestException("waitForScreen timeout " +
                        "for screen: " + screenName + " \nwas " + UIController.display.getCurrent().getClass());
            }
            display = UIController.display;
            Log.debug(TAG_LOG, "display=" + display);
        }

        Displayable current = UIController.display.getCurrent();
        String fullName = current.getClass().getName();
        String components[] = StringUtil.split(fullName, ".");
        String name = components[components.length - 1];
 
        while (!name.equals(screenName))
        {
            Log.debug(TAG_LOG, "Waiting for " + screenName + " " + milliseconds + " ...");
            Thread.sleep(WAIT_SCREEN_DELAY);
            milliseconds -= WAIT_SCREEN_DELAY;
            if(milliseconds < 0) {
                throw new ClientTestException("waitForScreen timeout " +
                        "for screen: " + screenName + " \nwas " + UIController.display.getCurrent().getClass());
            }
            current = UIController.display.getCurrent();

            fullName = current.getClass().getName();
            components = StringUtil.split(fullName, ".");
            name = components[components.length - 1];

            Log.info(TAG_LOG, "Current screen name=" + name);
        }
        if (current instanceof TestableScreen) {
            return (TestableScreen)current;
        } else {
            Log.error(TAG_LOG, "Waited for a non testable screen");
            throw new ClientTestException("waited for non testable screen " + screenName);
        }
    }
}
