/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient;

import com.funambol.mail.BodyPart;
import java.util.Hashtable;
import java.util.Vector;
import java.io.IOException;
import java.io.InputStream;

import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.view.InboxMessageList;
import com.funambol.mailclient.ui.view.OutboxMessageList;
import com.funambol.mailclient.ui.view.DraftMessageList;
import com.funambol.mailclient.ui.view.SentMessageList;
import com.funambol.storage.AbstractRecordStore;
import com.funambol.mailclient.FunambolTest;

import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.spds.SyncManager;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.syncml.spds.SyncListener;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.protocol.SyncFilter;
import com.funambol.syncml.protocol.SyncML;

import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.util.CodedException;
import com.funambol.util.HttpTransportAgent;
import com.funambol.platform.FileAdapter;

/**
 */
public class ScriptRunner extends TestCase {

    private static final String SCRIPT_URL_PROPERTY = "ScriptUrl";

    private static final String TEST_NAME = "Script Runner";
    private static final String TAG_LOG   = "ScriptRunner";

    private static final String CLICK_MENU_ITEM_COMMAND  = "ClickMenuItem";
    private static final String WAIT_FOR_SCREEN_COMMAND  = "WaitForScreen";
    private static final String SET_TEXT_FIELD_COMMAND   = "SetInputField";
    private static final String CHECK_LIST_COUNT_COMMAND = "CheckListCount";
    private static final String CHECK_ACTIVE_CONTACT_PROP_COMMAND = "CheckActiveContactProperty";
    private static final String WAIT_COMMAND = "Wait";
    private static final String BEGIN_TEST_COMMAND = "BeginTest";
    private static final String END_TEST_COMMAND = "EndTest";
    private static final String RESET_CONFIG_COMMAND = "ResetConfig";
    private static final String START_MAIN_APP_COMMAND = "StartMainApp";
    private static final String RESET_REMOTE_INBOX_COMMAND = "ResetRemoteInbox";
    private static final String KEY_SINGLE_PRESS_COMMAND = "KeySinglePress";
    private static final String CREATE_ACCOUNT_COMMAND = "CreateAccount";
    private static final String CREATE_FOLDER_COMMAND = "CreateFolder";
    private static final String CREATE_MESSAGE_COMMAND = "CreateMessage";
    private static final String WAIT_FOR_SYNC_TO_COMPLETE_COMMAND = "WaitForSyncToComplete";
    private static final String INCLUDE_COMMAND = "Include";
    private static final String CHECK_ACTIVE_MESSAGE_PROP_COMMAND = "CheckActiveMessageProperty";
    private static final String SAVE_ACCOUNT_COMMAND = "SaveAccount";
    private static final String CHECK_ALERT_TITLE_COMMAND = "CheckAlertTitle";
    private static final String CHECK_ALERT_MESSAGE_COMMAND = "CheckAlertMessage";
    private static final String RESET_DIR_COMMAND = "ResetDir";
    private static final String COPY_FILE_TO_DIR_COMMAND = "CopyFileToDir";
    private static final String ADD_MESSAGE_ATTACHMENT = "AddMessageAttachment";
    private static final String VERIFY_MESSAGE_ATTACHMENT = "VerifyMessageAttachment";

    public static final String ARROW_DOWN_KEY_NAME = "ArrowDown";
    public static final String ARROW_UP_KEY_NAME = "ArrowUp";
    public static final String ARROW_LEFT_KEY_NAME = "ArrowLeft";
    public static final String ARROW_RIGHT_KEY_NAME = "ArrowRight";
    public static final String FIRE_KEY_NAME = "Fire";
    public static final String KEY_CHAR_KEY_NAME = "KeyChar";

    private static final String PICTURES_DIR = "Pictures";
    private static final String PHOTO_0 = "Photo0.jpg";

    private String baseUrl = null;


    public ScriptRunner() {
        super(TEST_NAME);
    }

    public void test() throws Throwable {
        String scriptUrl = FunambolTest.getProperty(SCRIPT_URL_PROPERTY);
        baseUrl = getBaseUrl(scriptUrl);
        runScriptFile(scriptUrl, true);
    }

    private String getBaseUrl(String scriptUrl) {
        int pos = scriptUrl.indexOf('/');
        int lastPos = pos;

        while(pos != -1) {
            lastPos = pos;
            pos = scriptUrl.indexOf('/', pos + 1);
        }

        if (lastPos != -1) {
            return scriptUrl.substring(0, lastPos);
        } else {
            return null;
        }
    }


    private void runScriptFile(String scriptUrl, boolean mainScript) throws Throwable {

        int lineNumber = 0;
        String script = null;

        if (mainScript) {
            FunambolTest.updateSplashScreen("Loading remote script");
        }

        Log.info(TAG_LOG, "Running script at URL = " + scriptUrl);
        if (scriptUrl != null) {
            try {
                if (scriptUrl.startsWith("http")) {
                    script = getFileViaHttp(scriptUrl);
                } else if (scriptUrl.startsWith("file")) {
                    script = getScriptViaFile(scriptUrl);
                } else {
                    Log.error(TAG_LOG, "Unknwon protocol to fetch script " + scriptUrl);
                    throw new IllegalArgumentException("Cannot fetch script");
                }
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot load script at " + scriptUrl + " because " + e.toString());
                throw new ClientTestException("Cannot load script " + scriptUrl
                                              + " because " + e.toString());
            }
        }

        if (mainScript) {
            FunambolTest.updateSplashScreen("Starting script execution");
        }

        // Execute the script by interpreting it
        try {
            int idx = 0;
            int nextLine = -1;
            do {
                nextLine = script.indexOf('\n', idx);
                String line;
                if (nextLine != -1) {
                    line = script.substring(idx, nextLine);
                    idx = nextLine + 1;
                } else {
                    line = script.substring(idx);
                }
                lineNumber++;

                line = line.trim();
                if (line.length() > 0 && !line.startsWith("#")) {
                    int parPos = line.indexOf('(');
                    if (parPos == -1) {
                        Log.error(TAG_LOG, "Syntax error in script, missing '(' in: "
                                + line + " at line " + lineNumber);
                        return;
                    }
                    String command = line.substring(0, parPos);
                    Log.debug(TAG_LOG, "command=" + command);
                    String pars = line.substring(parPos);
                    Log.debug(TAG_LOG, "pars=" + pars);
                    if (CLICK_MENU_ITEM_COMMAND.equals(command)) {
                        // Expect one parameter
                        clickMenuItem(command, pars);
                    } else if (WAIT_FOR_SCREEN_COMMAND.equals(command)) {
                        waitForScreen(command, pars);
                    } else if (SET_TEXT_FIELD_COMMAND.equals(command)) {
                        setTextField(command, pars);
                    } else if (CHECK_LIST_COUNT_COMMAND.equals(command)) {
                        checkListCount(command, pars);
                    } else if (CHECK_ACTIVE_CONTACT_PROP_COMMAND.equals(command)) {
                        checkActiveContactProperty(command, pars);
                    } else if (CHECK_ACTIVE_MESSAGE_PROP_COMMAND.equals(command)) {
                        checkActiveMessageProperty(command, pars);
                    } else if (WAIT_COMMAND.equals(command)) {
                        wait(command, pars);
                    } else if (BEGIN_TEST_COMMAND.equals(command)) {
                        beginTest(command, pars);
                    } else if (END_TEST_COMMAND.equals(command)) {
                        endTest(command, pars);
                    } else if (RESET_CONFIG_COMMAND.equals(command)) {
                        resetConfig(command, pars);
                    } else if (START_MAIN_APP_COMMAND.equals(command)) {
                        startMainApp(command, pars);
                    } else if (RESET_REMOTE_INBOX_COMMAND.equals(command)) {
                        resetServerInbox(command, pars);
                    } else if (KEY_SINGLE_PRESS_COMMAND.equals(command)) {
                        keySinglePress(command, pars);
                    } else if (CREATE_ACCOUNT_COMMAND.equals(command)) {
                        createAccount(command, pars);
                    } else if (CREATE_FOLDER_COMMAND.equals(command)) {
                        createFolder(command, pars);
                    } else if (CREATE_MESSAGE_COMMAND.equals(command)) {
                        createMessage(command, pars);
                    } else if (CHECK_ALERT_TITLE_COMMAND.equals(command)) {
                        checkAlertTitle(command, pars);
                    } else if (CHECK_ALERT_MESSAGE_COMMAND.equals(command)) {
                        checkAlertMessage(command, pars);
                    } else if (WAIT_FOR_SYNC_TO_COMPLETE_COMMAND.equals(command)) {
                        waitForSyncToComplete(command, pars);
                    } else if (INCLUDE_COMMAND.equals(command)) {
                        includeScript(command, pars);
                    } else if (SAVE_ACCOUNT_COMMAND.equals(command)) {
                        saveAccount(command, pars);
                    } else if (RESET_DIR_COMMAND.equals(command)) {
                        resetDir(command, pars);
                    } else if (COPY_FILE_TO_DIR_COMMAND.equals(command)) {
                        copyFileToDir(command, pars);
                    } else if (ADD_MESSAGE_ATTACHMENT.equals(command)) {
                        addMessageAttachment(command, pars);
                    } else if (VERIFY_MESSAGE_ATTACHMENT.equals(command)) {
                        verifyMessageAttachment(command, pars);
                    } else {
                        Log.error(TAG_LOG, "Syntax error: unknown command " + command);
                    }
                }
            } while (nextLine != -1);
        } catch (Throwable t) {
            Log.error(TAG_LOG, "Script " + scriptUrl + " interrupted at line: " + lineNumber);
            throw t;
        }
    }

    public void tearDown() {
    }

    public void setUp() {
    }

    private String getParameter(String allPars, int index) {
        // Remove the paranthesis if necessary
        if (allPars.startsWith("(")) {
            allPars = allPars.substring(1);
        }
        if (allPars.endsWith(")")) {
            allPars = allPars.substring(0, allPars.length() - 1);
        }
        String args[] = StringUtil.split(allPars, ",");

        if (index < args.length) {
            String value = args[index];
            value=value.trim();
            if (value.startsWith("\"")) {
                value = value.substring(1);
            }
            if (value.endsWith("\"")) {
                value = value.substring(0, value.length() - 1);
            }
            return value;
        } else {
            return null;
        }
    }

    private void clickMenuItem(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String label = getParameter(args, 0);
        if (label == null) {
            Log.error(TAG_LOG, "Syntax error in script, missing label in command "
                               + CLICK_MENU_ITEM_COMMAND);
        } else {
            robot.simulateCommandAction(label);
        }
    }

    private void waitForScreen(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String screenName = getParameter(args, 0);
        String delay      = getParameter(args, 1);

        if (screenName == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find screen name for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (delay == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find delay " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        int d = Integer.parseInt(delay);
        robot.waitForScreen(screenName, d);
    }

    private void waitForSyncToComplete(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String minStart = getParameter(args, 0);
        String maxWait  = getParameter(args, 1);

        if (minStart == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find min wait time " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (maxWait == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find max wait time " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        int min = Integer.parseInt(minStart);
        int max = Integer.parseInt(maxWait);
        robot.waitForSyncToComplete(min, max);
    }

    private void saveAccount(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String url = getParameter(args, 0);
        String user  = getParameter(args, 1);
        String pwd  = getParameter(args, 2);

        if (url == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find url " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (user == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find user " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (pwd == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find pwd " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        robot.saveAccount(url, user, pwd);
    }

    private void includeScript(String command, String args) throws Throwable {
        String scriptUrl = getParameter(args, 0);

        if (scriptUrl == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find script url " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }
        if (!scriptUrl.startsWith("http") && !scriptUrl.startsWith("file")) {
            if (baseUrl != null) {
                scriptUrl = baseUrl + "/" + scriptUrl;
            }
        }
        runScriptFile(scriptUrl, false);
    }

    private void setTextField(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String fieldName = getParameter(args, 0);
        String value     = getParameter(args, 1);

        if (fieldName == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find field name for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (value == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find value " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        robot.setTextField(fieldName, value);
    }

    private void checkListCount(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String listName = getParameter(args, 0);
        String count    = getParameter(args, 1);

        if (listName == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find list name for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (count == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find count " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        int c = Integer.parseInt(count);
        robot.checkListCount(listName, c);
    }

    private void checkAlertTitle(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String title = getParameter(args, 0);

        if (title == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find title value for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        robot.checkAlertTitle(title);
    }

    private void checkAlertMessage(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String message = getParameter(args, 0);

        if (message == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find message value for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        robot.checkAlertMessage(message);
    }

    private void checkActiveContactProperty(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String fieldName = getParameter(args, 0);
        String value     = getParameter(args, 1);

        if (fieldName == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find field name for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (value == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find value " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        robot.checkActiveContactProperty(fieldName, value);
    }

    private void checkActiveMessageProperty(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String fieldName = getParameter(args, 0);
        String value     = getParameter(args, 1);

        if (fieldName == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find field name for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (value == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find value " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        robot.checkActiveMessageProperty(fieldName, value);
    }


    private void wait(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String delay = getParameter(args, 0);
        if (delay == null) {
            Log.error(TAG_LOG, "Syntax error in script, missing label in command "
                               + CLICK_MENU_ITEM_COMMAND);
        } else {
            int d = Integer.parseInt(delay);
            try {
                if (d > 0) {
                    Thread.sleep(d);
                } else {
                    // Sleep forever
                    while(true) {
                        Thread.sleep(1000000);
                    }
                }
            } catch (InterruptedException ie) {
            }
        }
    }

    private void beginTest(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String testName = getParameter(args, 0);
        if (testName == null) {
            Log.error(TAG_LOG, "Syntax error in script, missing test name in begin "
                               + BEGIN_TEST_COMMAND);
        } else {
            Log.info(TAG_LOG, "Starting test " + testName);
        }
    }

    private void endTest(String command, String args) throws Throwable {
    }

    private void resetConfig(String command, String args) throws Throwable {
        // No parameter for this command

        // Remove all the record stores
        String[] stores = AbstractRecordStore.listRecordStores();
        if(stores != null) {
            for(int i=0; i<stores.length; i++) {
                try {
                    AbstractRecordStore.deleteRecordStore(stores[i]);
                } catch(Exception ex) {
                    Log.error("Error while removing record store: " + stores[i] +
                              " Exception: " + ex);
                }
            }
        }
    }

    private void startMainApp(String command, String args) throws Throwable {
        // No parameter for this command
        FunambolTest.startMainApp();
    }

    private void keySinglePress(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String keyName = getParameter(args, 0);
        // This is an optional parameter
        String keyChar = getParameter(args, 1);
        if (keyName == null) {
            Log.error(TAG_LOG, "Syntax error in script, missing key name in "
                               + KEY_SINGLE_PRESS_COMMAND);
        } else {
            Log.info(TAG_LOG, "Simluating key single press " + keyName);
        }
        robot.keySinglePress(keyName, keyChar);
    }

    private void resetDir(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String dir = getParameter(args, 0);

        if (dir == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find sync dir for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (PICTURES_DIR.equals(dir)) {
            dir = System.getProperty("fileconn.dir.photos");
            robot.resetDir(dir);
        } else {
            throw new IllegalArgumentException("Invalid directory name");
        }
    }

    private void addMessageAttachment(String command, String args) {
        Log.debug(TAG_LOG, "Adding attachment to the message");
        Robot robot = Robot.getInstance();
        String file = getParameter(args, 0);
        String pattern = getParameter(args, 1);
        int repeat = Integer.parseInt(getParameter(args, 2));

        //Create file content
        //Repeat "repeat" times the pattern content
        Log.debug(TAG_LOG, "Creating file content: "  + pattern + " repeated for " + repeat + " times");
        StringBuffer fileContent = new StringBuffer();
        for (int i = 0; i < repeat; i++) {
            fileContent.append(pattern);
        }

        String dir = System.getProperty("fileconn.dir.photos");
        try {
            robot.createFile(dir, file, fileContent.toString().getBytes());
        } catch (IOException ex) {
            throw new IllegalArgumentException("Cannot create attachment");
        }

        Log.debug("Adding created file as attachment to the current message");
        robot.addMessageAttachment(dir, file, fileContent.length());

    }

    public void verifyMessageAttachment(String command, String args) throws ClientTestException {

        Log.debug(TAG_LOG, "Parsing input param");
        int position = Integer.parseInt(getParameter(args, 0));
        String file = getParameter(args, 1);
        String pattern = getParameter(args, 2);
        int repeat = Integer.parseInt(getParameter(args, 3));

        Log.debug(TAG_LOG, "Building file content");
        StringBuffer expectedFileContent = new StringBuffer();
        for (int i = 0; i < repeat; i++) {
            expectedFileContent.append(pattern);
        }

        String msg = "Cannot read original file";

        msg = "Attachment and original file contents don't match";

        Vector attach = UiUtils.getAttachments(UIController.getSentMessageList().getMessageAt(position));
        
        boolean equals = true;

        for (int i=0; i < attach.size(); i++) {
            Log.debug(TAG_LOG, "Parsing msg attachments");
            BodyPart bp = (BodyPart) attach.elementAt(i);

            Log.debug(TAG_LOG, "Attachment found: " + bp.getFileName());
            if (!bp.getFileName().equals(file)) {
                Log.error(TAG_LOG, bp.getFileName());
                equals = false;
            }
        }

        if (!equals) {
            throw new ClientTestException(msg);
        }
    }

    private void copyFileToDir(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String file = getParameter(args, 0);
        String dir = getParameter(args, 1);

        if (dir == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find dir for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (file == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find file for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (PICTURES_DIR.equals(dir)) {
            dir = System.getProperty("fileconn.dir.photos");

            String sourceDir = null;

            if (PHOTO_0.equals(file)) {
                sourceDir = "/res";
                file = "Photo0.jpg";
            } else {
                throw new IllegalArgumentException("Syntax error in script");
            }

            robot.copyFileToDir(sourceDir, file, dir);
        } else {
            throw new IllegalArgumentException("Invalid directory name");
        }
    }

    private void resetServerInbox(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String syncUrl = getParameter(args, 0);
        String user    = getParameter(args, 1);
        String pwd     = getParameter(args, 2);

        if (syncUrl == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find sync url for " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (user == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find user " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (pwd == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find pwd " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        // We use a special SyncSource for this purpose
        ResetInboxSyncSource riss = new ResetInboxSyncSource();
        try {
            // We perform a first sync to get all the keys of the emails
            SyncConfig config = new SyncConfig();
            config.syncUrl = syncUrl;
            config.userName = user;
            config.password = pwd;
            // Set the config
            SyncManager manager = new SyncManager(config);
            manager.sync(riss);
            // Now sync again to propagate all the deletes
            manager.sync(riss);
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot reset server inbox");
            throw e;
        }
    }

    private void createAccount(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String accountName = getParameter(args, 0);
        String emailAddress = getParameter(args, 1);
        String displayName  = getParameter(args, 2);
        if (accountName == null) {
            String msg = "Syntax error in script, missing account name in "
                               + CREATE_ACCOUNT_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        if (emailAddress == null) {
            String msg = "Syntax error in script, missing email address in "
                               + CREATE_ACCOUNT_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        if (displayName == null) {
            String msg = "Syntax error in script, missing display name in "
                               + CREATE_ACCOUNT_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        Log.debug(TAG_LOG, "Creating account " + accountName + "," + emailAddress + "," + displayName);
        robot.createAccount(accountName, emailAddress, displayName);
    }

    private void createFolder(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String accountName = getParameter(args, 0);
        String path = getParameter(args, 1);
        String role = getParameter(args, 2);
        if (accountName == null) {
            String msg = "Syntax error in script, missing account name in "
                               + CREATE_FOLDER_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        if (path == null) {
            String msg = "Syntax error in script, missing folder path in "
                               + CREATE_FOLDER_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        if (role == null) {
            String msg = "Syntax error in script, missing display name in "
                               + CREATE_FOLDER_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        Log.debug(TAG_LOG, "Creating folder " + accountName + "," + path + "," + role);
        robot.createFolder(accountName, path, role);
    }

    private void createMessage(String command, String args) throws Throwable {
        Robot robot = Robot.getInstance();
        String account = getParameter(args, 0);
        String folder  = getParameter(args, 1);
        String from    = getParameter(args, 2);
        String subject = getParameter(args, 3);
        String body    = getParameter(args, 4);
        String read    = getParameter(args, 5);
        String count   = getParameter(args, 6);
        int    c;

        if (account == null) {
            String msg = "Syntax error in script, missing account in "
                               + CREATE_MESSAGE_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }

        if (folder == null) {
            String msg = "Syntax error in script, missing folder in "
                               + CREATE_MESSAGE_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }

        if (from == null) {
            String msg = "Syntax error in script, missing from in "
                               + CREATE_MESSAGE_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        if (subject == null) {
            String msg = "Syntax error in script, missing subject in "
                               + CREATE_MESSAGE_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        if (body == null) {
            String msg = "Syntax error in script, missing body in "
                               + CREATE_MESSAGE_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        }
        if (count == null) {
            c = 1;
        } else {
            try {
                c = Integer.parseInt(count);
            } catch (Exception e) {
                throw new ClientTestException("Syntax error, invalid repeatition counter " + count);
            }
        }
        boolean readFlag;
        if (read == null) {
            String msg = "Syntax error in script, missing read flag in "
                               + CREATE_MESSAGE_COMMAND;
            Log.error(TAG_LOG, msg);
            throw new ClientTestException(msg);
        } else {
            if (StringUtil.equalsIgnoreCase("true", read)) {
                readFlag = true;
            } else {
                readFlag = false;
            }
        }

        Log.debug(TAG_LOG, "Creating message " + from + "," + subject + "," + body + "," + read);
        robot.createMessage(account, folder, from, subject, body, readFlag, c);
    }



    private String getFileViaHttp(String url) throws CodedException {
        HttpTransportAgent ta = new HttpTransportAgent(url, false, false);
        String response = ta.sendMessage("");
        return response;
    }

    private String getScriptViaFile(String url) throws IOException {
        FileAdapter fa = new FileAdapter(url, true);
        int size = (int)fa.getSize();
        byte data[] = new byte[size];
        InputStream is = fa.openInputStream();
        is.read(data);
        is.close();
        fa.close();
        return new String(data);
    }

    private class ResetInboxSyncSource implements SyncSource {

        private Vector items = new Vector();
        private long   lastAnchor = 0;
        private long   nextAnchor = 0;
        private int    delItemIdx = 0;
        private SourceConfig config;

        public ResetInboxSyncSource() {
        }

        public String getName() {
            return "ResetInboxSyncSource";
        }

        public String getSourceUri() {
            return "mail";
        }

        public String getType() {
            return "application/vnd.omads-email+xml";
        }

        public String getEncoding() {
            return SyncSource.ENCODING_NONE;
        }

        public int getSyncMode() {
            return SyncML.ALERT_CODE_FAST;
        }

        public SyncFilter getFilter() {
            return null;
        }

        public void setFilter(SyncFilter filter) {
        }

        public int addItem(SyncItem item) throws SyncException {

            // Filter out all folders and accouts. This is no very clean
            String key = item.getKey().toLowerCase();
            if (!key.equals("root/i") && !key.equals("root/o") &&
                !key.endsWith("inbox") && !key.endsWith("outbox")) {

                items.addElement(item.getKey());
            }
            return SyncSource.STATUS_SUCCESS;
        }
    
        public int updateItem(SyncItem item) throws SyncException {
            items.addElement(item.getKey());
            return SyncSource.STATUS_SUCCESS;
        }
    
        public int deleteItem(String key) throws SyncException {
            return SyncSource.STATUS_SUCCESS;
        }
    
        public SyncItem getNextItem() throws SyncException {
            return null;
        }
    
        public SyncItem getNextNewItem() throws SyncException {
            return null;
        }
    
        public SyncItem getNextUpdatedItem() throws SyncException {
            return null;
        }
    
        public SyncItem getNextDeletedItem() throws SyncException {
            if (delItemIdx < items.size()) {
                String key = (String) items.elementAt(delItemIdx++);
                SyncItem item = new SyncItem(key);
                return item;
            }
            return null;
        }
    
        public void setItemStatus(String key, int status) throws SyncException {
        }

        public void dataReceived(String date, int size) {
        }
    
        public int getClientItemsNumber() {
            return 0;
        }

        public int getClientAddNumber() {
            return 0;
        }

        public int getClientReplaceNumber() {
            return 0;
        }

        public int getClientDeleteNumber() {
            return items.size();
        }

        public int getServerItemsNumber() {
            return -1;
        }

        public void setServerItemsNumber(int number) {
        }

        public long getLastAnchor() {
            return lastAnchor;
        }
    
        public void setLastAnchor(long time) {
            lastAnchor = time;
        }
    
        public long getNextAnchor() {
            return nextAnchor;
        }
    
        public void setNextAnchor(long time) {
            nextAnchor = time;
        }

        public void beginSync(int syncMode) throws SyncException {
        }
    
        public void endSync() throws SyncException {
        }

        public void setListener(SyncListener listener) {
        }

        public SyncListener getListener() {
            return null;
        }

        public int getStatus() {
            return SyncSource.STATUS_SUCCESS;
        }

        public SourceConfig getConfig() {
            return config;
        }

        public void setConfig(SourceConfig config) {
            this.config = config;
        }

        public SyncItem createSyncItem(String key, String type, char state, String parent, long size)
        throws SyncException {
            return new SyncItem(key, type, state, parent);
        }
    }
}
 
