/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.view.InboxMessageList;
import com.funambol.mailclient.ui.controller.PopupAction;
import com.funambol.util.Log;
import com.funambol.util.LogContent;
import com.funambol.util.MailDateFormatter;
import java.util.Date;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.midlet.MIDlet;

/**
 * TestReport is a List which allows to show the test result
 */
public class TestReport extends List implements CommandListener {

    private static final String CONTENT_SEPARATOR = "#######################################\n";
    private static final String REPORT_TITLE      = "# Automatic test suite report results #\n";
    private static final String LOG_TITLE         = "# Client Log after the test suite run #\n";
    private static final String SEND_LOG_PROPERTY = "SendReportAddress";
    
    private MIDlet midlet;
    private Command exit = new Command("Exit", Command.EXIT, 1);
    private Command sendResults = new Command("Send", Command.OK, 2);
    
    private Message m = null;
    private String sendReportAddress = null;
    
    public TestReport (String title, String[] results, MIDlet midlet) {
        super(title, List.IMPLICIT, results, null);
        this.setFitPolicy(TEXT_WRAP_ON);
        this.midlet = midlet;

        sendReportAddress = FunambolTest.getProperty(SEND_LOG_PROPERTY);

        if (sendReportAddress != null) {
            createReportMessage();
            UIController.sendMessage(m);
            try {
                Thread.sleep(30000);
                UIController.midlet.exit();
            } catch (Throwable t) {
            }
        } else {
            addCommand(exit);
            addCommand(sendResults);
            setCommandListener(this);
        }
        // For convenience we always dump the report to console
        System.out.println(createContent());
    }
    
    /**
     * Called by the system to indicate that a command has been invoked
     * on a particular displayable.
     * @param command the Command that was invoked
     * @param displayable the Displayable on which the command was invoked
     */
    public void commandAction(Command command, Displayable displayable) {
        Log.debug("commandAction: " + command);
        if (command == exit) {
            Log.debug("exit: " + command);
            try {
                UIController.midlet.exit();
            } catch(Exception ex) { 
            }
        } else if (command==sendResults) {
            createReportMessage();
        }
    }

    private String createContent() {
        StringBuffer sb = new StringBuffer();

        String dev = System.getProperty("microedition.platform");
        if (dev == null) {
            dev = "Unknwon device";
        }
        sb.append("Tests run on: " + dev).append("\n");
        sb.append("Client info: " + ConfigManager.appInfoAttr).append("\n");
        sb.append("Test date: " + MailDateFormatter.dateToRfc2822(new Date())).append("\n");

        sb.append(CONTENT_SEPARATOR);
        sb.append(REPORT_TITLE);
        sb.append(CONTENT_SEPARATOR);
        sb.append("\n");
        for (int i = 0; i<this.size(); i++) {
            sb.append(this.getString(i) + "\n");
        }
        sb.append("\n");
        sb.append(CONTENT_SEPARATOR);
        sb.append(LOG_TITLE);
        sb.append(CONTENT_SEPARATOR);
        sb.append("\n");

        try {
            LogContent logContent = Log.getCurrentLogContent();
            if (logContent.getContentType() == LogContent.STRING_CONTENT) {
                sb.append(logContent.getContent());
            } else {
                sb.append("Log in files not supported");
            }
        } catch (Exception e) {
            sb.append("Cannot get log content");
        }
        return sb.toString();
    }

    private void createReportMessage() {
        m = new Message();
        m.setFrom(UIController.getMyFrom());
        m.setSubject(createSubject());

        try {
            String content = createContent();
            m.setContent(content);
            m.setTo(new Address[]{new Address(Address.TO, sendReportAddress)});
            // Dump the content to stdout for automatic test inspection
            System.out.println(content);
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error(this, "Mail exception creating address for support email");
        }
    }
    
    public class SendResultsPopupAction implements PopupAction {
        public void cancel() {
            UIController.showBackScreen();
        }
        
        public void confirm() {
            UIController.sendMessage(m);
        }
    }
    
    private String createSubject() {
        // TODO add test/failure info in the subject
        return "JavaME Email Client auto test: ";
    }
}
